function createsymbolics()
clc; clear all;
% @JR, 04.05.16

%% Model name:
ModelName = 'NorwegianClay-Average';
%materialGeneration(ModelName)

%% Material parameters, etc.
% Stiffness
K = sym('K', 'real');
G = sym('G', 'real');

% Intrinsic
K0nc = sym('K0nc', 'real');
rsi  = sym('rsi', 'real');
kp   = sym('kp', 'real');
pmi  = sym('pmi', 'real');
pmi0 = sym('pmi0', 'real');

% Initial state
OCR = sym('OCR', 'real');

% Destructuration
kx    = sym('kx', 'real');
w     = sym('w', 'real');
chi   = sym('chi', 'real');
chi0  = sym('chi0', 'real');
chi00 = sym('chi00', 'real');

% Rotation of surfaces
ca       = sym('ca', 'real');
bk0ncntx = sym('bk0ncntx', 'real');

% Reference values
tau    = sym('tau', 'real');
OCRmax = sym('OCRmax', 'real');

%% Include symbolics
commonsyms;
isotropicElastic;
rotationMatrix;

D = p*D;

%% Failure criterion
phi        = sym('phi', 'real');
phics      = sym('phics', 'real');
a1         = sym('a1', 'real');
a2         = sym('a2', 'real');
phi2       = phi*pi/180;
phics2     = phics*pi/180;

% Generalized Mohr-Coulomb
ctetaa = cos(1/3*asin(a1*(sin3tetaa)));
stetaa = sin(1/3*asin(a1*(sin3tetaa)));
ctetab = cos(1/3*asin(a1*(sin3tetab)));
stetab = sin(1/3*asin(a1*(sin3tetab)));

cteta0a = cos(1/3*asin(a1*(sin3teta0a)));
steta0a = sin(1/3*asin(a1*(sin3teta0a)));
cteta0b = cos(1/3*asin(a1*(sin3teta0b)));
steta0b = sin(1/3*asin(a1*(sin3teta0b)));

M = 3*a2*sin(phi2)  /(sqrt(3)*a2*ctetaa+stetaa*sin(phi2));
N = 3*a2*sin(phics2)/(sqrt(3)*a2*ctetab+stetab*sin(phics2));

M0 = 3*a2*sin(phics2)/(sqrt(3)*a2*cteta0a+steta0a*sin(phics2));
N0 = 3*a2*sin(phi2)  /(sqrt(3)*a2*cteta0b+steta0b*sin(phi2));

% Lade
sinphips   = sqrt((9-7*sin(phi2))/(3-sin(phi2)))*2*sin(phi2)/(3-sin(phi2));
sinphipscs = subs(sinphips, phi, phics);

ML = 3*sinphipscs/2/cos(1/3*acos(sinphipscs*sin3tetaa));
NL = 3*sinphips  /2/cos(1/3*acos(sinphips*sin3tetab));

ML0 = 3*sinphipscs/2/cos(1/3*acos(sinphipscs*sin3teta0a));
NL0 = 3*sinphips  /2/cos(1/3*acos(sinphips*sin3teta0b));

%  Convexity limit
b   = 1/3*asin(a1);
a2  = 1/sqrt(3)*(2*sin(3*b)*cos(b) + sin(2*b))/(2*sin(3*b)*sin(b) + cos(2*b));
NC  = 1*2*sin(phi2)/(1-1/3*sin(phi2))*(sqrt(3)*cos(b) - sin(b)/a2)/(sqrt(3)*ctetab+stetab/a2);
NC0 = 1*2*sin(phi2)/(1-1/3*sin(phi2))*(sqrt(3)*cos(b) - sin(b)/a2)/(sqrt(3)*cteta0b+steta0b/a2);


%% Helper variables
Mtx = subs(ML, [Sig;ad], [0.5;1;0.5;0;0;0;0;0;0;0;0;0]);
Ntx = subs(N,  [Sig;bd], [0.5;1;0.5;0;0;0;0;0;0;0;0;0]);

etak0nc = 3*(1-K0nc)/(1+2*K0nc);
ak0nc   = etak0nc - (Mtx^2 - etak0nc^2)/3;
mk0nc   = (Mtx^2 - ak0nc^2)/(Mtx^2-etak0nc^2);

%% Parameter vectors
% State variables
v  = [Sig;  pmi;  ad;  bd;  lam;  chi ];
v0 = [Sig0; pmi0; ad0; bd0; lam0; chi0];

%% Surfaces
% Reference
B = 3/2*sdb'*sdb;
Bb = 3/2*bdd'*bdd;
peq = p + B/((N0^2-Bb)*p);

% Plastic potential
A = 3/2*sda'*sda;
Aa = 3/2*add'*add;
k1 = (ML0/Mtx)^2;
Q = p + A/((Mtx^2-Aa)*p*k1);

%% Plastic strain increments
depdl = diffvec(Q,Sig);
dempdl = diff(Q,s11) + diff(Q,s22) + diff(Q,s33);
dedpdl = depdl - 1/3*dempdl*I;
dedpdl(4:6) = sqrt(2)*dedpdl(4:6);
deqpdl = sqrt(2/3*(dedpdl'*dedpdl));

%% Elasticity
Fel = D^-1;
depse = Fel*(Sig - Sig0);
depsp = diffvec(Q,Sig)*(lam-lam0);

%% Differentials

% Preconsolidation
km = 1/(1/kp - 1/K);
dpmidl = km*pmi*dempdl;

% Structure
dxdl = -chi*kx*sqrt(dempdl^2 + w^2*deqpdl^2);

% Rotation of surfaces potential
% Target values
atarget = ML0/(3/2)*eta/etav*tanh(atanh(3/2*ak0nc/Mtx)*(etav/ML0)/(etak0nc/Mtx));
btarget =  N0/(3/2)*eta/etav*tanh(atanh(3/2*bk0ncntx) *(etav/ N0)/(etak0nc/Ntx));

% For potential
daddl = (p/(pmi*(1+chi)))*ca*(atarget-ad);

% For reference surface
dbddl = (p/(pmi*(1+chi)))*ca*(btarget-bd);

% Plastic multiplier
dlamdt = 1/(rsi*tau)*mac((peq/((1+chi)*pmi))^(rsi/km) - 1/(OCRmax^(rsi/km)))*mk0nc;

%% Initialization of state variables
bk0nc = bk0ncntx*Ntx;
ad00  = [-1/3*ak0nc; 2/3*ak0nc; -1/3*ak0nc; 0; 0; 0];
bd00  = [-1/3*bk0nc; 2/3*bk0nc; -1/3*bk0nc; 0; 0; 0];
v00   = [Sig;  peq*OCR/(1+chi00); ad00; bd00; 0;  chi00];
v00   = subs(v00, [Sig;ad;bd;ad0;bd0], [Sig0; ad00;bd00;ad00;bd00]);

%% Residual equations
r1 = depse + depsp - dEps;
r2 = pmi-pmi0 - dpmidl*(lam-lam0);
r3 = ad-ad0 - daddl*(lam-lam0);
r4 = bd-bd0 - dbddl*(lam-lam0);
r5 = chi-chi0 - dxdl*(lam-lam0);
r6 = lam-lam0-dlamdt*dt;

% Normalization
p0 = hyd(Sig0);
r2 = r2/p0;

Res = [r1;r2;r3;r4;r5;r6];

% Jacobian matrix
Jacobis = jacobian(Res,v);

% % Trial stress
% sigTrial = Sig0 + D*dEps;


%% Output Fortran subroutines (for PLAXIS)
tic
%Output
folder = ['..\models\', ModelName];
mkdir(folder);
    
disp('Generating Fortran file...');
% %delete('residuals.f', 'jacobians.f', 'sigTrials.f', 'r.m', 'initStateVars.f', 'ElasticMatrix.f');
    
 fortran2(Res,      'File', [folder, '\residuals.f']);
 fortran2(Jacobis,  'File', [folder, '\jacobians.f']);
 fortran2(sigTrial, 'File', [folder, '\sigTrials.f']);
 fortran2(v00,      'File', [folder, '\initStateVars.f']);
 fortran2(D,        'File', [folder, '\ElasticMatrix.f']);
 
 materialGeneration(ModelName, c, v)
disp('Finished')
toc

% %% Output Matlab files (for simple strain driver)
% h = {v,v0,c,dEps};
% % file( INI1, INI2, INI3, INI4) = h(v, v0, c, dEps) where:
% % v    - Update state variables
% % v0   - Old state variables
% % c    - Material parameters
% % dEps - Strain increment
% 
% 
% disp('Generating Matlab functions..');
% tic
% delete('residuals.m', 'jacobian.m',...
%         'sigmaTrial.m','Yield.m','InitialStateVars.m');
% % Output
% folder = '..\1_Straindriver\';
% matlabFunction(Res,     'File', [folder,'residuals.m'],     'Vars',h);
% matlabFunction(Jacobis, 'File', [folder,'jacobians.m'],     'Vars',h);
% matlabFunction(sigTrial,'File', [folder,'sigTrials.m'],     'Vars',h); 
% matlabFunction(F,       'File', [folder,'failureCheck.m'],  'Vars',h);
% matlabFunction(v00,     'File', [folder,'initStateVars.m'], 'Vars',h);
% disp('Finished')
% toc
