% Common symbolics
% @JR, 04.05.16

%% Define variables
% Identity vector
I = [1;1;1;0;0;0];

% Plastic multiplier
lam = sym('lam', 'real');
lam0 = sym('lam0', 'real');

% Time increment
dt = sym('dt', 'real');

% Stresses
s11 = sym('s11', 'real');
s22 = sym('s22', 'real');
s33 = sym('s33', 'real');
s12 = sym('s12', 'real');
s23 = sym('s23', 'real');
s13 = sym('s13', 'real');

% Previous stresses
s110 = sym('s110', 'real');
s220 = sym('s220', 'real');
s330 = sym('s330', 'real');
s120 = sym('s120', 'real');
s230 = sym('s230', 'real');
s130 = sym('s130', 'real');

% Strain increment
de11 = sym('de11', 'real');
de22 = sym('de22', 'real');
de33 = sym('de33', 'real');
de12 = sym('de12', 'real');
de23 = sym('de23', 'real');
de13 = sym('de13', 'real');

% Vectors
Sig  = [s11;  s22;  s33;  s12;  s23;  s13 ];
Sig0 = [s110; s220; s330; s120; s230; s130];
dEps = [de11; de22; de33; de12; de23; de13];

%% Invariants
% Sig
p = (s11+s22+s33)/3;
s = Sig - p*I;
sd = s;
sd(4:6) = sqrt(2)*sd(4:6);

smatrix = tomatrix(s);
J2 = 1/2*trace(smatrix^2);
J3 = 1/3*trace(smatrix^3);
sin3teta = -3*sqrt(3)*J3/2/(J2)^(3/2);
q = sqrt(3/2*(sd)'*(sd));

eta        = sd/p;
etav       = sqrt(3/2*(sd'*sd))/p;


% Sig0
p0 = (s110+s220+s330)/3;
s0 = Sig0 - p0*I;
sd0 = s0;
sd0(4:6) = sqrt(2)*sd0(4:6);

s0matrix = tomatrix(s0);
J20 = 1/2*trace(s0matrix^2);
J30 = 1/3*trace(s0matrix^3);
sin3teta0 = -3*sqrt(3)*J30/2/(J20)^(3/2);
q0 = sqrt(3/2*(sd0)'*(sd0));

eta0       = sd0/p0;
etav0      = sqrt(3/2*(sd0'*sd0))/p;

% Eps
dEpsVol   = (de11+de22+de33);
eStrain   = dEps - dEpsVol/3*I;
eDev      = eStrain;
eDev(4:6) = sqrt(2)*eDev(4:6);

ematrix = tomatrix(eDev);
J2Eps = 1/2*trace(ematrix^2);
J3Eps = 1/3*trace(ematrix^3);
sin3tetaEps = -3*sqrt(3)*J3Eps/2/(J2Eps)^(3/2);
qEps = sqrt(3/2*(eDev)'*(eDev));