function createsymbolics()
clc; clear all;
% @JR, 04.05.16
% @Jomy, 16.05.17

%% Model name:
ModelName = 'Test';

%% Material parameters, etc.
% Strength
SuTXC = sym('sutxc', 'real');
SuTXE = sym('sutxe', 'real');
% Stiffness
K  = sym('E', 'real');
G = sym('nu', 'real');

%% Include common symbolics
commonsyms;
isotropicElastic;

%% Matrix properties
NonSym      =   0;   % 1 for non-symmetric D-matrix
iStrsDep    =   0;   % 1 for stress dependent D-matrix
iTang       =   0;   % 1 for tanget D-matrix
iTimeDep    =   0;   % 1 for time dependent D-matrix
matrixProp  = [NonSym; iStrsDep; iTang; iTimeDep; iTimeDep];

%% Input and State vectors
% Input parameters
c = [SuTXC; SuTXE; K; G];

% State variables
v  = [Sig;  lam ];
v0 = [Sig0; lam0];

%% Failure and plastic potential
Su = 0.5*(SuTXC+SuTXE);

% Yield function 
F = q - 2*Su;
% Associated flow
Q = F;

%% Differentials
dEpsPlastic     = diffvec(Q,Sig);
%dEpsPlasticMean = diffvec(Q,s11) + diffvec(Q,s22) + diffvec(Q,s33);

%% Residual equations
r1 = Sig - Sig0 - D*(dEps - dEpsPlastic)*(lam-lam0);
r2 = F;
Res = [r1;r2];

% Jacobian matrix
Jacobis = jacobian(Res,v);

% Trial stress
sigTrial = Sig0 + D*dEps;

% Initialization of state variables
v00 = [Sig; 0];


%% Output Fortran subroutines (for PLAXIS)
tic
%Output
folder = ['..\models\', ModelName];
mkdir(folder);
    
disp('Generating Fortran file...');
% %delete('residuals.f', 'jacobians.f', 'sigTrials.f', 'r.m', 'initStateVars.f', 'ElasticMatrix.f');
    
 fortran2(Res,      'File', [folder, '\residuals.f']);
 fortran2(Jacobis,  'File', [folder, '\jacobians.f']);
 fortran2(sigTrial, 'File', [folder, '\sigTrials.f']);
 fortran2(v00,      'File', [folder, '\initStateVars.f']);
 fortran2(D,        'File', [folder, '\ElasticMatrix.f']);
 
 materialGeneration(ModelName, c, v, v0, matrixProp)
disp('Finished')
toc


% %% Output Matlab files (for simple strain driver)
% h = {v,v0,c,dEps};
% % file( INI1, INI2, INI3, INI4) = h(v, v0, c, dEps) where:
% % v    - Update state variables
% % v0   - Old state variables
% % c    - Material parameters
% % dEps - Strain increment
% 
% 
% disp('Generating Matlab functions..');
% tic
% delete('residuals.m', 'jacobian.m',...
%         'sigmaTrial.m','Yield.m','InitialStateVars.m');
% % Output
% folder = '..\1_Straindriver\';
% matlabFunction(Res,     'File', [folder,'residuals.m'],     'Vars',h);
% matlabFunction(Jacobis, 'File', [folder,'jacobians.m'],     'Vars',h);
% matlabFunction(sigTrial,'File', [folder,'sigTrials.m'],     'Vars',h); 
% matlabFunction(F,       'File', [folder,'failureCheck.m'],  'Vars',h);
% matlabFunction(v00,     'File', [folder,'initStateVars.m'], 'Vars',h);
% disp('Finished')
% toc
