function materialGeneration(ModelName, matrixProp, c, v, v0, Res)
    folder = ['..\models\', ModelName];
    file = fopen([folder,'\', ModelName,'.f90'],'w');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, '!---------------------------   MATLAB GENERATED CODE   -------------------------\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, 'module %sVars ! This module contains material specific variables.\n', ModelName);
    fprintf(file, ' use precision\n');
    fprintf(file, ' use standardVars \n\n');
    fprintf(file, ' real(kind=realDBL), dimension(%d) :: matrStVars, matrStVars0\n', (length(v)-7));
    fprintf(file, ' real(kind=realDBL), dimension(%d) :: c\n', length(c));
    for i = 1:length(c)
        fprintf(file, ' real(kind=realDBL) :: %s \n', c(i));
    end
    if(length(v)>7)
        for i = 8:length(v)
        fprintf(file, ' real(kind=realDBL) :: %s \n', v(i));
        fprintf(file, ' real(kind=realDBL) :: %s \n', v0(i));
        end
    end
    fprintf(file, ' integer            :: state\n');
    fprintf(file, ' integer, parameter :: lenStateVars = size(minStVars)+size(matrStVars)\n'); 
    fprintf(file, ' real(kind=realDBL), dimension(lenStateVars) :: stateVars, stateVars0\n');
    fprintf(file, 'end module %sVars\n', ModelName);
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, 'module %sClass\n', ModelName);
    fprintf(file, ' use precision\n');
    fprintf(file, ' use standardVars\n');
    fprintf(file, ' use mapping\n');
    fprintf(file, ' use %sVars\n', ModelName);
    fprintf(file, ' use matlabCalculations\n');
    fprintf(file, ' use PLAXISroutines\n');
    fprintf(file, ' implicit none\n');
    fprintf(file, ' private\n\n');
    fprintf(file, ' type, public, extends(matlabSymbolics) :: %s\n', ModelName);
    fprintf(file, ' contains\n');
    fprintf(file, '  procedure :: materialMapping  =>  materialMapping\n');
    fprintf(file, '  procedure :: update           =>  update\n');
    fprintf(file, '  procedure :: updateStateVars  =>  updateStateVars\n');
    fprintf(file, '  procedure :: initialize       =>  initialize\n');
    fprintf(file, '  procedure :: stressUpdate     =>  stressUpdate\n');
    fprintf(file, '  procedure :: calcond          =>  calcond\n');
    fprintf(file, '  procedure :: NR               =>  NR\n');
    fprintf(file, '  procedure :: v00              =>  v00\n');
    fprintf(file, '  procedure :: DMatrix          =>  DMatrix\n');
    fprintf(file, '  procedure :: SigTr            =>  SigTr\n');
    fprintf(file, '  procedure :: resid            =>  resid\n');
    fprintf(file, '  procedure :: jacobian         =>  jacobian\n');
    fprintf(file, '  procedure :: IDTasker         =>  IDTasker\n');
    fprintf(file, ' end type %s\n', ModelName);
    fprintf(file, ' contains\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine materialMapping(this)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  ! Material parameters defined in Matlab\n');
    for i = 1:length(c)
        fprintf(file, '     %s = c(%i) \n', c(i), i);
    end
    if(length(v)>7)
        for i = 8:length(v)
        fprintf(file, '     %s = matrStVars(%i) \n', v(i), i-7);
        fprintf(file, '     %s = matrStVars0(%i) \n', v0(i), i-7);
        end
    end
    fprintf(file, ' end subroutine materialMapping\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine update(this)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  call this%%updateStateVars\n');
    fprintf(file, '  call this%%standardMapping\n');
    fprintf(file, '  call this%%materialMapping\n');
    fprintf(file, '  call this%%numhelp\n');
    fprintf(file, ' end subroutine update\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine updateStateVars(this)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  minStVars   = stateVars(1:7)\n');
    fprintf(file, '  matrStVars  = stateVars(8:(size(matrStVars)+8))\n\n');
    fprintf(file, '  minStVars0   = stateVars0(1:7)\n');
    fprintf(file, '  matrStVars0  = stateVars0(8:(size(matrStVars0)+8))\n');
    fprintf(file, ' end subroutine updateStateVars\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine initialize(this)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  call this%%update()\n');
    fprintf(file, '  call this%%v00(stateVars)     ! Initialize state variables\n');
    fprintf(file, ' end subroutine initialize\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine stressUpdate(this)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  logical :: ii\n\n');
    fprintf(file, '  call this%%calcond(ii) ! Check if iterations are needed\n');
    fprintf(file, '  if (ii) then\n');
    fprintf(file, '   call this%%NR()\n');
    fprintf(file, '  end if\n');
    fprintf(file, ' end subroutine stressUpdate\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine calcond(this, aa)\n');  
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '   real(kind=realDBL), dimension(6)   :: Sig0\n');
    fprintf(file, '   real(kind=realDBL), dimension(6,6) :: D\n');
    fprintf(file, '   real(kind=realDBL), dimension(lenStateVars) :: fr\n');
    fprintf(file, '   logical :: aa\n');
    fprintf(file, '   call this%%update()\n\n');
    fprintf(file, '     !Trial predictor - check yield condition\n');
    fprintf(file, '     call this%%DMatrix(D)\n');
    fprintf(file, '     call this%%SigTr(stateVars)\n');
    fprintf(file, '     call this%%resid(fr)\n');
    fprintf(file, '     aa = fr(%i) > 0\n', length(Res(:,1)) );
    fprintf(file, ' end subroutine calcond\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' recursive subroutine NR(this)\n');
    fprintf(file, '  use toolBox, only: gauss\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '   real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: J0\n');
    fprintf(file, '   real(kind=realDBL), dimension(lenStateVars)               :: r , x\n');
    fprintf(file, '   real(kind=realDBL) :: tol, res\n');
    fprintf(file, '   integer :: maxIter, Iter\n\n');
    fprintf(file, '   Iter    = 0\n');
    fprintf(file, '   tol     = 1.0D-8\n');
    fprintf(file, '   maxIter = 75\n');
    fprintf(file, '   res     = 1.0D0/tol\n');
    fprintf(file, '   x       = 0.0D0\n');
    fprintf(file, '   r       = 0.0D0\n');
    fprintf(file, '     do while (res > tol .and. Iter < maxIter)\n');
    fprintf(file, '         ! Calculate new position\n');
    fprintf(file, '         call this%%resid(r)\n');
    fprintf(file, '         call this%%jacobian(J0)\n');
    fprintf(file, '         !call WriMat ( 1, ''Jacobian'' , J0 , 6, 6, 6 )\n');
    fprintf(file, '         !call flush(1)\n');
    fprintf(file, '         call gauss(J0, -r, x, lenStateVars)\n');
    fprintf(file, '         stateVars = stateVars + x\n');
    fprintf(file, '         ! Check residual\n');
    fprintf(file, '         call this%%resid(r)\n');
    fprintf(file, '         Iter = Iter + 1\n');
    fprintf(file, '         res = dot_product(r,r)\n');
    fprintf(file, '     end do\n');
    fprintf(file, '     ! In case of to many iterations\n');
    fprintf(file, '     if (iter > maxIter) then\n');
    fprintf(file, '       state = 2\n');
    fprintf(file, '       !write(100, *) res\n');
    fprintf(file, '     end if\n');
    fprintf(file, ' end subroutine NR\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine v00(this, A0) ! Initialization of state variables\n');
    fprintf(file, '  !The routine returns statevariables according to the CAS calculation in Matlab.\n');
    fprintf(file, '  Implicit Double Precision (A-Z)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  real(kind=realDBL), dimension(lenStateVars, 1) :: A0\n');
    fprintf(file, '  call this%%update()\n');
    fprintf(file, '  A0    = 0.D0   ! Hard reset variables\n');
    fprintf(file, '  !Matlab code\n');
    % Open and copy initStateVars.
    temp = fopen(['..\models\', ModelName,'\', 'initStateVars.f'],'r');
    tline = fgets(temp);
    while ischar(tline)
        fprintf(file, tline);
        tline = fgets(temp);
    end
    fclose(temp);
    fprintf(file, ' end subroutine v00\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine DMatrix(this, A0) ! Elastic stiffness matrix\n');
    fprintf(file, '  !The routine returns the elastic stiffness matrix\n');
    fprintf(file, '  Implicit Double Precision (A-Z)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  real(kind=realDBL), dimension(6,6) :: A0\n');
    fprintf(file, '  call this%%update()\n');
    fprintf(file, '  A0 = 0.D0   ! Hard reset variables\n');
    fprintf(file, '  !Matlab code\n');
    % Open and copy Elastic Matrix.
    temp = fopen(['..\models\', ModelName,'\', 'ElasticMatrix.f'],'r');
    tline = fgets(temp);
    while ischar(tline)
        fprintf(file, tline);
        tline = fgets(temp);
    end
    fclose(temp);   
    fprintf(file, ' end subroutine Dmatrix\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine SigTr(this, A0) ! Trial stress\n');
    fprintf(file, '  !The routine calculates the elastic predictor stress\n');
    fprintf(file, '  Implicit Double Precision (A-Z)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  real(kind=realDBL), dimension(6, 1) :: A0\n');
    fprintf(file, '  call this%%update()\n');
    fprintf(file, '  A0 = 0.D0   ! Hard reset variables\n');
    fprintf(file, '  !Matlab code\n');
    % Open and copy sigTrials.
    temp = fopen(['..\models\', ModelName,'\', 'sigTrials.f'],'r');
    tline = fgets(temp);
    while ischar(tline)
        fprintf(file, tline);
        tline = fgets(temp);
    end
    fclose(temp);   
    fprintf(file, ' end subroutine SigTr\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' subroutine resid(this, A0) ! Residual equation\n');
    fprintf(file, '  !The routine calculates the residuals\n');
    fprintf(file, '  Implicit Double Precision (A-Z)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  real(kind=realDBL), dimension(lenStateVars, 1) :: A0\n');
    fprintf(file, '  call this%%update()\n');
    fprintf(file, '  A0 = 0.D0   ! Hard reset variables\n');
    fprintf(file, '  !Matlab code\n');
    % Open and copy residuals.
    temp = fopen(['..\models\', ModelName,'\', 'residuals.f'],'r');
    tline = fgets(temp);
    while ischar(tline)
        fprintf(file, tline);
        tline = fgets(temp);
    end
    fclose(temp);   
    fprintf(file, ' end subroutine resid\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    
    fprintf(file, ' subroutine jacobian(this, A0) ! Jacobian matrix\n');
    fprintf(file, '  Implicit Double Precision (A-Z)\n');
    fprintf(file, '  class(%s) :: this\n', ModelName);
    fprintf(file, '  real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: A0\n');
    fprintf(file, '  call this%%update()\n');
    fprintf(file, '  A0 = 0.D0   ! Hard reset variables\n');
    fprintf(file, '  !Matlab code\n');
    % Open and copy residuals.
    temp = fopen(['..\models\', ModelName,'\', 'jacobians.f'],'r');
    tline = fgets(temp);
    while ischar(tline)
        fprintf(file, tline);
        tline = fgets(temp);
    end
    fclose(temp);   
    fprintf(file, ' end subroutine jacobian\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, [' subroutine IDTasker(this, IDTask, IsUndr, &\n\t\t\t\t\tiStep, iTer, iEl, Int, &\n' ...
                    '\t\t\t\t\tX, Y, Z, &\n\t\t\t\t\tTime0, dTime, &\n\t\t\t\t\tProps, Sig0, Swp0, StVar0,' ...
                    '&\n\t\t\t\t\tdEps, D, BulkW, & \n\t\t\t\t\tSig, Swp, StVar, ipl, &\n\t\t\t\t\tnStat,' ...
                    'NonSym, iStrsDep, &\n\t\t\t\t\tiTimeDep, iTang, iAbort, &\n \t\t\t\t iPrjDir, iPrjLen, IO)\n']);
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, '! Import local scripts and local modules\n');
    fprintf(file, '  ! use toolBox\n');
    fprintf(file, '  use PLAXISroutines\n');
    fprintf(file, '  use precision\n');
    fprintf(file, '  use standardVars\n');
    fprintf(file, '  use %sVars\n', ModelName);
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, '! PLAXIS - Dictionary\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, '!  Depending on IDTask, 1 : Initialize state variables\n');
    fprintf(file, '!                       2 : calculate stresses\n');
    fprintf(file, '!                       3 : calculate material stiffness matrix\n');
    fprintf(file, '!                       4 : return number of state variables\n');
    fprintf(file, '!                       5 : inquire matrix properties\n');
    fprintf(file, '!                           return switch for non-symmetric D-matrix\n');
    fprintf(file, '!                           stress/time dependent matrix\n');
    fprintf(file, '!                       6 : calculate elastic material stiffness matrix\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' Implicit Double Precision (A-H, O-Z)\n');
    fprintf(file, ' class(%s) :: this\n', ModelName);
    fprintf(file, '! Declare variable types\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' Character(len=255) :: PrjDir\n');
    fprintf(file, ' Character(len=*)   :: iPrjDir\n');
    fprintf(file, ' INTEGER :: IDTask, IsUndr, iStep, iTer, iEl, Int, iPrjLen IO\n');
    fprintf(file, ' INTEGER :: ipl,  nStat, NonSym, iStrsDep, iTimeDep, iTang, iAbort, IO\n');
    fprintf(file, ' real(kind=realDBL), DIMENSION(6,6) ::  D\n');
    fprintf(file, ' real(kind=realDBL), DIMENSION(6)   ::  Sig0, Sig, dEps\n');
    fprintf(file, ' real(kind=realDBL), DIMENSION(lenStateVars)  ::  StVar0, StVar, StVarPrevMatr\n');
    fprintf(file, ' real(kind=realDBL), DIMENSION(lenStateVars)  ::  StVarPrev, StVarInitPrev\n');
    fprintf(file, ' real(kind=realDBL), DIMENSION(size(c))       ::  Props\n');
    fprintf(file, ' ! Local variables\n');
    fprintf(file, ' INTEGER ::  ne, nt, n, i, msf, iOpt\n');
    fprintf(file, ' real(kind=realDBL) :: dNorm, dEpsMax, dTimeMax\n');
    fprintf(file, ' real(kind=realDBL), DIMENSION(6)   ::  dAbsEps\n');
    fprintf(file, ' Logical :: existence = .FALSE.\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, '!******************************* MODEL DESCRIPTION *****************************\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, '! PLAXIS - IDTask\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, '! Initialize state variables\n');
    fprintf(file, ' Task_1: if (IDTask == 1) then\n');
    fprintf(file, '     stateVars       =  0.0D0\n');
    fprintf(file, '     stateVars0      =  0.0D0\n');
    fprintf(file, '     stateVars(1:6)  = -1.0D0*Sig0(1:6)  ! Define compression as positive\n');
    fprintf(file, '     c               =  Props\n\n');
    fprintf(file, '     if (norm2(stateVars0) == 0) then\n');
    fprintf(file, '       call this%%initialize()\n');
    fprintf(file, '       StVar0 = stateVars\n');
    fprintf(file, '     end if\n');
    fprintf(file, '  end if Task_1\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' ! Calculate stresses\n');
    fprintf(file, ' Task_2: if (IDTask == 2) then\n');
    fprintf(file, '     ipl            = 0\n');
    fprintf(file, '     stateVars      = StVar0\n');
    fprintf(file, '     stateVars(1:6) = -Sig0(1:6)\n');
    fprintf(file, '     stateVars0     = stateVars\n');
    fprintf(file, '     dt             = dTime\n');
    fprintf(file, '     dEpsilon       = 0.D0\n');
    fprintf(file, '     dEpsilon(1:6)  = -1.0D0*dEps(1:6)\n');
    fprintf(file, '     state          = StVar0(lenStateVars + 1)\n');
    fprintf(file, '     c              = Props\n\n');
    fprintf(file, '     If (IsUndr == 1) then\n');
    fprintf(file, '       Swp = Swp0 + BulkW*(dEps(1) + dEps(2) + dEps(3))\n');
    fprintf(file, '     else\n');
    fprintf(file, '       Swp = Swp0\n');
    fprintf(file, '     end if\n\n');
    fprintf(file, '     dNorm = norm2(dEpsilon)\n\n');
    fprintf(file, '     if (dnorm /= 0) then\n');
    fprintf(file, '       dAbsEps  = abs(dEpsilon)\n');
    fprintf(file, '       dEpsMax  = max(dAbsEps(1), dAbsEps(2), dAbsEps(3), dAbsEps(4), dAbsEps(5), dAbsEps(6))\n');
    fprintf(file, '       dTimeMax = dt\n');
    fprintf(file, '       ne       = nint(demax/1.0D-3)\n');
    fprintf(file, '       nt       = nint(dtmax/1.0D3)\n\n');
    fprintf(file, '       if (n == 0) then\n');
    fprintf(file, '         n = 1\n');
    fprintf(file, '       end if\n\n');
    fprintf(file, '       if (nt == 0) then\n');
    fprintf(file, '         nt = 1\n');
    fprintf(file, '       end if\n\n');
    fprintf(file, '       n   = max(ne, nt)\n');
    fprintf(file, '       dEpsilon  = dEpsilon/n*1.0D0\n');
    fprintf(file, '       dt  = dt/n*1.0D0\n\n');
    fprintf(file, '       do i = 1, n\n');
    fprintf(file, '         call this%%stressupdate()     ! Note: Updates stateVars\n');
    fprintf(file, '         stateVars0  = stateVars\n');
    fprintf(file, '         ipl         = state\n');
    fprintf(file, '        end do\n');
    fprintf(file, '     end if\n\n');
    fprintf(file, '     ! Store updated values\n');
    fprintf(file, '     Sig(1:6)              = -1.0D0*stateVars(1:6)\n');
    fprintf(file, '     StVar                 = stateVars\n');
    fprintf(file, '     StVar(lenStateVars+1) = state\n');
    fprintf(file, '  end if Task_2\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' ! Calculate material stiffness matrix\n');
    fprintf(file, ' Task_3: if (IDTask == 3) then\n');
    if matrixProp(5)==1
        fprintf(file, '     stateVars = StVar0     ! Used if matrix is stress dependent\n');
    end
    fprintf(file, '     call this%%DMatrix(D)\n');
    fprintf(file, '     !D = 2.0d0*D     ! TODO: Why 2x ?\n');
    fprintf(file, '     msf = Sig0(15)  ! TODO: What does this do??\n');
    fprintf(file, '  end if Task_3\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' ! Number of state variables\n');
    fprintf(file, ' Task_4: if (IDTask == 4) then\n');
    fprintf(file, '     nStat = lenStateVars + 1\n');
    fprintf(file, '  end if Task_4\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' ! Inquire matrix properties\n');
    fprintf(file, ' Task_5: if (IDTask == 5) then\n');
    fprintf(file, '     NonSym      =   %i   ! 1 for non-symmetric D-matrix\n',     matrixProp(1));
    fprintf(file, '     iStrsDep    =   %i   ! 1 for stress dependent D-matrix\n',  matrixProp(2));
    fprintf(file, '     iTang       =   %i   ! 1 for tanget D-matrix\n',            matrixProp(3));
    fprintf(file, '     iTimeDep    =   %i   ! 1 for time dependent D-matrix\n',    matrixProp(4));
    fprintf(file, '  end if Task_5\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, ' ! Calculate elastic material stiffness matrix\n');
    fprintf(file, ' Task_6: if (IDTask == 6) then\n');
    if matrixProp(5)==1
        fprintf(file, '     stateVars = StVar0     ! Used if matrix is stress dependent\n');
    end
    fprintf(file, '     call this%%DMatrix(D)\n');
    fprintf(file, '     !D = 2.0d0*D     ! TODO: Why 2x ?\n');
    fprintf(file, '  end if Task_6\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, '!-------------------------------------------------------------------------------\n');
    fprintf(file, 'end subroutine IDTasker\n');
    fprintf(file, 'end module %sClass\n',ModelName);
     
    fclose(file);
 delete([folder, '\residuals.f'],     [folder, '\jacobians.f'], ...
        [folder, '\sigTrials.f'],     [folder, '\initStateVars.f'], ...
        [folder, '\ElasticMatrix.f']);



    filey = fopen([folder,'\', ModelName,'.txt'],'w');
     fprintf(filey, 'Material parameters:\n');
     for i = 1:length(c)
          fprintf(filey, '%s\n', c(i));
     end
     fprintf(filey, 'State variables:\n');
     for i = 1:length(v)
          fprintf(filey, '%s\n', v(i));
     end
     fclose(filey)
end