!-------------------------------------------------------------------------------
!---------------------------   MATLAB GENERATED CODE   -------------------------
!-------------------------------------------------------------------------------
module ModifiedCamClayAverageVars ! This module contains material specific variables.
 use precision
 use standardVars

 real(kind=realDBL), dimension(2) :: matrStVars, matrStVars0
 real(kind=realDBL), dimension(8) :: c
 real(kind=realDBL) :: phi
 real(kind=realDBL) :: l
 real(kind=realDBL) :: k
 real(kind=realDBL) :: nu
 real(kind=realDBL) :: OCR
 real(kind=realDBL) :: av
 real(kind=realDBL) :: w
 real(kind=realDBL) :: chi00
 real(kind=realDBL) :: pmi
 real(kind=realDBL) :: pmi0
 real(kind=realDBL) :: chi
 real(kind=realDBL) :: chi0
 integer            :: state
 integer, parameter :: lenStateVars = size(minStVars)+size(matrStVars)
 real(kind=realDBL), dimension(lenStateVars) :: stateVars, stateVars0
end module ModifiedCamClayAverageVars
!-------------------------------------------------------------------------------
module ModifiedCamClayAverageClass
 use precision
 use standardVars
 use mapping
 use ModifiedCamClayAverageVars
 use matlabCalculations
 use PLAXISroutines
 implicit none
 private

 type, public, extends(matlabSymbolics) :: ModifiedCamClayAverage
 contains
  procedure :: materialMapping  =>  materialMapping
  procedure :: update           =>  update
  procedure :: updateStateVars  =>  updateStateVars
  procedure :: initialize       =>  initialize
  procedure :: stressUpdate     =>  stressUpdate
  procedure :: calcond          =>  calcond
  procedure :: NR               =>  NR
  procedure :: v00              =>  v00
  procedure :: DMatrix          =>  DMatrix
  procedure :: SigTr            =>  SigTr
  procedure :: resid            =>  resid
  procedure :: jacobian         =>  jacobian
  procedure :: IDTasker         =>  IDTasker
 end type ModifiedCamClayAverage
 contains
!-------------------------------------------------------------------------------
 subroutine materialMapping(this)
  class(ModifiedCamClayAverage) :: this
  ! Material parameters defined in Matlab
     phi = c(1)
     l = c(2)
     k = c(3)
     nu = c(4)
     OCR = c(5)
     av = c(6)
     w = c(7)
     chi00 = c(8)
     pmi = matrStVars(1)
     pmi0 = matrStVars0(1)
     chi = matrStVars(2)
     chi0 = matrStVars0(2)
 end subroutine materialMapping
!-------------------------------------------------------------------------------
 subroutine update(this)
  class(ModifiedCamClayAverage) :: this
  call this%updateStateVars
  call this%standardMapping
  call this%materialMapping
  call this%numhelp
 end subroutine update
!-------------------------------------------------------------------------------
 subroutine updateStateVars(this)
  class(ModifiedCamClayAverage) :: this
  minStVars   = stateVars(1:7)
  matrStVars  = stateVars(8:(size(matrStVars)+8))

  minStVars0   = stateVars0(1:7)
  matrStVars0  = stateVars0(8:(size(matrStVars0)+8))
 end subroutine updateStateVars
!-------------------------------------------------------------------------------
 subroutine initialize(this)
  class(ModifiedCamClayAverage) :: this
  call this%update()
  call this%v00(stateVars)     ! Initialize state variables
 end subroutine initialize
!-------------------------------------------------------------------------------
 subroutine stressUpdate(this)
  class(ModifiedCamClayAverage) :: this
  logical :: ii

  call this%calcond(ii) ! Check if iterations are needed
  if (ii) then
   call this%NR()
  end if
 end subroutine stressUpdate
!-------------------------------------------------------------------------------
 subroutine calcond(this, aa)
  class(ModifiedCamClayAverage) :: this
   real(kind=realDBL), dimension(6)   :: Sig0
   real(kind=realDBL), dimension(6,6) :: D
   real(kind=realDBL), dimension(lenStateVars) :: fr
   logical :: aa

   integer :: ios
   call this%update()

     !Trial predictor - check yield condition
    !  call this%DMatrix(D)
     call this%SigTr(stateVars)
     call this%resid(fr)
     aa = fr(9) > 0

 end subroutine calcond
!-------------------------------------------------------------------------------
 recursive subroutine NR(this)
  use toolBox, only: gauss
  class(ModifiedCamClayAverage) :: this
   real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: J0
   real(kind=realDBL), dimension(lenStateVars)               :: r , x
   real(kind=realDBL) :: tol, res
   integer :: maxIter, Iter

   Iter    = 0
   tol     = 1.0D-8
   maxIter = 75
   res     = 1.0D0/tol
   x       = 0.0D0
   r       = 0.0D0
     do while (res > tol .and. Iter < maxIter)
         ! Calculate new position
         call this%resid(r)
         call this%jacobian(J0)
         !call WriMat ( 1, 'Jacobian' , J0 , 6, 6, 6 )
         !call flush(1)
         call gauss(J0, -r, x, lenStateVars)
         stateVars = stateVars + x
         ! Check residual
         call this%resid(r)
         Iter = Iter + 1
         res = dot_product(r,r)
     end do
     ! In case of to many iterations
     if (iter > maxIter) then
       state = 2
       !write(100, *) res
     end if
 end subroutine NR
!-------------------------------------------------------------------------------
 subroutine v00(this, A0) ! Initialization of state variables
  !The routine returns statevariables according to the CAS calculation in Matlab.
  Implicit Double Precision (A-Z)
  class(ModifiedCamClayAverage) :: this
  real(kind=realDBL), dimension(lenStateVars, 1) :: A0
  call this%update()
  A0    = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = phi*3.141592653589793D0*(1.0D0/1.8D2)
      t3 = sin(t2)
      t4 = t3-3.0D0
      t5 = s22*(1.0D0/3.0D0)
      t6 = s33*(1.0D0/3.0D0)
      t7 = s33*(1.0D0/2.0D0)
      t8 = s11*(1.0D0/3.0D0)
      t9 = s11*(1.0D0/2.0D0)
      t10 = s22*(1.0D0/2.0D0)
      t11 = t5+t6+t8+1.0D0/1.0D1
      A0(1,1) = s11
      A0(2,1) = s22
      A0(3,1) = s33
      A0(4,1) = s12
      A0(5,1) = s23
      A0(6,1) = s13
      A0(8,1) = (OCR*1.0D0/t3**2*t4**2*((s11*(-2.0D0/3.0D0)+t5+t6)*(-s11+t7+t10)+(-s22+t7+t9)*(s22*(-2.0D0/3.0D0)+t6+t8)+(s33*(-2.0D0/3.0D0)+t5+t8)*(-s33+t9+t10)+s12**2*3.0D0+s13**2*3.0D0+s23**2*3.0D0+t3**2*1.0D0/t4**2*t11**2*3.6D1)*(1.0D0/3.6D1))/(t11*(chi+1.0D0)*(chi00+1.0D0))
      A0(9,1) = chi00
 end subroutine v00
!-------------------------------------------------------------------------------
 subroutine DMatrix(this, A0) ! Elastic stiffness matrix
  !The routine returns the elastic stiffness matrix
  Implicit Double Precision (A-Z)
  class(ModifiedCamClayAverage) :: this
  real(kind=realDBL), dimension(6,6) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = 1.0D0/k
      t3 = s11*(1.0D0/3.0D0)
      t4 = s22*(1.0D0/3.0D0)
      t5 = s33*(1.0D0/3.0D0)
      t6 = t3+t4+t5+1.0D0/1.0D1
      t7 = t2*t6
      t8 = nu*6.0D0
      t9 = t8-3.0D0
      t10 = nu+1.0D0
      t11 = 1.0D0/t10
      t12 = t2*t6*t9*t11*(1.0D0/3.0D0)
      t13 = t7+t12
      t15 = t2*t6*t9*t11*(2.0D0/3.0D0)
      t14 = t7-t15
      A0(1,1) = t14
      A0(1,2) = t13
      A0(1,3) = t13
      A0(2,1) = t13
      A0(2,2) = t14
      A0(2,3) = t13
      A0(3,1) = t13
      A0(3,2) = t13
      A0(3,3) = t14
      A0(4,4) = t2*t6*t9*t11*(-1.0D0/2.0D0)
      A0(5,5) = t2*t6*t9*t11*(-1.0D0/2.0D0)
      A0(6,6) = t2*t6*t9*t11*(-1.0D0/2.0D0)
 end subroutine Dmatrix
!-------------------------------------------------------------------------------
 subroutine SigTr(this, A0) ! Trial stress
  !The routine calculates the elastic predictor stress
  Implicit Double Precision (A-Z)
  class(ModifiedCamClayAverage) :: this
  real(kind=realDBL), dimension(6, 1) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = 1.0D0/k
      t3 = s11*(1.0D0/3.0D0)
      t4 = s22*(1.0D0/3.0D0)
      t5 = s33*(1.0D0/3.0D0)
      t6 = t3+t4+t5+1.0D0/1.0D1
      t7 = t2*t6
      t8 = nu*6.0D0
      t9 = t8-3.0D0
      t10 = nu+1.0D0
      t11 = 1.0D0/t10
      t12 = t2*t6*t9*t11*(1.0D0/3.0D0)
      t13 = t7+t12
      t18 = t2*t6*t9*t11*(2.0D0/3.0D0)
      t14 = t7-t18
      t15 = de33*t13
      t16 = de11*t13
      t17 = de22*t13
      A0(1,1) = s110+t15+t17+de11*t14
      A0(2,1) = s220+t15+t16+de22*t14
      A0(3,1) = s330+t16+t17+de33*t14
      A0(4,1) = s120-de12*t2*t6*t9*t11*(1.0D0/2.0D0)
      A0(5,1) = s230-de23*t2*t6*t9*t11*(1.0D0/2.0D0)
      A0(6,1) = s130-de13*t2*t6*t9*t11*(1.0D0/2.0D0)
 end subroutine SigTr
!-------------------------------------------------------------------------------
 subroutine resid(this, A0) ! Residual equation
  !The routine calculates the residuals
  Implicit Double Precision (A-Z)
  class(ModifiedCamClayAverage) :: this
  real(kind=realDBL), dimension(lenStateVars, 1) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t3 = phi*3.141592653589793D0*(1.0D0/1.8D2)
      t2 = sin(t3)
      t4 = t2**2
      t5 = t2-3.0D0
      t6 = 1.0D0/t5**2
      t7 = s11*(1.0D0/3.0D0)
      t8 = s22*(1.0D0/3.0D0)
      t9 = s33*(1.0D0/3.0D0)
      t10 = t7+t8+t9+1.0D0/1.0D1
      t11 = 1.0D0/k
      t12 = lam-lam0
      t13 = chi+1.0D0
      t22 = pmi*t13
      t14 = t7+t8+t9-t22+1.0D0/1.0D1
      t15 = t4*t6*t14*1.2D1
      t16 = t4*t6*t10*1.2D1
      t17 = t10*t11
      t18 = nu*6.0D0
      t19 = t18-3.0D0
      t20 = nu+1.0D0
      t21 = 1.0D0/t20
      t23 = t10*t11*t19*t21*(1.0D0/3.0D0)
      t24 = t17+t23
      t25 = s11*2.0D0
      t26 = -s22-s33+t15+t16+t25
      t35 = t12*t26
      t27 = de11-t35
      t28 = s22*2.0D0
      t29 = -s11-s33+t15+t16+t28
      t36 = t12*t29
      t30 = de22-t36
      t38 = t10*t11*t19*t21*(2.0D0/3.0D0)
      t31 = t17-t38
      t32 = s33*2.0D0
      t33 = -s11-s22+t15+t16+t32
      t37 = t12*t33
      t34 = de33-t37
      t39 = t4*t6*t14*3.6D1
      t40 = t4*t6*t10*3.6D1
      t41 = t39+t40
      t42 = s11+s22-t32
      t43 = s11+s33-t28
      t44 = s22+s33-t25
      t45 = s33*(1.0D0/2.0D0)
      t46 = s11*(1.0D0/2.0D0)
      t47 = s22*(1.0D0/2.0D0)
      t48 = s12**2
      t49 = s13**2
      t50 = s23**2
      A0(1,1) = s11-s110-t24*t30-t24*t34-t27*t31
      A0(2,1) = s22-s220-t24*t27-t24*t34-t30*t31
      A0(3,1) = s33-s330-t24*t27-t24*t30-t31*t34
      A0(4,1) = s12-s120+t10*t11*t19*t21*(de12-s12*t12*6.0D0)*(1.0D0/2.0D0)
      A0(5,1) = s23-s230+t10*t11*t19*t21*(de23-s23*t12*6.0D0)*(1.0D0/2.0D0)
      A0(6,1) = s13-s130+t10*t11*t19*t21*(de13-s13*t12*6.0D0)*(1.0D0/2.0D0)
      A0(7,1) = pmi-pmi0+(pmi*t12*t41)/(k-l)
      A0(8,1) = chi-chi0+av*chi*t12*sqrt(w**2*(t48*4.8D1+t49*4.8D1+t50*4.8D1+t42**2*(2.0D0/3.0D0)+t43**2*(2.0D0/3.0D0)+t44**2*(2.0D0/3.0D0))+t41**2)
      A0(9,1) = t48*3.0D0+t49*3.0D0+t50*3.0D0+(s11*(-2.0D0/3.0D0)+t8+t9)*(-s11+t45+t47)+(s22*(-2.0D0/3.0D0)+t7+t9)*(-s22+t45+t46)+(s33*(-2.0D0/3.0D0)+t7+t8)*(-s33+t46+t47)+t4*t6*t10*t14*3.6D1
 end subroutine resid
!-------------------------------------------------------------------------------
 subroutine jacobian(this, A0) ! Jacobian matrix
  Implicit Double Precision (A-Z)
  class(ModifiedCamClayAverage) :: this
  real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = 1.0D0/k
      t4 = phi*3.141592653589793D0*(1.0D0/1.8D2)
      t3 = sin(t4)
      t5 = t3**2
      t6 = t3-3.0D0
      t7 = 1.0D0/t6**2
      t8 = s11*(1.0D0/3.0D0)
      t9 = s22*(1.0D0/3.0D0)
      t10 = s33*(1.0D0/3.0D0)
      t11 = t2*(1.0D0/3.0D0)
      t12 = nu*6.0D0
      t13 = t12-3.0D0
      t14 = nu+1.0D0
      t15 = 1.0D0/t14
      t16 = lam-lam0
      t17 = chi+1.0D0
      t24 = pmi*t17
      t18 = t8+t9+t10-t24+1.0D0/1.0D1
      t19 = t5*t7*t18*1.2D1
      t20 = t8+t9+t10+1.0D0/1.0D1
      t21 = t5*t7*t20*1.2D1
      t22 = t2*t13*t15*(1.0D0/9.0D0)
      t23 = t11+t22
      t25 = t5*t7*8.0D0
      t26 = t2*t20
      t42 = t2*t13*t15*(2.0D0/9.0D0)
      t27 = t11-t42
      t28 = s11*2.0D0
      t29 = -s22-s33+t19+t21+t28
      t43 = t16*t29
      t30 = de11-t43
      t31 = s22*2.0D0
      t32 = -s11-s33+t19+t21+t31
      t44 = t16*t32
      t33 = de22-t44
      t34 = s33*2.0D0
      t35 = -s11-s22+t19+t21+t34
      t45 = t16*t35
      t36 = de33-t45
      t37 = t25-1.0D0
      t38 = t2*t13*t15*t20*(1.0D0/3.0D0)
      t39 = t26+t38
      t47 = t2*t13*t15*t20*(2.0D0/3.0D0)
      t40 = t26-t47
      t41 = t25+2.0D0
      t46 = t16*t37*t39
      t48 = t16*t37*t40
      t49 = t16*t39*t41
      t51 = t23*t36
      t59 = t23*t33
      t50 = t46+t48+t49-t51-t59-t27*t30
      t52 = t16*t37*t39*2.0D0
      t53 = t16*t40*t41
      t58 = t23*t30
      t54 = t46+t48+t49-t51-t58-t27*t33
      t55 = t35*t39
      t64 = t5*t7*t16*t17*t39*2.4D1
      t65 = t5*t7*t16*t17*t40*1.2D1
      t56 = -t64-t65
      t66 = pmi*t5*t7*t16*t39*2.4D1
      t67 = pmi*t5*t7*t16*t40*1.2D1
      t57 = -t66-t67
      t61 = t27*t36
      t60 = t46+t48+t49-t58-t59-t61
      t62 = t32*t39
      t63 = t29*t39
      t70 = s12*t16*6.0D0
      t68 = de12-t70
      t69 = t2*t13*t15*t68*(1.0D0/6.0D0)
      t73 = s23*t16*6.0D0
      t71 = de23-t73
      t72 = t2*t13*t15*t71*(1.0D0/6.0D0)
      t74 = lam*6.0D0
      t75 = lam0*6.0D0
      t76 = t74-t75
      t81 = t2*t13*t15*t20*t76*(1.0D0/2.0D0)
      t77 = -t81+1.0D0
      t80 = s13*t16*6.0D0
      t78 = de13-t80
      t79 = t2*t13*t15*t78*(1.0D0/6.0D0)
      t82 = k-l
      t83 = 1.0D0/t82
      t84 = pmi*t5*t7*t16*t83*2.4D1
      t85 = t5*t7*t18*3.6D1
      t86 = t5*t7*t20*3.6D1
      t87 = t85+t86
      t88 = s11+s22-t34
      t89 = s11+s33-t31
      t90 = s22+s33-t28
      t91 = w**2
      t92 = t87**2
      t93 = t88**2
      t94 = t93*(2.0D0/3.0D0)
      t95 = t89**2
      t96 = t95*(2.0D0/3.0D0)
      t97 = t90**2
      t98 = t97*(2.0D0/3.0D0)
      t99 = s12**2
      t100 = t99*4.8D1
      t101 = s13**2
      t102 = t101*4.8D1
      t103 = s23**2
      t104 = t103*4.8D1
      t105 = t94+t96+t98+t100+t102+t104
      t106 = t91*t105
      t107 = t92+t106
      t108 = 1.0D0/sqrt(t107)
      t109 = s33*4.0D0
      t110 = s11*4.0D0
      t111 = s22*4.0D0
      t112 = sqrt(t107)
      A0(1,1) = t52+t53-t23*t33-t27*t30-t23*t36+1.0D0
      A0(1,2) = t50
      A0(1,3) = t50
      A0(1,7) = t55+t62+t29*t40
      A0(1,8) = t56
      A0(1,9) = t57
      A0(2,1) = t54
      A0(2,2) = -t51+t52+t53-t23*t30-t27*t33+1.0D0
      A0(2,3) = t54
      A0(2,7) = t55+t63+t32*t40
      A0(2,8) = t56
      A0(2,9) = t57
      A0(3,1) = t60
      A0(3,2) = t60
      A0(3,3) = t52+t53-t58-t59-t61+1.0D0
      A0(3,7) = t62+t63+t35*t40
      A0(3,8) = t56
      A0(3,9) = t57
      A0(4,1) = t69
      A0(4,2) = t69
      A0(4,3) = t69
      A0(4,4) = t77
      A0(4,7) = s12*t2*t13*t15*t20*(-3.0D0)
      A0(5,1) = t72
      A0(5,2) = t72
      A0(5,3) = t72
      A0(5,5) = t77
      A0(5,7) = s23*t2*t13*t15*t20*(-3.0D0)
      A0(6,1) = t79
      A0(6,2) = t79
      A0(6,3) = t79
      A0(6,6) = t77
      A0(6,7) = s13*t2*t13*t15*t20*(-3.0D0)
      A0(7,1) = t84
      A0(7,2) = t84
      A0(7,3) = t84
      A0(7,7) = pmi*t83*t87
      A0(7,8) = t16*t83*t87-pmi*t5*t7*t16*t17*t83*3.6D1+1.0D0
      A0(7,9) = pmi**2*t5*t7*t16*t83*(-3.6D1)
      A0(8,1) = av*chi*t16*t108*(t91*(s11*(-8.0D0)+t109+t111)-t5*t7*t87*4.8D1)*(-1.0D0/2.0D0)
      A0(8,2) = av*chi*t16*t108*(t91*(s22*(-8.0D0)+t109+t110)-t5*t7*t87*4.8D1)*(-1.0D0/2.0D0)
      A0(8,3) = av*chi*t16*t108*(t91*(s33*(-8.0D0)+t110+t111)-t5*t7*t87*4.8D1)*(-1.0D0/2.0D0)
      A0(8,4) = av*chi*s12*t16*t91*t108*4.8D1
      A0(8,5) = av*chi*s23*t16*t91*t108*4.8D1
      A0(8,6) = av*chi*s13*t16*t91*t108*4.8D1
      A0(8,7) = av*chi*t112
      A0(8,8) = av*chi*t5*t7*t16*t17*t87*t108*(-3.6D1)
      A0(8,9) = av*t16*t112-av*chi*pmi*t5*t7*t16*t87*t108*3.6D1+1.0D0
      A0(9,1) = t29
      A0(9,2) = t32
      A0(9,3) = t35
      A0(9,4) = s12*6.0D0
      A0(9,5) = s23*6.0D0
      A0(9,6) = s13*6.0D0
      A0(9,8) = t5*t7*t17*t20*(-3.6D1)
      A0(9,9) = pmi*t5*t7*t20*(-3.6D1)
 end subroutine jacobian
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
 subroutine IDTasker(this, IDTask, IsUndr, &
					iStep, iTer, iEl, Int, &
					X, Y, Z, &
					Time0, dTime, &
					Props, Sig0, Swp0, StVar0,&
					dEps, D, BulkW, &
					Sig, Swp, StVar, ipl, &
					nStat,NonSym, iStrsDep, &
					iTimeDep, iTang, iAbort, &
 				 iPrjDir, iPrjLen, IO)
!-------------------------------------------------------------------------------
! Import local scripts and local modules
  ! use toolBox
  use PLAXISroutines
  use precision
  use standardVars
  use ModifiedCamClayAverageVars
  use toolBox
!-------------------------------------------------------------------------------
! PLAXIS - Dictionary
!-------------------------------------------------------------------------------
!  Depending on IDTask, 1 : Initialize state variables
!                       2 : calculate stresses
!                       3 : calculate material stiffness matrix
!                       4 : return number of state variables
!                       5 : inquire matrix properties
!                           return switch for non-symmetric D-matrix
!                           stress/time dependent matrix
!                       6 : calculate elastic material stiffness matrix
!-------------------------------------------------------------------------------
 Implicit Double Precision (A-H, O-Z)
 class(ModifiedCamClayAverage) :: this
! Declare variable types
!-------------------------------------------------------------------------------
  Character(len=255) :: PrjDir
  Character(len=*)   :: iPrjDir
  INTEGER :: IDTask, IsUndr, iStep, iTer, iEl, Int, iPrjLen
  INTEGER :: ipl,  nStat, NonSym, iStrsDep, iTimeDep, iTang, iAbort, IO
  real(kind=realDBL), DIMENSION(6,6) ::  D
  real(kind=realDBL), DIMENSION(6)   ::  Sig0, Sig, dEps
  real(kind=realDBL), DIMENSION(lenStateVars)  ::  StVar0, StVar
  real(kind=realDBL), DIMENSION(size(c))       ::  Props
  ! Local variables
  INTEGER ::  ne, nt, n, i, msf, iOpt
  real(kind=realDBL) :: dNorm, dEpsMax, dTimeMax
  real(kind=realDBL), DIMENSION(6)   ::  dAbsEps
  INTEGER, PARAMETER :: StVarPrevLen = 9
  real(kind=realDBL), DIMENSION(StVarPrevLen)  ::  StVarPrev, StVarInitPrev
  Logical :: existence = .FALSE.
!-------------------------------------------------------------------------------
!******************************* MODEL DESCRIPTION *****************************
!-------------------------------------------------------------------------------
! PLAXIS - IDTask
!-------------------------------------------------------------------------------
! Initialize state variables
 Task_1: if (IDTask == 1) then
     stateVars       =  0.0D0
     stateVars0      =  0.0D0
     stateVars(1:6)  = -1.0D0*Sig0(1:6)  ! Define compression as positive
     c               =  Props

     if (norm2(stateVars0) == 0) then
       call this%initialize()
       StVar0 = stateVars
     end if

     Call getPrjDir( iPrjDir, iPrjLen, PrjDir)
     Call checkIfCyclingCalculate ( PrjDir, existence )
     IF (existence) THEN
       ! TODO: Make fix so that cyclic model only update relevant statevars based on same model
       !Call readStVarInGP ( PrjDir, 10, StVarPrevMatr )
       Call storePrevStateVarsToFile( PrjDir )
       !Call sleep(1)
       Call readStVarInGP ( PrjDir, StVarPrevLen, StVarPrev, 1)
      !  Call readStVarInGP ( PrjDir, StVarPrevLen, StVarInitPrev, 2)
       Call updateGPtracker( PrjDir, StVarPrevLen )
      StVar0(lenStateVars) = StVarPrev(StVarPrevLen-1)  ! Change statevariable
    END IF
  end if Task_1
!-------------------------------------------------------------------------------
 ! Calculate stresses
 Task_2: if (IDTask == 2) then
     ipl            = 0
     stateVars      = StVar0
     stateVars(1:6) = -Sig0(1:6)
     stateVars0     = stateVars
     dt             = dTime
     dEpsilon       = 0.D0
     dEpsilon(1:6)  = -1.0D0*dEps(1:6)
     state          = StVar0(lenStateVars + 1)
     c              = Props

     If (IsUndr == 1) then
       Swp = Swp0 + BulkW*(dEps(1) + dEps(2) + dEps(3))
     else
       Swp = Swp0
     end if

     dNorm = norm2(dEpsilon)

     if (dnorm /= 0) then
       dAbsEps  = abs(dEpsilon)
       dEpsMax  = max(dAbsEps(1), dAbsEps(2), dAbsEps(3), dAbsEps(4), dAbsEps(5), dAbsEps(6))
       dTimeMax = dt
       ne       = nint(demax/1.0D-3)
       nt       = nint(dtmax/1.0D3)

       if (n == 0) then
         n = 1
       end if

       if (nt == 0) then
         nt = 1
       end if

       n   = max(ne, nt)
       dEpsilon  = dEpsilon/n*1.0D0
       dt  = dt/n*1.0D0

       do i = 1, n
         call this%stressupdate()     ! Note: Updates stateVars
         stateVars0  = stateVars
         ipl         = state
        end do
     end if

     ! Store updated values
     Sig(1:6)              = -1.0D0*stateVars(1:6)
     StVar                 = stateVars
     StVar(lenStateVars+1) = state
  end if Task_2
!-------------------------------------------------------------------------------
 ! Calculate material stiffness matrix
 Task_3: if (IDTask == 3) then
     stateVars = StVar0     ! Used if matrix is stress dependent
     call this%DMatrix(D)
     !D = 2.0d0*D     ! TODO: Why 2x ?
     msf = Sig0(15)  ! TODO: What does this do??
  end if Task_3
!-------------------------------------------------------------------------------
 ! Number of state variables
 Task_4: if (IDTask == 4) then
     nStat = lenStateVars + 1
  end if Task_4
!-------------------------------------------------------------------------------
 ! Inquire matrix properties
 Task_5: if (IDTask == 5) then
     NonSym      =   0   ! 1 for non-symmetric D-matrix
     iStrsDep    =   0   ! 1 for stress dependent D-matrix
     iTang       =   0   ! 1 for tanget D-matrix
     iTimeDep    =   0   ! 1 for time dependent D-matrix
  end if Task_5
!-------------------------------------------------------------------------------
 ! Calculate elastic material stiffness matrix
 Task_6: if (IDTask == 6) then
     stateVars = StVar0     ! Used if matrix is stress dependent
     call this%DMatrix(D)
     !D = 2.0d0*D     ! TODO: Why 2x ?
  end if Task_6
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
end subroutine IDTasker
end module ModifiedCamClayAverageClass
