!-------------------------------------------------------------------------------
!---------------------------   MATLAB GENERATED CODE   -------------------------
!-------------------------------------------------------------------------------
module nonLinearIsoVonMisesVars ! This module contains material specific variables.
 use precision
 use standardVars

 real(kind=realDBL), dimension(1) :: matrStVars, matrStVars0
 real(kind=realDBL), dimension(3) :: c
 real(kind=realDBL) :: suCyc
 real(kind=realDBL) :: E
 real(kind=realDBL) :: alpha
 real(kind=realDBL) :: chi
 real(kind=realDBL) :: chi0
 integer            :: state
 integer, parameter :: lenStateVars = size(minStVars)+size(matrStVars)
 real(kind=realDBL), dimension(lenStateVars) :: stateVars, stateVars0
end module nonLinearIsoVonMisesVars
!-------------------------------------------------------------------------------
module nonLinearIsoVonMisesClass
 use precision
 use standardVars
 use mapping
 use nonLinearIsoVonMisesVars
 use matlabCalculations
 use PLAXISroutines
 implicit none
 private

 type, public, extends(matlabSymbolics) :: nonLinearIsoVonMises
 contains
  procedure :: materialMapping  =>  materialMapping
  procedure :: update           =>  update
  procedure :: updateStateVars  =>  updateStateVars
  procedure :: initialize       =>  initialize
  procedure :: stressUpdate     =>  stressUpdate
  procedure :: calcond          =>  calcond
  procedure :: NR               =>  NR
  procedure :: v00              =>  v00
  procedure :: DMatrix          =>  DMatrix
  procedure :: SigTr            =>  SigTr
  procedure :: resid            =>  resid
  procedure :: jacobian         =>  jacobian
  procedure :: IDTasker         =>  IDTasker
 end type nonLinearIsoVonMises
 contains
!-------------------------------------------------------------------------------
 subroutine materialMapping(this)
  class(nonLinearIsoVonMises) :: this
  ! Material parameters defined in Matlab
     suCyc = c(1)
     E = c(2)
     alpha = c(3)
     chi = matrStVars(1)
     chi0 = matrStVars0(1)
 end subroutine materialMapping
!-------------------------------------------------------------------------------
 subroutine update(this)
  class(nonLinearIsoVonMises) :: this
  call this%updateStateVars
  call this%standardMapping
  call this%materialMapping
  call this%numhelp
 end subroutine update
!-------------------------------------------------------------------------------
 subroutine updateStateVars(this)
  class(nonLinearIsoVonMises) :: this
  minStVars   = stateVars(1:7)
  matrStVars  = stateVars(8:(size(matrStVars)+8))

  minStVars0   = stateVars0(1:7)
  matrStVars0  = stateVars0(8:(size(matrStVars0)+8))
 end subroutine updateStateVars
!-------------------------------------------------------------------------------
 subroutine initialize(this)
  class(nonLinearIsoVonMises) :: this
  call this%update()
  call this%v00(stateVars)     ! Initialize state variables
 end subroutine initialize
!-------------------------------------------------------------------------------
 subroutine stressUpdate(this)
  class(nonLinearIsoVonMises) :: this
  logical :: ii

  call this%calcond(ii) ! Check if iterations are needed
  if (ii) then
   call this%NR()
  end if
 end subroutine stressUpdate
!-------------------------------------------------------------------------------
 subroutine calcond(this, aa)
  class(nonLinearIsoVonMises) :: this
   real(kind=realDBL), dimension(6)   :: Sig0
   real(kind=realDBL), dimension(6,6) :: D
   real(kind=realDBL), dimension(lenStateVars) :: fr
   logical :: aa
   call this%update()

     !Trial predictor - check yield condition
     call this%DMatrix(D)
     call this%SigTr(stateVars)
     call this%resid(fr)
     aa = fr(7) > 0
     aa = .FALSE.
 end subroutine calcond
!-------------------------------------------------------------------------------
 recursive subroutine NR(this)
  use toolBox, only: gauss
  class(nonLinearIsoVonMises) :: this
   real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: J0
   real(kind=realDBL), dimension(lenStateVars)               :: r , x
   real(kind=realDBL) :: tol, res
   integer :: maxIter, Iter

   Iter    = 0
   tol     = 1.0D-8
   maxIter = 75
   res     = 1.0D0/tol
   x       = 0.0D0
   r       = 0.0D0
     do while (res > tol .and. Iter < maxIter)
         ! Calculate new position
         call this%resid(r)
         call this%jacobian(J0)
         !call WriMat ( 1, 'Jacobian' , J0 , 6, 6, 6 )
         !call flush(1)
         call gauss(J0, -r, x, lenStateVars)
         stateVars = stateVars + x
         ! Check residual
         call this%resid(r)
         Iter = Iter + 1
         res = dot_product(r,r)
     end do
     ! In case of to many iterations
     if (iter > maxIter) then
       state = 2
       !write(100, *) res
     end if
 end subroutine NR
!-------------------------------------------------------------------------------
 subroutine v00(this, A0) ! Initialization of state variables
  !The routine returns statevariables according to the CAS calculation in Matlab.
  Implicit Double Precision (A-Z)
  class(nonLinearIsoVonMises) :: this
  real(kind=realDBL), dimension(lenStateVars, 1) :: A0
  call this%update()
  A0    = 0.D0   ! Hard reset variables
  !Matlab code
      A0(1,1) = s11
      A0(2,1) = s22
      A0(3,1) = s33
      A0(4,1) = s12
      A0(5,1) = s23
      A0(6,1) = s13
 end subroutine v00
!-------------------------------------------------------------------------------
 subroutine DMatrix(this, A0) ! Elastic stiffness matrix
  !The routine returns the elastic stiffness matrix
  Implicit Double Precision (A-Z)
  class(nonLinearIsoVonMises) :: this
  real(kind=realDBL), dimension(6,6) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = s33*(1.0D0/2.0D0)
      t3 = s33*(1.0D0/3.0D0)
      t4 = s11*(1.0D0/2.0D0)
      t5 = s22*(1.0D0/2.0D0)
      t6 = s11*(1.0D0/3.0D0)
      t7 = s22*(1.0D0/3.0D0)
      t8 = E*(1.0D2/3.0D0)
      t9 = 1.0D0/suCyc
      t10 = -s11+t2+t5
      t29 = s11*(2.0D0/3.0D0)
      t11 = t3+t7-t29
      t12 = t10*t11
      t13 = -s22+t2+t4
      t30 = s22*(2.0D0/3.0D0)
      t14 = t3+t6-t30
      t15 = t13*t14
      t16 = -s33+t4+t5
      t31 = s33*(2.0D0/3.0D0)
      t17 = t6+t7-t31
      t18 = t16*t17
      t19 = s12**2
      t20 = t19*3.0D0
      t21 = s13**2
      t22 = t21*3.0D0
      t23 = s23**2
      t24 = t23*3.0D0
      t25 = t12+t15+t18+t20+t22+t24
      t26 = sqrt(t25)
      t27 = t9*t26*(1.0D0/2.0D0)
      t28 = t27-1.01D2/1.0D2
      t32 = E*t28*(2.0D0/9.0D0)
      t33 = t8+t32
      t35 = E*t28*(4.0D0/9.0D0)
      t34 = t8-t35
      A0(1,1) = t34
      A0(1,2) = t33
      A0(1,3) = t33
      A0(2,1) = t33
      A0(2,2) = t34
      A0(2,3) = t33
      A0(3,1) = t33
      A0(3,2) = t33
      A0(3,3) = t34
      A0(4,4) = E*t28*(-1.0D0/3.0D0)
      A0(5,5) = E*t28*(-1.0D0/3.0D0)
      A0(6,6) = E*t28*(-1.0D0/3.0D0)
 end subroutine Dmatrix
!-------------------------------------------------------------------------------
 subroutine SigTr(this, A0) ! Trial stress
  !The routine calculates the elastic predictor stress
  Implicit Double Precision (A-Z)
  class(nonLinearIsoVonMises) :: this
  real(kind=realDBL), dimension(6, 1) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = s33*(1.0D0/2.0D0)
      t3 = s33*(1.0D0/3.0D0)
      t4 = s11*(1.0D0/2.0D0)
      t5 = s22*(1.0D0/2.0D0)
      t6 = s11*(1.0D0/3.0D0)
      t7 = s22*(1.0D0/3.0D0)
      t8 = E*(1.0D2/3.0D0)
      t9 = 1.0D0/suCyc
      t10 = -s11+t2+t5
      t29 = s11*(2.0D0/3.0D0)
      t11 = t3+t7-t29
      t12 = t10*t11
      t13 = -s22+t2+t4
      t30 = s22*(2.0D0/3.0D0)
      t14 = t3+t6-t30
      t15 = t13*t14
      t16 = -s33+t4+t5
      t31 = s33*(2.0D0/3.0D0)
      t17 = t6+t7-t31
      t18 = t16*t17
      t19 = s12**2
      t20 = t19*3.0D0
      t21 = s13**2
      t22 = t21*3.0D0
      t23 = s23**2
      t24 = t23*3.0D0
      t25 = t12+t15+t18+t20+t22+t24
      t26 = sqrt(t25)
      t27 = t9*t26*(1.0D0/2.0D0)
      t28 = t27-1.01D2/1.0D2
      t32 = E*t28*(2.0D0/9.0D0)
      t33 = t8+t32
      t38 = E*t28*(4.0D0/9.0D0)
      t34 = t8-t38
      t35 = de33*t33
      t36 = de11*t33
      t37 = de22*t33
      A0(1,1) = s110+t35+t37+de11*t34
      A0(2,1) = s220+t35+t36+de22*t34
      A0(3,1) = s330+t36+t37+de33*t34
      A0(4,1) = s120-E*de12*t28*(1.0D0/3.0D0)
      A0(5,1) = s230-E*de23*t28*(1.0D0/3.0D0)
      A0(6,1) = s130-E*de13*t28*(1.0D0/3.0D0)
 end subroutine SigTr
!-------------------------------------------------------------------------------
 subroutine resid(this, A0) ! Residual equation
  !The routine calculates the residuals
  Implicit Double Precision (A-Z)
  class(nonLinearIsoVonMises) :: this
  real(kind=realDBL), dimension(lenStateVars, 1) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = s33*(1.0D0/2.0D0)
      t3 = s33*(1.0D0/3.0D0)
      t4 = s11*(1.0D0/2.0D0)
      t5 = s22*(1.0D0/2.0D0)
      t6 = s11*(1.0D0/3.0D0)
      t7 = s22*(1.0D0/3.0D0)
      t8 = -s11+t2+t5
      t25 = s11*(2.0D0/3.0D0)
      t9 = t3+t7-t25
      t10 = t8*t9
      t11 = -s22+t2+t4
      t26 = s22*(2.0D0/3.0D0)
      t12 = t3+t6-t26
      t13 = t11*t12
      t14 = -s33+t4+t5
      t27 = s33*(2.0D0/3.0D0)
      t15 = t6+t7-t27
      t16 = t14*t15
      t17 = s12**2
      t18 = t17*3.0D0
      t19 = s13**2
      t20 = t19*3.0D0
      t21 = s23**2
      t22 = t21*3.0D0
      t23 = t10+t13+t16+t18+t20+t22
      t24 = sqrt(t23)
      t28 = conjg(t24)
      t29 = 1.0D0/t28
      t30 = E*(1.0D2/3.0D0)
      t31 = 1.0D0/suCyc
      t32 = t24*t31*(1.0D0/2.0D0)
      t33 = t32-1.01D2/1.0D2
      t34 = E*t33*(2.0D0/9.0D0)
      t35 = t30+t34
      t36 = lam-lam0
      t48 = s11*2.0D0
      t37 = s22+s33-t48
      t38 = t29*t37*(1.0D0/2.0D0)
      t39 = de11+t38
      t50 = s22*2.0D0
      t40 = s11+s33-t50
      t41 = t29*t40*(1.0D0/2.0D0)
      t42 = de22+t41
      t53 = E*t33*(4.0D0/9.0D0)
      t43 = t30-t53
      t52 = s33*2.0D0
      t44 = s11+s22-t52
      t45 = t29*t44*(1.0D0/2.0D0)
      t46 = de33+t45
      t47 = t35*t46
      t49 = t35*t39
      t51 = t35*t42
      A0(1,1) = s11-s110-t36*(t47+t51+t39*t43)
      A0(2,1) = s22-s220-t36*(t47+t49+t42*t43)
      A0(3,1) = s33-s330-t36*(t49+t51+t43*t46)
      A0(4,1) = s12-s120+E*t33*t36*(de12-s12*t29*3.0D0)*(1.0D0/3.0D0)
      A0(5,1) = s23-s230+E*t33*t36*(de23-s23*t29*3.0D0)*(1.0D0/3.0D0)
      A0(6,1) = s13-s130+E*t33*t36*(de13-s13*t29*3.0D0)*(1.0D0/3.0D0)
      A0(7,1) = suCyc*(-2.0D0)+t24
 end subroutine resid
!-------------------------------------------------------------------------------
 subroutine jacobian(this, A0) ! Jacobian matrix
  Implicit Double Precision (A-Z)
  class(nonLinearIsoVonMises) :: this
  real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = s33*(1.0D0/2.0D0)
      t3 = s33*(1.0D0/3.0D0)
      t4 = s11*(1.0D0/2.0D0)
      t5 = s22*(1.0D0/2.0D0)
      t6 = s11*(1.0D0/3.0D0)
      t7 = s22*(1.0D0/3.0D0)
      t36 = s11*2.0D0
      t8 = s22+s33-t36
      t9 = -s11+t2+t5
      t27 = s11*(2.0D0/3.0D0)
      t10 = t3+t7-t27
      t11 = t9*t10
      t12 = -s22+t2+t4
      t28 = s22*(2.0D0/3.0D0)
      t13 = t3+t6-t28
      t14 = t12*t13
      t15 = -s33+t4+t5
      t29 = s33*(2.0D0/3.0D0)
      t16 = t6+t7-t29
      t17 = t15*t16
      t18 = s12**2
      t19 = t18*3.0D0
      t20 = s13**2
      t21 = t20*3.0D0
      t22 = s23**2
      t23 = t22*3.0D0
      t24 = t11+t14+t17+t19+t21+t23
      t25 = sqrt(t24)
      t26 = conjg(t25)
      t30 = E*(1.0D2/3.0D0)
      t31 = 1.0D0/suCyc
      t32 = t25*t31*(1.0D0/2.0D0)
      t33 = t32-1.01D2/1.0D2
      t34 = 1.0D0/t26
      t35 = 1.0D0/t26**3
      t37 = E*t33*(2.0D0/9.0D0)
      t38 = t30+t37
      t39 = t34*(1.0D0/2.0D0)
      t44 = s22*2.0D0
      t40 = s11+s33-t44
      t41 = 1.0D0/sqrt(t24)
      t45 = s33*2.0D0
      t42 = s11+s22-t45
      t43 = lam-lam0
      t58 = E*t33*(4.0D0/9.0D0)
      t46 = t30-t58
      t47 = t8*t35*t40*(1.0D0/4.0D0)
      t48 = t39+t47
      t49 = t8*t34*(1.0D0/2.0D0)
      t50 = de11+t49
      t51 = t34*t40*(1.0D0/2.0D0)
      t52 = de22+t51
      t53 = t34*t42*(1.0D0/2.0D0)
      t54 = de33+t53
      t55 = t35*t40*t42*(1.0D0/4.0D0)
      t56 = t39+t55
      t57 = t38*t56
      t59 = t8*t35*t42*(1.0D0/4.0D0)
      t60 = t39+t59
      t61 = t8**2
      t73 = t35*t61*(1.0D0/4.0D0)
      t62 = t34-t73
      t63 = t38*t60
      t64 = t46*t48
      t65 = t40**2
      t75 = t35*t65*(1.0D0/4.0D0)
      t66 = t34-t75
      t67 = t38*t48
      t68 = t42**2
      t77 = t35*t68*(1.0D0/4.0D0)
      t69 = t34-t77
      t70 = s12*t35*t38*t42*(3.0D0/2.0D0)
      t71 = s23*t35*t38*t42*(3.0D0/2.0D0)
      t72 = s13*t35*t38*t42*(3.0D0/2.0D0)
      t74 = t46*t60
      t76 = t46*t56
      t78 = s12*t35*t38*t40*(3.0D0/2.0D0)
      t79 = s12*t8*t35*t38*(3.0D0/2.0D0)
      t80 = s23*t35*t38*t40*(3.0D0/2.0D0)
      t81 = s23*t8*t35*t38*(3.0D0/2.0D0)
      t82 = s13*t35*t38*t40*(3.0D0/2.0D0)
      t83 = s13*t8*t35*t38*(3.0D0/2.0D0)
      t85 = s12*t34*3.0D0
      t84 = de12-t85
      t87 = s23*t34*3.0D0
      t86 = de23-t87
      t88 = E*s12*s23*t33*t35*t43*3.0D0
      t89 = t34*3.0D0
      t91 = s13*t34*3.0D0
      t90 = de13-t91
      t92 = E*s12*s13*t33*t35*t43*3.0D0
      t93 = E*s13*s23*t33*t35*t43*3.0D0
      A0(1,1) = -t43*(t63+t67-t46*t62+E*t8*t31*t41*t50*(1.0D0/9.0D0)-E*t8*t31*t41*t52*(1.0D0/1.8D1)-E*t8*t31*t41*t54*(1.0D0/1.8D1))+1.0D0
      A0(1,2) = -t43*(t57+t64-t38*t66+E*t31*t40*t41*t50*(1.0D0/9.0D0)-E*t31*t40*t41*t52*(1.0D0/1.8D1)-E*t31*t40*t41*t54*(1.0D0/1.8D1))
      A0(1,3) = -t43*(t57+t74-t38*t69+E*t31*t41*t42*t50*(1.0D0/9.0D0)-E*t31*t41*t42*t52*(1.0D0/1.8D1)-E*t31*t41*t42*t54*(1.0D0/1.8D1))
      A0(1,4) = t43*(t70+t78+s12*t8*t35*t46*(3.0D0/2.0D0)+E*s12*t31*t41*t50*(2.0D0/3.0D0)-E*s12*t31*t41*t52*(1.0D0/3.0D0)-E*s12*t31*t41*t54*(1.0D0/3.0D0))
      A0(1,5) = t43*(t71+t80+s23*t8*t35*t46*(3.0D0/2.0D0)+E*s23*t31*t41*t50*(2.0D0/3.0D0)-E*s23*t31*t41*t52*(1.0D0/3.0D0)-E*s23*t31*t41*t54*(1.0D0/3.0D0))
      A0(1,6) = t43*(t72+t82+s13*t8*t35*t46*(3.0D0/2.0D0)+E*s13*t31*t41*t50*(2.0D0/3.0D0)-E*s13*t31*t41*t52*(1.0D0/3.0D0)-E*s13*t31*t41*t54*(1.0D0/3.0D0))
      A0(1,7) = -t38*t52-t38*t54-t46*t50
      A0(2,1) = -t43*(t63+t64-t38*t62-E*t8*t31*t41*t50*(1.0D0/1.8D1)+E*t8*t31*t41*t52*(1.0D0/9.0D0)-E*t8*t31*t41*t54*(1.0D0/1.8D1))
      A0(2,2) = -t43*(t57+t67-t46*t66-E*t31*t40*t41*t50*(1.0D0/1.8D1)+E*t31*t40*t41*t52*(1.0D0/9.0D0)-E*t31*t40*t41*t54*(1.0D0/1.8D1))+1.0D0
      A0(2,3) = -t43*(t63+t76-t38*t69-E*t31*t41*t42*t50*(1.0D0/1.8D1)+E*t31*t41*t42*t52*(1.0D0/9.0D0)-E*t31*t41*t42*t54*(1.0D0/1.8D1))
      A0(2,4) = t43*(t70+t79+s12*t35*t40*t46*(3.0D0/2.0D0)-E*s12*t31*t41*t50*(1.0D0/3.0D0)+E*s12*t31*t41*t52*(2.0D0/3.0D0)-E*s12*t31*t41*t54*(1.0D0/3.0D0))
      A0(2,5) = t43*(t71+t81+s23*t35*t40*t46*(3.0D0/2.0D0)-E*s23*t31*t41*t50*(1.0D0/3.0D0)+E*s23*t31*t41*t52*(2.0D0/3.0D0)-E*s23*t31*t41*t54*(1.0D0/3.0D0))
      A0(2,6) = t43*(t72+t83+s13*t35*t40*t46*(3.0D0/2.0D0)-E*s13*t31*t41*t50*(1.0D0/3.0D0)+E*s13*t31*t41*t52*(2.0D0/3.0D0)-E*s13*t31*t41*t54*(1.0D0/3.0D0))
      A0(2,7) = -t38*t50-t38*t54-t46*t52
      A0(3,1) = -t43*(t67+t74-t38*t62-E*t8*t31*t41*t50*(1.0D0/1.8D1)-E*t8*t31*t41*t52*(1.0D0/1.8D1)+E*t8*t31*t41*t54*(1.0D0/9.0D0))
      A0(3,2) = -t43*(t67+t76-t38*t66-E*t31*t40*t41*t50*(1.0D0/1.8D1)-E*t31*t40*t41*t52*(1.0D0/1.8D1)+E*t31*t40*t41*t54*(1.0D0/9.0D0))
      A0(3,3) = -t43*(t57+t63-t46*t69-E*t31*t41*t42*t50*(1.0D0/1.8D1)-E*t31*t41*t42*t52*(1.0D0/1.8D1)+E*t31*t41*t42*t54*(1.0D0/9.0D0))+1.0D0
      A0(3,4) = t43*(t78+t79+s12*t35*t42*t46*(3.0D0/2.0D0)-E*s12*t31*t41*t50*(1.0D0/3.0D0)-E*s12*t31*t41*t52*(1.0D0/3.0D0)+E*s12*t31*t41*t54*(2.0D0/3.0D0))
      A0(3,5) = t43*(t80+t81+s23*t35*t42*t46*(3.0D0/2.0D0)-E*s23*t31*t41*t50*(1.0D0/3.0D0)-E*s23*t31*t41*t52*(1.0D0/3.0D0)+E*s23*t31*t41*t54*(2.0D0/3.0D0))
      A0(3,6) = t43*(t82+t83+s13*t35*t42*t46*(3.0D0/2.0D0)-E*s13*t31*t41*t50*(1.0D0/3.0D0)-E*s13*t31*t41*t52*(1.0D0/3.0D0)+E*s13*t31*t41*t54*(2.0D0/3.0D0))
      A0(3,7) = -t38*t50-t38*t52-t46*t54
      A0(4,1) = E*s12*t8*t33*t35*t43*(-1.0D0/2.0D0)-E*t8*t31*t41*t43*t84*(1.0D0/1.2D1)
      A0(4,2) = E*s12*t33*t35*t40*t43*(-1.0D0/2.0D0)-E*t31*t40*t41*t43*t84*(1.0D0/1.2D1)
      A0(4,3) = E*s12*t33*t35*t42*t43*(-1.0D0/2.0D0)-E*t31*t41*t42*t43*t84*(1.0D0/1.2D1)
      A0(4,4) = E*t33*t43*(t89-t18*t35*9.0D0)*(-1.0D0/3.0D0)+E*s12*t31*t41*t43*t84*(1.0D0/2.0D0)+1.0D0
      A0(4,5) = t88+E*s23*t31*t41*t43*t84*(1.0D0/2.0D0)
      A0(4,6) = t92+E*s13*t31*t41*t43*t84*(1.0D0/2.0D0)
      A0(4,7) = E*t33*t84*(1.0D0/3.0D0)
      A0(5,1) = E*s23*t8*t33*t35*t43*(-1.0D0/2.0D0)-E*t8*t31*t41*t43*t86*(1.0D0/1.2D1)
      A0(5,2) = E*s23*t33*t35*t40*t43*(-1.0D0/2.0D0)-E*t31*t40*t41*t43*t86*(1.0D0/1.2D1)
      A0(5,3) = E*s23*t33*t35*t42*t43*(-1.0D0/2.0D0)-E*t31*t41*t42*t43*t86*(1.0D0/1.2D1)
      A0(5,4) = t88+E*s12*t31*t41*t43*t86*(1.0D0/2.0D0)
      A0(5,5) = E*t33*t43*(t89-t22*t35*9.0D0)*(-1.0D0/3.0D0)+E*s23*t31*t41*t43*t86*(1.0D0/2.0D0)+1.0D0
      A0(5,6) = t93+E*s13*t31*t41*t43*t86*(1.0D0/2.0D0)
      A0(5,7) = E*t33*t86*(1.0D0/3.0D0)
      A0(6,1) = E*s13*t8*t33*t35*t43*(-1.0D0/2.0D0)-E*t8*t31*t41*t43*t90*(1.0D0/1.2D1)
      A0(6,2) = E*s13*t33*t35*t40*t43*(-1.0D0/2.0D0)-E*t31*t40*t41*t43*t90*(1.0D0/1.2D1)
      A0(6,3) = E*s13*t33*t35*t42*t43*(-1.0D0/2.0D0)-E*t31*t41*t42*t43*t90*(1.0D0/1.2D1)
      A0(6,4) = t92+E*s12*t31*t41*t43*t90*(1.0D0/2.0D0)
      A0(6,5) = t93+E*s23*t31*t41*t43*t90*(1.0D0/2.0D0)
      A0(6,6) = E*t33*t43*(t89-t20*t35*9.0D0)*(-1.0D0/3.0D0)+E*s13*t31*t41*t43*t90*(1.0D0/2.0D0)+1.0D0
      A0(6,7) = E*t33*t90*(1.0D0/3.0D0)
      A0(7,1) = t8*t41*(-1.0D0/2.0D0)
      A0(7,2) = t40*t41*(-1.0D0/2.0D0)
      A0(7,3) = t41*t42*(-1.0D0/2.0D0)
      A0(7,4) = s12*t41*3.0D0
      A0(7,5) = s23*t41*3.0D0
      A0(7,6) = s13*t41*3.0D0
 end subroutine jacobian
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
subroutine IDTasker(this, IDTask, IsUndr, &
         iStep, iTer, iEl, Int, &
         X, Y, Z, &
         Time0, dTime, &
         Props, Sig0, Swp0, StVar0,&
         dEps, D, BulkW, &
         Sig, Swp, StVar, ipl, &
         nStat,NonSym, iStrsDep, &
         iTimeDep, iTang, iAbort, &
        iPrjDir, iPrjLen, IO)
!-------------------------------------------------------------------------------
! Import local scripts and local modules
 ! use toolBox
 use PLAXISroutines
 use precision
 use standardVars
 use nonLinearIsoVonMisesVars
 use toolBox
!-------------------------------------------------------------------------------
! PLAXIS - Dictionary
!-------------------------------------------------------------------------------
!  Depending on IDTask, 1 : Initialize state variables
!                       2 : calculate stresses
!                       3 : calculate material stiffness matrix
!                       4 : return number of state variables
!                       5 : inquire matrix properties
!                           return switch for non-symmetric D-matrix
!                           stress/time dependent matrix
!                       6 : calculate elastic material stiffness matrix
!-------------------------------------------------------------------------------
Implicit Double Precision (A-H, O-Z)
class(nonLinearIsoVonMises) :: this
! Declare variable types
!-------------------------------------------------------------------------------
 Character(len=255) :: PrjDir
 Character(len=*)   :: iPrjDir
 INTEGER :: IDTask, IsUndr, iStep, iTer, iEl, Int, iPrjLen
 INTEGER :: ipl,  nStat, NonSym, iStrsDep, iTimeDep, iTang, iAbort, IO
 real(kind=realDBL), DIMENSION(6,6) ::  D
 real(kind=realDBL), DIMENSION(6)   ::  Sig0, Sig, dEps
 real(kind=realDBL), DIMENSION(lenStateVars)  ::  StVar0, StVar
 real(kind=realDBL), DIMENSION(size(c))       ::  Props
 ! Local variables
 INTEGER ::  ne, nt, n, i, msf, iOpt, ios
 INTEGER :: Nekv
 real(kind=realDBL) :: dNorm, dEpsMax, dTimeMax
 real(kind=realDBL) :: paramA, paramB, paramC, devQ, deltaChi, dChi, tauZero
 real(kind=realDBL), DIMENSION(6)   ::  dAbsEps
 INTEGER, PARAMETER :: StVarPrevLen = 10
 real(kind=realDBL), DIMENSION(StVarPrevLen)  ::  StVarPrev, StVarInitPrev
 Logical :: existence = .FALSE.
!-------------------------------------------------------------------------------
!******************************* MODEL DESCRIPTION *****************************
!-------------------------------------------------------------------------------
! PLAXIS - IDTask
!-------------------------------------------------------------------------------
! Initialize state variables
Task_1: if (IDTask == 1) then
    stateVars       =  0.0D0
    stateVars0      =  0.0D0
    stateVars(1:6)  = -1.0D0*Sig0(1:6)  ! Define compression as positive
    c               =  Props

    if (norm2(stateVars0) == 0) then
      call this%initialize()
      StVar0 = stateVars
    end if

    Call getPrjDir( iPrjDir, iPrjLen, PrjDir)
    Call checkIfCyclingCalculate ( PrjDir, existence )
    IF (existence) THEN
      ! TODO: Make fix so that cyclic model only update relevant statevars based on same model
      !Call readStVarInGP ( PrjDir, 10, StVarPrevMatr )
      Call storePrevStateVarsToFile( PrjDir )
      Call readStVarInGP ( PrjDir, StVarPrevLen, StVarPrev, 1)
     !  Call readStVarInGP ( PrjDir, StVarPrevLen, StVarInitPrev, 2)
      Call updateGPtracker( PrjDir, StVarPrevLen )
     StVar0(lenStateVars) = StVarPrev(StVarPrevLen-1)  ! Change statevariable
   END IF
 end if Task_1
!-------------------------------------------------------------------------------
! Calculate stresses
Task_2: if (IDTask == 2) then
    ipl            = 0
    stateVars      = StVar0
    stateVars(1:6) = -Sig0(1:6)
    stateVars0     = stateVars
    dt             = dTime
    dEpsilon       = 0.D0
    dEpsilon(1:6)  = -1.0D0*dEps(1:6)
    state          = StVar0(lenStateVars + 1)
    c              = Props

    If (IsUndr == 1) then
      Swp = Swp0 + BulkW*(dEps(1) + dEps(2) + dEps(3))
    else
      Swp = Swp0
    end if

    dNorm = norm2(dEpsilon)

    if (dnorm /= 0) then
      dAbsEps  = abs(dEpsilon)
      dEpsMax  = max(dAbsEps(1), dAbsEps(2), dAbsEps(3), dAbsEps(4), dAbsEps(5), dAbsEps(6))
      dTimeMax = dt
      ne       = nint(demax/1.0D-3)
      nt       = nint(dtmax/1.0D3)

      if (n == 0) then
        n = 1
      end if

      if (nt == 0) then
        nt = 1
      end if

      n   = max(ne, nt)
      dEpsilon  = dEpsilon/n*1.0D0
      dt  = dt/n*1.0D0

      do i = 1, n
        call this%stressupdate()     ! Note: Updates stateVars
        stateVars0  = stateVars
        ipl         = state
       end do
    end if

    ! Store updated values
    Sig(1:6)              = -1.0D0*stateVars(1:6)
    StVar                 = stateVars
    StVar(lenStateVars+1) = state


    !! ******** Calculate degradation ***********
      paramA = 2.0D0
      paramB = 1.0D0
      paramC = 10.0D0

     Call getPrjDir( iPrjDir, iPrjLen, PrjDir)
     Call checkIfCyclingCalculate ( PrjDir, existence )
     IF (existence) THEN
       Call getInitialStructure( prjDir, chi00 )
       deltaChi = abs(chi00 - chi)
       Time0  = Time0 + nint(dTime)
       Call this%update()
       devQ = this%dev(stateVars(1:6), 6)  ! Numerical helper
       Nekv = 1
       dChi = 0.0D0
         if (deltaChi > 0.01 .and. devQ > 5.0D-2*SuCyc ) Then
           Do while (dChi <= deltaChi)
             tauZero = 0.5D0*devQ * ((paramC*Nekv+paramB)/(paramC+paramB))**(paramA/paramC)
             dChi = dChi + alpha*paramA*0.5D0*devQ*(paramC+paramB)**(paramA/paramC)*(paramC*Nekv+paramB)**(-paramA/paramC-1.0D0)
             Nekv = Nekv + 1
             if (Nekv > 10000) then
               dChi=0
               exit
             end if
           end do
         end if
         If (nint(Time0) > 0 .and. Nekv<10000 .and. devQ > 5.0D-2*SuCyc) Then
           do i = Nekv, nint(Nekv+Time0), 1
             tauZero = 0.5D0*devQ * ((paramC*i+paramB)/(paramC+paramB))**(paramA/paramC)
             dChi = dChi + alpha*paramA*0.5D0*devQ*(paramC+paramB)**(paramA/paramC)*(paramC*i+paramB)**(-paramA/paramC-1.0D0)
           end do
         End if
         StVar(lenStateVars) = StVar(lenStateVars) - dChi
         if (StVar(lenStateVars) < 0.0D0) Then
         StVar(lenStateVars) = 0.01D0
         end if
     End IF
 end if Task_2
!-------------------------------------------------------------------------------
 ! Calculate material stiffness matrix
 Task_3: if (IDTask == 3) then
     stateVars = StVar0     ! Used if matrix is stress dependent
     call this%DMatrix(D)
     !D = 2.0d0*D     ! TODO: Why 2x ?
     msf = Sig0(15)  ! TODO: What does this do??
  end if Task_3
!-------------------------------------------------------------------------------
 ! Number of state variables
 Task_4: if (IDTask == 4) then
     nStat = lenStateVars + 1
  end if Task_4
!-------------------------------------------------------------------------------
 ! Inquire matrix properties
 Task_5: if (IDTask == 5) then
     NonSym      =   0   ! 1 for non-symmetric D-matrix
     iStrsDep    =   0   ! 1 for stress dependent D-matrix
     iTang       =   0   ! 1 for tanget D-matrix
     iTimeDep    =   0   ! 1 for time dependent D-matrix
  end if Task_5
!-------------------------------------------------------------------------------
 ! Calculate elastic material stiffness matrix
 Task_6: if (IDTask == 6) then
     stateVars = StVar0     ! Used if matrix is stress dependent
     call this%DMatrix(D)
     !D = 2.0d0*D     ! TODO: Why 2x ?
  end if Task_6
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
end subroutine IDTasker
end module nonLinearIsoVonMisesClass
