module modelInfoVars
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  Integer, parameter :: numberMods = 1
  Integer, parameter :: maxCharLen = 50

  Character (Len=maxCharLen), dimension(numberMods) :: names =    &
      ['Mod. CamClay with destr.']!, 'Nonlin Von Mises']
  Integer, dimension(numberMods) :: numberParam =                 &
      [size(matrParam1(:,1))]!, size(matrParam2(:,1))]
  Integer, dimension(numberMods) :: numberOfStateVars =           &
      [size(matrStVars1(:,1))]!, size(matrStVars2(:,1))]

  ! CamClay - Average
  Character (Len=maxCharLen), dimension(8,2) :: matrParam1 =              [&
      ['@j' , '@l^*#', '@k^*#', '@u#', 'OCR', '@x#', '@w#', '@c_0#'],          &
      ['deg',    '-',    '-',  '-',   '-',  '-',  '-',      '-']           ]
  Character (Len=maxCharLen), dimension(9,2) :: matrStVars1 =             					[&
      ['S11'   ,    'S22',    'S33',    'S12',    'S23',    'S13', '@l#',     'p_m#', '@c#'], &
      ['F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#',  '-', 'F/L^2#' ,    '-']  ]

	! ! NonLin. VonMises - Cyclic
  ! Character (Len=maxCharLen), dimension(3,2) :: matrParam2 =              [&
  !     ['S_u', 			'E', 	   '@a'],                          		   &
  !     ['F/L^2#',   'F/L^2#',  		'-']      			                   ]
  ! Character (Len=maxCharLen), dimension(8,2) :: matrStVars2 =             [&
  !     ['S11',       'S22',    'S33',    'S12',    'S23',    'S13', '@l#', '@c#'],  &
  !     ['F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#',  '-', '-']   ]

  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  contains
    function returnMatrParam(iMod, iParam, choice) result(output)
      integer :: iMod, iParam, choice
      Character (Len=255) :: output
      if (iMod==1 .and. (iParam<=size(matrParam1(:,1)))) then
        output = matrParam1(iParam, choice)
	    ! else if (iMod==2 .and. (iParam<=size(matrParam2(:,1)))) then
		  !   output = matrParam2(iParam, choice)
	    else
        output = '???'
      end if
    end function returnMatrParam
  !-----------------------------------------------------------------------------
    function returnStVarParam(iMod, iVar, choice) result(output)
      integer :: iMod, iVar, choice
      Character (Len=255) :: output
      if (iMod==1 .and. (iVar<=size(matrStVars1(:,1))) ) then
        output = matrStVars1(iVar, choice)
	    ! else if (iMod==2 .and. (iParam<=size(matrStVars2(:,1)))) then
		  !   output = matrStVars2(iParam, choice)
      else
        output = '???'
      end if
    end function returnStVarParam
  !-----------------------------------------------------------------------------
  subroutine selectModel(  IDTask, iMod, IsUndr, &
                           iStep, iTer, iEl, Int, &
                           X, Y, Z, &
                           Time0, dTime, &
                           Props, Sig0, Swp0, StVar0, &
                           dEps, D, BulkW, &
                           Sig, Swp, StVar, ipl, &
                           nStat, NonSym, iStrsDep, &
                           iTimeDep, iTang, iAbort, &
                           iPrjDir, iPrjLen, IO  )
    ! Import soil models
  use precision
  use ModifiedCamClayAverageClass
	! use nonLinearisoVonMisesClass

    Implicit Double Precision (A-H,O-Z)

    INTEGER :: IDTask, iMod, IsUndr, iStep, iTer, iEl, Int, iPrjLen
    INTEGER :: ipl,  nStat, NonSym, iStrsDep, iTimeDep, iTang, iAbort, IO
    real(kind=realDBL), DIMENSION(6,6) ::  D
    real(kind=realDBL), DIMENSION(6)   ::  Sig0, Sig, dEps
    real(kind=realDBL), DIMENSION(*)   ::  StVar0, StVar
    real(kind=realDBL), DIMENSION(*)   ::  Props
    Character(len=*) :: iPrjDir

    type(ModifiedCamClayAverage)   :: material_1
	! type(nonLinearisoVonMises)     :: material_2
      material_1 = ModifiedCamClayAverage()
	  ! material_2 = nonLinearisoVonMises()

    Select Case (iMod)
      Case(1)
          call material_1%IDTasker(  IDTask, IsUndr, &
                                   iStep, iTer, iEl, Int, &
                                   X, Y, Z, &
                                   Time0, dTime, &
                                   Props, Sig0, Swp0, StVar0, &
                                   dEps, D, BulkW, &
                                   Sig, Swp, StVar, ipl, &
                                   nStat, NonSym, iStrsDep, &
                                   iTimeDep, iTang, iAbort, &
                                   iPrjDir, iPrjLen, IO)

	  ! Case(2)
		!   call material_2%IDTasker(  IDTask, IsUndr, &
    !                                iStep, iTer, iEl, Int, &
    !                                X, Y, Z, &
    !                                Time0, dTime, &
    !                                Props, Sig0, Swp0, StVar0, &
    !                                dEps, D, BulkW, &
    !                                Sig, Swp, StVar, ipl, &
    !                                nStat, NonSym, iStrsDep, &
    !                                iTimeDep, iTang, iAbort, &
    !                                iPrjDir, iPrjLen, IO)
	  Case Default
        !Write(IO,*) 'invalid model number in UsrMod', iMod
        !Write(IO,*) 'IDTask: ',IDTask
        Stop 'invalid model number in UsrMod'
        iAbort=1
        Return
      end select
    end subroutine selectModel
  !-----------------------------------------------------------------------------
end module modelInfoVars
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
