! Exported:
!  Subroutine GetModelCount( nMod )
!  Subroutine GetModelName ( iMod , ModelName )
!  Subroutine GetParamCount( iMod , nParam )
!  Subroutine GetParamName ( iMod , iParam, ParamName )
!  Subroutine GetParamUnit ( iMod , iParam, Units )
!  Subroutine GetStateVarCount( iMod , nVar )
!  Subroutine GetStateVarName ( iMod , iVar, Name )
!  Subroutine GetStateVarUnit ( iMod , iVar, Unit )
!
! Local:
!  Subroutine GetParamAndUnit( iMod , iParam, ParamName, Units )
!  Subroutine GetStateVarNameAndUnit( iMod , iVar, Name, Unit )
!-----------------------------------------------------------------------------------------
!---------------------------------        Exported       ---------------------------------
!-----------------------------------------------------------------------------------------
Subroutine GetModelCount(nMod)
    use modelInfoVars, only: numberMods
    Integer :: nMod
    nMod = numberMods
    ! Return the maximum model number (iMod) in this DLL
    !DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetModelCount
end Subroutine GetModelCount
!-------------------------------------------------------------------------------
Subroutine GetModelName (iMod, ModelName)
! Return the name of the different models
    use modelInfoVars, only: numberMods, names
    Character (Len=255) :: ModelName, tName
    Integer             :: nMod, iMod
    nMod = numberMods
    !DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetModelName
    if (iMod>nMod) then
      tName = 'Not in DLL'
    else
      tName = names(iMod)
    end if
    LT = Len_Trim(tName)
    ModelName= Char(lt) // tName(1:Lt)
end Subroutine GetModelName
!-------------------------------------------------------------------------------
Subroutine GetParamCount (iMod, nParam )
    use modelInfoVars, only: numberMods, numberParam
    ! Return the number of parameters of the different models
    Integer :: nMod, iMod, nParam
    nMod = numberMods
    !DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetParamCount
    if (iMod>nMod) then
      nParam = 0
    else
      nParam = numberParam(iMod)
    end if
end Subroutine GetParamCount
!-------------------------------------------------------------------------------
subroutine GetParamName ( iMod, iParam, ParamName)
    ! Return the parameters name of the different models
    Character (Len=255) ParamName, Units
    !DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetParamName
    Call GetParamAndUnit( iMod, iParam, ParamName, Units)
end Subroutine GetParamName
!-------------------------------------------------------------------------------
subroutine GetParamUnit ( iMod, iParam, Units )
    ! Return the units of the different parameters of the different models
    Character (Len=255) ParamName, Units
    !DEC$ ATTRIBUTES DLLExport, StdCall, reference ::  GetParamUnit
    Call GetParamAndUnit ( iMod, iParam, ParamName, Units )
end Subroutine GetParamUnit
!-------------------------------------------------------------------------------
subroutine GetStateVarCount (iMod, nVar)
    use modelInfoVars, only: numberMods, numberOfStateVars
! Return the numbers of state variables of the different models
    Integer           :: nMod, iMod, nVar
    nMod = numberMods
    !DEC$ ATTRIBUTES dllexport, stdcall, reference :: GetStateVarCount
    if (iMod>nMod) then
      nVar = 0
    else
      nVar = numberOfStateVars(iMod)
    end if
end Subroutine GetStateVarCount
!-------------------------------------------------------------------------------
subroutine GetStateVarName (iMod, iVar, Name)
    ! Return the name of the different state variables of the different models
    Character(Len=255) Name, Unit
    Integer :: iVar
    !DEC$ ATTRIBUTES dllexport, stdcall, reference :: GetStateVarName
    Call GetStateVarNameAndUnit (iMod, iVar, Name, Unit)
end Subroutine GetStateVarName
!-------------------------------------------------------------------------------
subroutine GetStateVarUnit (iMod, iVar, Name)
    ! Return the units of the different state variables of the different models
    Character(Len=255) Name, Unit
    Integer :: iVar
    !DEC$ ATTRIBUTES dllexport, stdcall, reference :: GetStateVarUnit
    Call GetStateVarNameAndUnit( iMod , iVar, Name, Unit )
end Subroutine GetStateVarUnit
!-------------------------------------------------------------------------------
!---------------------------------          Local        -----------------------
!-------------------------------------------------------------------------------
subroutine GetParamAndUnit ( iMod, iParam, ParamName, Units )
    use modelInfoVars, only: numberMods, returnMatrParam
! Return the parameters name and units of the different models
!
! Units: use F for force unit
!            L for length unit
!            T for time unit
    Integer        :: nMod, iMod, iParam
    Character (Len=255) ParamName, Units, tName
    nMod = numberMods
    if (iMod>nMod) then
      ParamName = '???' ; Units = '???'
    else
      ParamName = returnMatrParam(iMod, iParam, 1)
      Units     = returnMatrParam(iMod, iParam, 2)
    end if
    tName    = ParamName
    LT       = Len_Trim(tName)
    ParamName= Char(lt) // tName(1:Lt)

    tName = Units
    LT    = Len_Trim(tName)
    Units = Char(lt) // tName(1:Lt)
end Subroutine GetParamAndUnit
!-------------------------------------------------------------------------------
subroutine GetStateVarNameAndUnit ( iMod, iVar, Name, Unit )
      use modelInfoVars, only: numberMods, returnStVarParam
! Return the name and unit of the different state variables of the different models
      Integer        :: nMod, iMod, iVar
      Character (Len=255) Name, Unit, tName
      nMod = numberMods
      if (iMod>nMod) then
        Name = '???' ; Unit = '???'
      else
        Name = returnStVarParam(iMod, iVar, 1)
        Unit = returnStVarParam(iMod, iVar, 2)
      end if
      tName    = Name
      LT       = Len_Trim(tName)
      Name     = Char(lt) // tName(1:Lt)

      tName    = Unit
      LT       = Len_Trim(tName)
      Unit     = Char(lt) // tName(1:Lt)
end Subroutine GetStateVarNameAndUnit
