module PLAXISroutines
!-------------------------------------------------------------------------------
! PLAXIS subroutines info:
!-------------------------------------------------------------------------------
! MZEROR    - Takes in real array and sets all entries to zero
! MZEROI    - Takes in an integer array with dimenion K and sets all entries to zero
! SETRVAL   - Fills a real array R of size K with values V
! SETIVAL   - Fills an integer array I of size K with values IV
! COPYIVEC  - Copies an integer array I1 with Dimension K to I2
! COPYRVEC  - Copies a Double array R1 of size K to R2
! MulVec
! MatVec
! AddVec
! DInProd
! MatMat
! MatMatSq
! WriVal
! WriIVl
! WriIVc
! WriVec
! WriVec5
! WriMat
! MatInvPiv
! PrnSig
! Eig_3
! Eig_3a
! CarSig
!-------------------------------------------------------------------------------
! PLAXIS Function info:
!-------------------------------------------------------------------------------
! IS0ARR
! IS0IARR
! LEqual
! CrossProd
! ArcSin
! setveclen
!-------------------------------------------------------------------------------
contains
!-------------------------------------------------------------------------------
    Subroutine MZEROR(R,K)
! Purpose:
!    Takes in a real array R with dimension K and sets all entries to zero
      Implicit Double Precision (A-H,O-Z)
      Dimension R(*)

      Do J=1,K
        R(J) = 0.0D0
      End Do
    End Subroutine MZEROR
!-------------------------------------------------------------------------------
    Subroutine MZEROI(I,K)
! Purpose:
!   Takes in an integer array with dimenion K and sets all entries to zero
      Dimension I(*)

      Do J=1,K
        I(J)=0
      End Do

    End Subroutine MZEROI
!-------------------------------------------------------------------------------
    Subroutine SETRVAL(R,K,V)
! Purpose:
!   Fills a real array R of size K with values V
      Implicit Double Precision (A-H,O-Z)
      Dimension R(*)

      Do J=1,K
        R(J)=V
      End Do

    End Subroutine SETRVAL
!-------------------------------------------------------------------------------
    Subroutine SETIVAL(I,K,IV)
! Purpose:
!   Fills an integer array I of size K with values IV
      Implicit Double Precision (A-H,O-Z)
      Dimension I(*)

      Do J=1,K
        I(J)=IV
      End Do

    End Subroutine SETIVAL
!-------------------------------------------------------------------------------
    Subroutine COPYIVEC(I1,I2,K)
! Purpose:
!   Copies an integer array I1 with Dimension K to I2
      Implicit Double Precision (A-H,O-Z)
      Dimension I1(*),I2(*)

      Do  J=1,K
        I2(J)=I1(J)
      End Do

    End Subroutine COPYIVEC
!-------------------------------------------------------------------------------
    Subroutine COPYRVEC(R1,R2,K)
! Purpose:
!   Copies a Double array R1 of size K to R2
      Implicit Double Precision (A-H,O-Z)
      Dimension R1(*),R2(*)

      Do J=1,K
        R2(J)=R1(J)
      End Do

    End Subroutine COPYRVEC
!-------------------------------------------------------------------------------
    Logical Function IS0ARR(A,N)
!C    Function :  To check whether a real array contains only zero values.
!C                When an array contains only zero's is might not need to be
!C                written to the XXX file.
!C                exit Function when first non-zero value occured or when
!C                all elements are checked and are zero.
!C
!C    Input:  A : array to be checked
!C            N : number of elements in array that should be checked
!C
!C    Output : .TRUE.  when all elements are 0
!C             .FALSE. when at least one element is not zero
!C
!C    Called by :  Subroutine TOBXX
      Implicit Double Precision (A-H,O-Z)
      Dimension A(*)
      Is0Arr=.False.
      Do I=1,N
        If ( A(I) .Ne. 0 ) Return
      End Do
      Is0Arr=.True.
      Return
    End Function IS0ARR
!-------------------------------------------------------------------------------
      Logical Function IS0IARR(IARR,N)
!C    Function :  To check whether a integer array contains only zero values.
!C                Similar to IS0ARR
!C
!C    Input:  IARR : array to be checked
!C            N    : number of elements in array that should be checked
!C
!C    Output : .TRUE.  when all elements are 0
!C             .FALSE. when at least one element is not zero
!C
!C    Called by :  Subroutine TOBXX
!-------------------------------------------------------------------------------
      Implicit Double Precision (A-H,O-Z)
      Dimension IARR(*)

      Is0IArr=.False.
      Do I=1,N
        If ( IARR(I) .Ne. 0 ) Return
      End Do
      Is0IArr=.True.
      Return
      End  Function IS0IARR
!-------------------------------------------------------------------------------
      Subroutine MulVec(V,F,K)
!C     Function: To multiply a real vector V with dimension K by F
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION V(*)

      Do J=1,K
        V(J)=F*V(J)
      End Do

      Return
      End Subroutine MulVec     ! Subroutine Mulvec
!-------------------------------------------------------------------------------
      Subroutine MatVec(xMat,IM,Vec,N,VecR)
!C     Calculate VecR = xMat*Vec
!C
!C I   xMat  : (Square) Matrix (IM,*)
!C I   Vec   : Vector
!C I   N     : Number of rows/colums
!C O   VecR  : Resulting vector
      Implicit Double Precision (A-H,O-Z)
      Dimension xMat(IM,*),Vec(*),VecR(*)
        Do I=1,N
          X=0
          Do J=1,N
            X=X+xMat(I,J)*Vec(J)
          End Do
          VecR(I)=X
        End Do
      Return
      End Subroutine MatVec    ! Subroutine MatVec
!-------------------------------------------------------------------------------
      Subroutine AddVec(Vec1,Vec2,R1,R2,N,VecR)
! Purpose:
!   Calculate VecR() = R1*Vec1()+R2*Vec2()
!
! I   Vec1,
! I   Vec2  : Vectors
! I   R1,R2 : Multipliers
! I   N     : Number of rows
! O   VecR  : Resulting vector
      Implicit Double Precision (A-H,O-Z)
      Dimension Vec1(*),Vec2(*),VecR(*)
        Do I=1,N
          X=R1*Vec1(I)+R2*Vec2(I)
          VecR(I)=X
        End Do
      Return
      End Subroutine AddVec    ! Subroutine AddVec
!-------------------------------------------------------------------------------
      Double Precision Function DInProd(A,B,N)
!C     Returns the Inproduct of two vectors
!C
!C I   A,B  : Two vectors
!C I   N    : Used length of vectors
      Implicit Double Precision (A-H,O-Z)
      Dimension A(*),B(*)
      X = 0
      Do I=1,N
        X = X + A(I)*B(I)
      End Do
      DInProd = X
      Return
      End Function DInProd    ! Function DInProd
!-------------------------------------------------------------------------------
      Subroutine MatMat(xMat1,Id1,xMat2,Id2,nR1,nC2,nC1,xMatR,IdR)
!C     Calculate xMatR = xMat1*xMat2
!C
!C I   xMat1 : Matrix (Id1,*)
!C I   xMat2 : Matrix (Id2,*)
!C I   nR1   : Number of rows in resulting matrix    (= No rows in xMat1)
!C I   nC2   : Number of columns in resulting matrix (= No cols in xMat2)
!C I   nC1   : Number of columns in matrix xMat1
!C             = Number  rows    in matrix xMat2
!C O   xMatR : Resulting matrix (IdR,*)
      Implicit Double Precision (A-H,O-Z)
      Dimension xMat1(Id1,*),xMat2(Id2,*),xMatR(IdR,*)

      Do I=1,nR1
        Do J=1,nC2
          X=0
          Do K=1,nC1
            X=X+xMat1(I,K)*xMat2(K,J)
          End Do
          xMatR(I,J)=X
        End Do
      End Do

      Return
      End Subroutine MatMat     ! Subroutine MatMat
!-------------------------------------------------------------------------------
      Subroutine MatMatSq(n, xMat1, xMat2, xMatR)
!C     Calculate xMatR = xMat1*xMat2 for square matrices, size n
!C
!C I   n     : Dimension of matrices
!C I   xMat1 : Matrix (n,*)
!C I   xMat2 : Matrix (n,*)
!C O   xMatR : Resulting matrix (n,*)
      Implicit Double Precision (A-H,O-Z)
      Dimension xMat1(n,*),xMat2(n,*),xMatR(n,*)
        Do I=1,n
          Do J=1,n
            X=0
            Do K=1,n
              X=X+xMat1(I,K)*xMat2(K,J)
            End Do
            xMatR(I,J)=X
          End Do
        End Do
      Return
      End Subroutine MatMatSq     ! Subroutine MatMatSq
!-------------------------------------------------------------------------------
      Subroutine WriVal ( io, C , V )
!C Write (Double) value to file unit io (when io>0)
      Implicit Double Precision (A-H,O-Z)
      Character C*(*)
      If (io.Le.0) Return
      Write(io,*) C,V
    1 Format( A,3x, 1x,1p,e12.5)
      Return
      End Subroutine WriVal
!-------------------------------------------------------------------------------
      Subroutine WriIVl ( io, C , I )
!C Write (integer) value to file unit io (when io>0)
      Implicit Double Precision (A-H,O-Z)
      Character C*(*)

      If (io.Le.0) Return

      Write(io,*) C,I
    1 Format( A,3x, 1x,I6)
      Return
      End Subroutine WriIVl
!-------------------------------------------------------------------------------
      Subroutine WriIVc ( io, C , iV , n )
!C Write (integer) vector to file unit io (when io>0)
      Character C*(*)
      Dimension iV(*)
      If (io.Le.0) Return
      Write(io,*) C
      Write(io,1) (iv(i),i=1,n)
    1 Format( ( 2(3x,5i4) ) )
      Return
      End Subroutine WriIVc
!-------------------------------------------------------------------------------
      Subroutine WriVec ( io, C , V , n )
!C Write (Double) vector to file unit io (when io>0)
!C 6 values per line
      Implicit Double Precision (A-H,O-Z)
      Character C*(*)
      Dimension V(*)

      If (io.Le.0) Return

      If (Len_Trim(C).Le.6) Then
        Write(io,2) C,( V(i),i=1,n)
      Else
        Write(io,*) C
        Write(io,1) ( V(i),i=1,n)
      End If
    1 Format( ( 2(1x, 3(1x,1p,e10.3) ) ) )
    2 Format( A, ( T7, 2(1x, 3(1x,1p,e10.3) ) ) )
      Return
      End Subroutine WriVec
!-------------------------------------------------------------------------------
      Subroutine WriVec5( io, C , V , n )
!C Write (Double) vector to file unit io (when io>0)
!C 5 values per line
      Implicit Double Precision (A-H,O-Z)
      Character C*(*)
      Dimension V(*)

      If (io.Le.0) Return

      Write(io,*) C
      Write(io,1) ( V(i),i=1,n)
    1 Format( 5(1x,1p,e12.5) )
      Return
      End Subroutine WriVec5
!-------------------------------------------------------------------------------
      Subroutine WriMat ( io, C , V , nd, nr, nc )
!C Write (Double) matrix to file unit io (when io>0)
!C 6 values per line
      Implicit Double Precision (A-H,O-Z)
      Character C*(*)
      Dimension V(nd,*)

      If (io.Le.0) Return

      Write(io,*) C
      Do j=1,nr
        Write(io,1) j,( V(j,i),i=1,nc)
      End Do
    1 Format(i4, (  T7,2(1x, 3(1x,1p,e10.3) ) ) )
      Return
      End Subroutine WriMat
!-------------------------------------------------------------------------------
      Subroutine MatInvPiv(Aorig,B,N)
      Implicit Double Precision (A-H,O-Z)
      Dimension Aorig(n,*), B(n,*),A(:,:)
      Allocatable :: A
      Allocate ( A(n,n) ) ! No error checking !!
      Call CopyRVec(AOrig, A, n*n )
      Call MZeroR(B,n*n)
      Do i=1,n
        B(i,i) = 1d0
      End Do
      Do I=1,n
        T=A(I,I)
        iPiv=i
        Do j=i+1,n
          If ( Abs(A(j,i)) .Gt. Abs(A(iPiv,i))  ) iPiv=j
        End Do
        If (iPiv.Ne.i) Then
          Do j=1,n
            x         = A( i  ,j)
            A( i  ,j) = A(iPiv,j)
            A(iPiv,j) = x
            x         = B( i  ,j)
            B( i  ,j) = B(iPiv,j)
            B(iPiv,j) = x
          End Do
          T=A(I,I)
        End If
        Do J=1,n
          A(I,J)=A(I,J)/T
          B(I,J)=B(I,J)/T
        End Do
        Do K=1,n
          If (K.Ne.I) Then
            T=A(K,I)
            Do J=1,n
              A(K,J)=A(K,J)-T*A(I,J)
              B(K,J)=B(K,J)-T*B(I,J)
            End Do
          End If
        End Do
      End Do
      DeAllocate ( A  )
      Return
      End Subroutine MatInvPiv ! MatinvPiv
!-------------------------------------------------------------------------------
      Subroutine PrnSig(IOpt,S,xN1,xN2,xN3,S1,S2,S3,P,Q)
      Implicit Double Precision (A-H,O-Z)
      Dimension S(*),xN1(*),xN2(*),xN3(*)

      If (iOpt.Eq.1) Then
        Call Eig_3(0,S,xN1,xN2,xN3,S1,S2,S3,P,Q) ! with Eigenvectors
      Else
        Call Eig_3a(0,S,S1,S2,S3,P,Q) ! no Eigenvectors
      End If
      Return
      End Subroutine PrnSig
!-------------------------------------------------------------------------------
      Subroutine Eig_3(iOpt,St,xN1,xN2,xN3,S1,S2,S3,P,Q)
      Implicit Double Precision (A-H,O-Z)
      Dimension St(6),A(3,3),V(3,3),xN1(3),xN2(3),xN3(3)
      !
      ! Get Eigenvalues/Eigenvectors for 3*3 matrix
      ! Wim Bomhof 15/11/'01
      ! PGB : adaption to Principal stress calculation
      !
      ! Applied on principal stresses, directions
      ! Stress vector St(): XX, YY, ZZ, XY, YZ, ZX
      !
      A(1,1) = St(1) ! xx
      A(1,2) = St(4) ! xy = yx
      A(1,3) = St(6) ! zx = xz

      A(2,1) = St(4) ! xy = yx
      A(2,2) = St(2) ! yy
      A(2,3) = St(5) ! zy = yz

      A(3,1) = St(6) ! zx = xz
      A(3,2) = St(5) ! zy = yz
      A(3,3) = St(3) ! zz

      ! Set V to unity matrix
      V(1,1) = 1
      V(2,1) = 0
      V(3,1) = 0

      V(1,2) = 0
      V(2,2) = 1
      V(3,2) = 0

      V(1,3) = 0
      V(2,3) = 0
      V(3,3) = 1


      abs_max_s=0.0
      Do i=1,3
        Do j=1,3
          if (abs(a(i,j)) .Gt. abs_max_s) abs_max_s=abs(a(i,j))
        End Do
      End Do
      Tol = 1d-20 * abs_max_s
      it = 0
      itmax = 50
      Do While ( it.Lt.itMax .And.(abs(a(1,2))+abs(a(2,3))+abs(a(1,3))) .Gt. Tol )
        it=it+1
        Do k=1,3
          If (k .Eq. 1) Then
            ip=1
            iq=2
          Else If (k .Eq.2) Then
            ip=2
            iq=3
          Else
            ip=1
            iq=3
          End If
          If (a(ip,iq) .Ne. 0.0) Then
            tau=(a(iq,iq)-a(ip,ip))/(2.0*a(ip,iq))
            If (tau .Ge.0.0) Then
              sign_tau=1.0
            Else
              sign_tau=-1.0
            End If
            t=sign_tau/(abs(tau)+sqrt(1.0+tau*tau))
            c=1.0/sqrt(1.0+t*t)
            s=t*c
            a1p=c*a(1,ip)-s*a(1,iq)
            a2p=c*a(2,ip)-s*a(2,iq)
            a3p=c*a(3,ip)-s*a(3,iq)
            a(1,iq)=s*a(1,ip)+c*a(1,iq)
            a(2,iq)=s*a(2,ip)+c*a(2,iq)
            a(3,iq)=s*a(3,ip)+c*a(3,iq)
            a(1,ip)=a1p
            a(2,ip)=a2p
            a(3,ip)=a3p

            v1p=c*v(1,ip)-s*v(1,iq)
            v2p=c*v(2,ip)-s*v(2,iq)
            v3p=c*v(3,ip)-s*v(3,iq)
            v(1,iq)=s*v(1,ip)+c*v(1,iq)
            v(2,iq)=s*v(2,ip)+c*v(2,iq)
            v(3,iq)=s*v(3,ip)+c*v(3,iq)
            v(1,ip)=v1p
            v(2,ip)=v2p
            v(3,ip)=v3p

            ap1=c*a(ip,1)-s*a(iq,1)
            ap2=c*a(ip,2)-s*a(iq,2)
            ap3=c*a(ip,3)-s*a(iq,3)
            a(iq,1)=s*a(ip,1)+c*a(iq,1)
            a(iq,2)=s*a(ip,2)+c*a(iq,2)
            a(iq,3)=s*a(ip,3)+c*a(iq,3)
            a(ip,1)=ap1
            a(ip,2)=ap2
            a(ip,3)=ap3
          End If ! a(ip,iq)<>0
        End Do ! k
      End Do ! While
      ! principal values on diagonal of a
      S1 = a(1,1)
      S2 = a(2,2)
      S3 = a(3,3)
      ! Derived invariants
      P = (S1+S2+S3)/3
      Q = Sqrt( ( (S1-S2)**2 + (S2-S3)**2 + (S3-S1)**2 ) / 2 )

      ! Sort eigenvalues S1 <= S2 <= S3
      is1 = 1
      is2 = 2
      is3 = 3
      if (s1.Gt.s2) Then
        t   = s2
        s2  = s1
        s1  = t
        it  = is2
        is2 = is1
        is1 = it
      End If
      if (s2.Gt.s3) Then
        t   = s3
        s3  = s2
        s2  = t
        it  = is3
        is3 = is2
        is2 = it
      End If
      if (s1.Gt.s2) Then
        t   = s2
        s2  = s1
        s1  = t
        it  = is2
        is2 = is1
        is1 = it
      End If
      Do i=1,3
        xN1(i) = v(i,is1) ! first  column
        xN2(i) = v(i,is2) ! second column
        xN3(i) = v(i,is3) ! third  column
      End Do
      Return
      End Subroutine Eig_3 ! Eig_3

      Subroutine Eig_3a(iOpt,St,S1,S2,S3,P,Q) ! xN1,xN2,xN3,
      Implicit Double Precision (A-H,O-Z)
      Dimension St(6),A(3,3)   !  V(3,3),xN1(3),xN2(3),xN3(3)
      !
      ! Get Eigenvalues ( no Eigenvectors) for 3*3 matrix
      ! Wim Bomhof 15/11/'01
      !
      ! Applied on principal stresses, directions
      ! Stress vector XX, YY, ZZ, XY, YZ, ZX
      !
      A(1,1) = St(1) ! xx
      A(1,2) = St(4) ! xy = yx
      A(1,3) = St(6) ! zx = xz

      A(2,1) = St(4) ! xy = yx
      A(2,2) = St(2) ! yy
      A(2,3) = St(5) ! zy = yz

      A(3,1) = St(6) ! zx = xz
      A(3,2) = St(5) ! zy = yz
      A(3,3) = St(3) ! zz

      abs_max_s=0.0
      Do i=1,3
        Do j=1,3
          if (abs(a(i,j)) .Gt. abs_max_s) abs_max_s=abs(a(i,j))
        End Do
      End Do
      Tol = 1d-20 * abs_max_s
      If (iOpt.Eq.1) Tol = 1d-50*abs_max_s
      it=0
      itmax = 50
      Do While ( it.lt.itmax .And.(abs(a(1,2))+abs(a(2,3))+abs(a(1,3))) .Gt. Tol )

        it=it+1
        Do k=1,3
          If (k .Eq. 1) Then
            ip=1
            iq=2
          Else If (k .Eq.2) Then
            ip=2
            iq=3
          Else
            ip=1
            iq=3
          End If
          If (a(ip,iq) .Ne. 0.0) Then         ! ongelijk nul ?
            tau=(a(iq,iq)-a(ip,ip))/(2.0*a(ip,iq))
            If (tau .Ge.0.0) Then
              sign_tau=1.0
            Else
              sign_tau=-1.0
            End If
            t=sign_tau/(abs(tau)+sqrt(1.0+tau*tau))
            c=1.0/sqrt(1.0+t*t)
            s=t*c
            a1p=c*a(1,ip)-s*a(1,iq)
            a2p=c*a(2,ip)-s*a(2,iq)
            a3p=c*a(3,ip)-s*a(3,iq)
            a(1,iq)=s*a(1,ip)+c*a(1,iq)
            a(2,iq)=s*a(2,ip)+c*a(2,iq)
            a(3,iq)=s*a(3,ip)+c*a(3,iq)
            a(1,ip)=a1p
            a(2,ip)=a2p
            a(3,ip)=a3p

            ap1=c*a(ip,1)-s*a(iq,1)
            ap2=c*a(ip,2)-s*a(iq,2)
            ap3=c*a(ip,3)-s*a(iq,3)
            a(iq,1)=s*a(ip,1)+c*a(iq,1)
            a(iq,2)=s*a(ip,2)+c*a(iq,2)
            a(iq,3)=s*a(ip,3)+c*a(iq,3)
            a(ip,1)=ap1
            a(ip,2)=ap2
            a(ip,3)=ap3
          End If ! a(ip,iq)<>0
        End Do ! k
      End Do ! While
      ! principal values on diagonal of a
      S1 = a(1,1)
      S2 = a(2,2)
      S3 = a(3,3)
      ! Derived invariants
      P = (S1+S2+S3)/3
      Q = Sqrt( ( (S1-S2)**2 + (S2-S3)**2 + (S3-S1)**2 ) / 2 )

      if (s1.Gt.s2) Then
        t   = s2
        s2  = s1
        s1  = t
      End If
      if (s2.Gt.s3) Then
        t   = s3
        s3  = s2
        s2  = t
      End If
      if (s1.Gt.s2) Then
        t   = s2
        s2  = s1
        s1  = t
      End If
      Return
      End Subroutine Eig_3a ! Eig_3a
!-------------------------------------------------------------------------------
      Logical Function LEqual(A,B,Eps)
!C     Returns .TRUE.  when two real values are (almost) equal,
!C             .FALSE. otherwise
!C
!C I   A,B  : Two real values to be compared
!C I   Eps  : Toleration (Magnitude ~= 1E-5)
      Implicit Double Precision (A-H,O-Z)
      LEqual =.True.
      If (A .Eq. B) Return
      If (DAbs(A-B) .LT. 0.5D0*Eps*( DAbs(A) + DAbs(B) + Eps ) )Return
      LEqual =.False.
      Return
      End Function LEqual     ! function LEqual
!-------------------------------------------------------------------------------
      Subroutine CrossProd(xN1,xN2,xN3)
!C     Returns cross product of xN1 and xN2
!C
!C I   xN1,xN2 : Two basic vectors
!C O   xN3     : Resulting vector
      Implicit Double Precision (A-H,O-Z)
      Dimension xN1(*),xN2(*),xN3(*)

      xN3(1) = xN1(2)*xN2(3) - xN1(3)*xN2(2)
      xN3(2) = xN1(3)*xN2(1) - xN1(1)*xN2(3)
      xN3(3) = xN1(1)*xN2(2) - xN1(2)*xN2(1)

      Return
      End Subroutine CrossProd     ! Subroutine CrossProd
!-------------------------------------------------------------------------------
      Double Precision Function ArcSin(X,ie)
!C     Returns the Arc Sine of X
!C
!C I   X : Input value
!C
!C     Note : In stead of using default routine DASIN we use this one
!C            because ³X³ can be slightly beyond 1 and this will give
!C            a RTE using DASIN(X)
      Implicit Double Precision (A-H,O-Z)

      Ie=0
      S = (1-X*X)
!      If (S .Lt. -1E-10) Ie=1
!      If (S .Lt. -1E-10) Write(*,1) X,S
!      If (S .Lt. -1E-10) Write(2,1) X,S
    1 Format(' ArcSin(',1x,1p,e13.5e3,') , S =',1x,1p,e13.5e3)
      If (S.LT.0) S = 0
      S = DSQRT(S)
      ArcSin = DATan2(X,S)
      Return
      End Function ArcSin     ! function ArcSin
!-------------------------------------------------------------------------------
      Subroutine CarSig(S1,S2,S3,xN1,xN2,xN3,SNew)
!C     Returns the Cartesian stresses using the principal stresses S1..S3
!C     and the principal directions
!C
!C I   S1..S3   : Principal stresses
!C I   xN1..xN3 : Principal directions (xNi for Si)
      Implicit Double Precision (A-H,O-Z)
      Dimension xN1(*),xN2(*),xN3(*),SNew(*)
      Dimension SM(3,3),T(3,3),TT(3,3),STT(3,3)
!C**** Fill transformation (rotation) matrix
      Do I=1,3
        T(I,1) = xN1(I)
        T(I,2) = xN2(I)
        T(I,3) = xN3(I)
        TT(1,I) = T(I,1)
        TT(2,I) = T(I,2)
        TT(3,I) = T(I,3)
      End Do
!      Call MatTranspose(T,3,TT,3,3,3)

      Call MZeroR(SM,9)
      SM(1,1) = S1
      SM(2,2) = S2
      SM(3,3) = S3
!C
!C**** SMnew = T*SM*TT
!C
      Call MatMat(SM ,3,  TT,3 , 3,3,3 ,STT,3)
      Call MatMat( T ,3, STT,3 , 3,3,3 ,SM ,3)
!     Call MatMatSq(3, SM,  TT, STT )   ! STT = SM*TT
!     Call MatMatSq(3,  T, STT, SM  )   ! SM  =  T*STT
!C
!C**** Extract cartesian stress vector from stress matrix
!C
      Do I=1,3
        SNew(I) = SM(I,I)
      End Do
      SNew(4) = SM(2,1)
      SNew(5) = SM(3,2)
      SNew(6) = SM(3,1)

      Return
      End Subroutine CarSig     ! Subroutine CarSig
!-------------------------------------------------------------------------------
      subroutine setveclen(xn,n,xl)
      Implicit Double Precision (A-H,O-Z)
      Dimension xN(*)
      x=0
      do i=1,n
        x=x+xn(i)**2
      end do
      if (x.Ne.0) Then
        f=xl/sqrt(x)
        do i=1,3
          xn(i)=xn(i)*f
        end do
      end if
      return
      end subroutine setveclen ! setveclen
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
end module PLAXISroutines
