! Purpose:
!    The program code creates a userdefined soil model which can be used together with PLAXIS.
!    This file acts as an interface between the constitutive soil models and PLAXIS.
!    The spesific models is contained in other files.
!
! Record of revisions:
!       Date        Programmer          Description of change
!       ====        ==========          =====================
!     07.03.17     J. O. Mydland        Original code
!
!
!
! Import scripts and global modules
!-----------------------------------------------------------------------------------------
include 'usr_add.f90'     ! Needed for GUI material input in PLAXIS
!-----------------------------------------------------------------------------------------
! PLAXIS - Subroutine
!-----------------------------------------------------------------------------------------
Subroutine User_Mod( IDTask, iMod, IsUndr, &
                     iStep, iTer, iEl, Int, &
                     X, Y, Z, &
                     Time0, dTime, &
                     Props, Sig0, Swp0, StVar0, &
                     dEps, D, BulkW, &
                     Sig, Swp, StVar, ipl, &
                     nStat, NonSym, iStrsDep, iTimeDep,iTang, &
                     iPrjDir, iPrjLen, iAbort )
!-----------------------------------------------------------------------------------------
! Import local scripts and local modules
  use modelInfoVars                      ! Contains info of available models
  use precision                          ! Precision used in all calculation
  use PLAXISroutines                     ! PLAXIS defined subroutines
 !use toolBox                            ! User defined subroutines
!-----------------------------------------------------------------------------------------
! PLAXIS - Dictionary
!-----------------------------------------------------------------------------------------
! Arguments:
!          I/O  Type
!  IDTask   I   I    : see above
!  iMod     I   I    : model number (1..10)
!  IsUndr   I   I    : =1 for undrained, 0 otherwise
!  iStep    I   I    : Global step number
!  iter     I   I    : Global iteration number
!  iel      I   I    : Global element number
!  Int      I   I    : Global integration point number
!  X        I   R    : X-Position of integration point
!  Y        I   R    : Y-Position of integration point
!  Z        I   R    : Z-Position of integration point
!  Time0    I   R    : Time at start of step
!  dTime    I   R    : Time increment
!  Props    I   R()  : List with model parameters
!  Sig0     I   R()  : Stresses at start of step
!  Swp0     I   R    : Excess pore pressure start of step
!  StVar0   I   R()  : State variable at start of step
!  dEps     I   R()  : Strain increment
!  D       I/O  R(,) : Material stiffness matrix
!  BulkW   I/O  R    : Bulkmodulus for water (undrained only)
!  Sig      O   R()  : Resulting stresses
!  Swp      O   R    : Resulting excess pore pressure
!  StVar    O   R()  : Resulting values state variables
!  ipl      O   I    : Plasticity indicator
!  nStat    O   I    : Number of state variables
!  NonSym   O   I    : Non-Symmetric D-matrix ?
!  iStrsDep O   I    : =1 for stress dependent D-matrix
!  iTimeDep O   I    : =1 for time dependent D-matrix
!  iTang    O   I    : =1 for full Newton-Raphson iteration
!  iPrjDir  I   I    : Project directory (for debugging purposes)
!  iPrjLen  I   I    : Length of project directory name (for debugging purposes)
!  iAbort   O   I    : =1 to force stopping of calculation
!-----------------------------------------------------------------------------------------
Implicit Double Precision (A-H,O-Z)
! Declare variable types
!-----------------------------------------------------------------------------------------
INTEGER :: IDTask, iMod, IsUndr, iStep, iTer, iEl, Int, iPrjLen
INTEGER :: ipl,  nStat, NonSym, iStrsDep, iTimeDep, iTang, iAbort, IO
real(kind=realDBL), DIMENSION(6,6) ::  D
real(kind=realDBL), DIMENSION(6)   ::  Sig0, Sig, dEps
real(kind=realDBL), DIMENSION(*)   ::  StVar0, StVar
real(kind=realDBL), DIMENSION(*)   ::  Props
Character(len=255)                 ::  Dbg_Name
Character(len=*)                   ::  iPrjDir
!-----------------------------------------------------------------------------------------
!-----------------------------------------------------------------------------------------
! Save to file for debugging
IO = 0
! Next line should be used for DLL-option (DF compiler)
!DEC$ ATTRIBUTES DLLExport, StdCall, Reference :: User_Mod

If (IO > 0) Then
    Open( IO, STATUS = 'Replace',File = 'E:\OneDrive - NTNU\4-Master\3. Fortran\Debug\dbg.txt', iostat = ios)
    !Open( IO, STATUS = 'Replace',File = 'C:\Users\johannom\OneDrive - NTNU\4-Master\3. Fortran\Debug\dbg.txt', iostat = ios)
! !DEC$ IF DEFINED(_X86_)
!    ! this 32-bit ??
!    Write(IO,1050)'IF32'
! !DEC$ ELSE
!    ! this 64-bit ??
!    Write(IO,1050)'IF64'
! !DEC$ ENDIF
!
! 1050   format ( 1X,A,1x,I0 )

    ! Call WriVec(IO, 'Props', Props, 5)
    ! Call Flush(IO)
    ! Call WriVec(IO, 'StateVars', ???)
    ! Call WriIvl(IO, 'IDTask    ', IDTask )
    ! Call Flush(IO)
    !
    ! If (ios == 1) Then
    !     Close( IO, Status = 'delete', iostat = ios)
    ! end if
end if
!-----------------------------------------------------------------------------------------
! Select the model of interest
!-----------------------------------------------------------------------------------------
  call selectModel(  IDTask, iMod, IsUndr, &
                     iStep, iTer, iEl, Int, &
                     X, Y, Z, &
                     Time0, dTime, &
                     Props, Sig0, Swp0, StVar0, &
                     dEps, D, BulkW, &
                     Sig, Swp, StVar, ipl, &
                     nStat, NonSym, iStrsDep, &
                     iTimeDep, iTang, iAbort, &
                     iPrjDir, iPrjLen, IO )
!-----------------------------------------------------------------------------------------
! Extra shiz for debugging
!-----------------------------------------------------------------------------------------
    ! If (IDTask .Eq. 5.And.iel+int.Eq.2) Then
    !     Write(IO,*)'nStat   : ',nStat
    !     Write(IO,*)'NonSym  : ',NonSym
    !     Write(IO,*)'StrsDep : ',iStrsDep
    !     Write(IO,*)'TimeDep : ',iTimeDep
    !     Write(IO,*)'Tangent : ',iTang
    ! End If

    ! If (IDTask == 3 .And. (iel+int) == 2) Then
    !     Write(IO,*)'IDTask: ',IDTask,' iStep,iTer',iStep,iTer
    !     Call Flush(IO)
    ! End If

! Write(IO, *) '-----------------------------------------------------------------'
!-----------------------------------------------------------------------------------------
!-----------------------------------------------------------------------------------------
end subroutine User_Mod
