function createsymbolics()
% Modified Cam-Clay with destructuration
% @JR,   04.05.16
% @Jomy, 16.05.17

%% Model name:
ModelName = 'VonMises';

%% Material parameters, etc.
% Stiffness
E     = sym('E', 'real');
nu    = sym('nu', 'real');

% Strength
Su   = sym('Su', 'real');

%% Include common symbolics
commonsyms;
% Stiffness
K = E/(3*(1-2*nu));
G = E/(2*(1+nu));
isotropicElastic

%% Matrix properties
NonSym      =   0;   % 1 for non-symmetric D-matrix
iStrsDep    =   0;   % 1 for stress dependent D-matrix
iTang       =   0;   % 1 for tanget D-matrix
iTimeDep    =   0;   % 1 for time dependent D-matrix
includeStateVars=   0;   % 1 for include statevariables in D-matrix
matrixProp  = [NonSym; iStrsDep; iTang; iTimeDep; includeStateVars];

%% Input and state vectors
% Input parameters
c = [Su;E;nu;];

% State variables
v  = [Sig ;lam ];
v0 = [Sig0;lam0];

%% Failure criteria
% Yield function 
F = q - 2*Su;

% Associated flow
Q = F;

%% Differentials
dEps_p    = diffvec(Q,Sig);

%% Residual equations
r1 = Sig - Sig0 - D*(dEps - dEps_p*(lam-lam0));
r2 = F;
Res = [r1;r2];

% Jacobian matrix
Jacobis = jacobian(Res,v);

% Trial stress
sigTrial = Sig0 + D*dEps;

% Initialization of state variables
v00 = [Sig; 0];


%% Output Fortran subroutines (for PLAXIS)
tic
%Output
folder = ['..\models\', ModelName];
mkdir(folder);
    
disp('Generating Fortran file...');
% %delete('residuals.f', 'jacobians.f', 'sigTrials.f', 'r.m', 'initStateVars.f', 'ElasticMatrix.f');
    
 fortran2(Res,      'File', [folder, '\residuals.f']);
 fortran2(Jacobis,  'File', [folder, '\jacobians.f']);
 fortran2(sigTrial, 'File', [folder, '\sigTrials.f']);
 fortran2(v00,      'File', [folder, '\initStateVars.f']);
 fortran2(D,        'File', [folder, '\ElasticMatrix.f']);
 
 materialGeneration(ModelName, matrixProp, c, v, v0, Res)
disp('Finished')
toc


%% Output Matlab files (for simple strain driver)
%h = {v,v0,c,de};

%disp('Generating Matlab functions..');
%tic

% Output
%delete('r.m', 'ja.m','s.m','f.m', 'v00.m');
%matlabFunction(R,'File','r.m','Vars',h);
%matlabFunction(J,'File','ja.m','Vars',h);
%matlabFunction(S,'File','s.m','Vars',h);
%matlabFunction(F,'File','f.m','Vars',h);
%matlabFunction(v00,'File','v00.m','Vars',h);

%toc
