% Made 22.01.16 Jon A. Ronningen
clear all; clc; close all;

%% Initial stress
stress = [100;100;100;0;0;0];
props = [];

%% Input parameters
%c = [Su;E;nu];
c = [1; 100; 0.2; 0.3];

%% Toleranse of error
tol = 10^-12;

%% Type of strain increment
straininc = 1*10^-4;
dt = 0;

% Undrained triaxial compression - 1
% Undrained triaxial extension   - 2
% Oedometer (/pure tension.)     - 3
% Pure volumetric                - 4
% Pure shear                     - 5
% Plane strain compression       - 6
% Plane strain extension         - 7
                             
                             val = 1;
switch val
    case 1 
        dEps = straininc*[ 2/3;-1/3;-1/3; 0;0;0];
    case 2
        dEps = straininc*[-2/3; 1/3; 1/3; 0;0;0];                           
    case 3
        dEps = straininc*[   1;   0;   0; 0;0;0];        
    case 4
        dEps = straininc*[ 1/3; 1/3; 1/3; 0;0;0];
    case 5
        dEps = straininc*[   0;   0;   0; 1;0;0];
    case 6
        dEps = straininc*[  -1;   1;   0; 0;0;0];
    case 7
        dEps = straininc*[   1;  -1;   0; 0;0;0];
    otherwise
        Error('Not valid')
end

% Run to amount of strain (approximately)
maxStrain = 0.25;
iter = round(maxStrain/norm(dEps));

%% Allocate memory
epsilon  = zeros(6, iter);
sigma    = zeros(6, iter);
p        = zeros(1, iter);
q        = zeros(1, iter);
sigma1   = zeros(1, iter);
sigma3   = zeros(1, iter);
epsilon1 = zeros(1, iter);
epsilon3 = zeros(1, iter);
tauXY    = zeros(1, iter);
gamXY    = zeros(1, iter);


%% Initialization of recorded values
p(1) = invariantStresses.hyd(stress);
q(1) = invariantStresses.dev(stress);

Eps  = [0;0;0;0;0;0];
sigma(:,1) = stress;

sigma1(1) = max(stress(1:3));
sigma3(1) = min(stress(1:3));

epsilon1(1) = max(Eps(1:3));
epsilon3(1) = min(Eps(1:3));

tauXY(1)    = max(stress(4:6));
gamXY(1)    = max(Eps(4:6));

%% Run analysis
for i = 2:1:iter
    [newStress, propsNew] = umatex(stress, dEps, dt, c', props, tol);
    props = propsNew;
    Eps = Eps + dEps;
    stress = newStress;
    
% Save data for plotting 
   sigma(:,i)   = stress;
   epsilon(:,i) = Eps;
   p(i) = invariantStresses.hyd(stress);
   q(i) = invariantStresses.dev(stress);
   sigma1(i) = max(stress(1:3));
   sigma3(i) = min(stress(1:3));
   epsilon1(i) = max(Eps(1:3));
   epsilon3(i) = min(Eps(1:3));
   tauXY(i)    = max(stress(4:6));
   gamXY(i)    = max(Eps(4:6));
end

figgy1 = figure('Position', [100, 100, 1500, 895]);
scale  = 1.2;
limits = 0.01;

% Plot stress/strain, or stress/stress curves
subplot(3,3,2); plot(epsilon(1,:),sigma(1,:)); 
    legend('\sigma_{11}'); grid on; hold on;
    plot(epsilon(1,1),sigma(1,1), 'r.','MarkerSize',10)
    xlim([min(-limits, min(epsilon(1,:))*scale), max(limits, max(epsilon(1,:))*scale)]);
    ylim([min( limits, min(sigma(1,:))*scale),   max(limits, max(sigma(1,:))*scale)]);   
subplot(3,3,5); plot(epsilon(2,:),sigma(2,:)); 
    legend('\sigma_{22}'); grid on; hold on;
    plot(epsilon(2,2),sigma(2,2), 'r.','MarkerSize',10)
    xlim([min(-limits, min(epsilon(2,:))*scale), max(limits, max(epsilon(2,:))*scale)]);
    ylim([min( limits, min(sigma(2,:))*scale),   max(limits, max(sigma(2,:))*scale)]);   
subplot(3,3,8); plot(epsilon(3,:),sigma(3,:)); 
    legend('\sigma_{33}'); grid on; hold on;
    plot(epsilon(3,3),sigma(3,3), 'r.','MarkerSize',10)
    xlim([min(-limits, min(epsilon(3,:))*scale), max(limits, max(epsilon(3,:))*scale)]);
    ylim([min( limits, min(sigma(3,:))*scale),   max(limits, max(sigma(3,:))*scale)]);   
subplot(3,3,3); plot(epsilon(4,:),sigma(4,:));
    legend('\sigma_{12}'); grid on; hold on;
    plot(epsilon(4,1),sigma(4,1), 'r.','MarkerSize',10)
    xlim([min(-limits, min(epsilon(4,:))*scale), max(limits, max(epsilon(4,:))*scale)]);
    ylim([min( limits, min(sigma(4,:))*scale),   max(limits, max(sigma(4,:))*scale)]);   
subplot(3,3,6); plot(epsilon(5,:),sigma(5,:));
    legend('\sigma_{13}'); grid on; hold on;
    plot(epsilon(5,1),sigma(5,1), 'r.','MarkerSize',10)    
    xlim([min(-limits, min(epsilon(5,:))*scale), max(limits, max(epsilon(5,:))*scale)]);
    ylim([min( limits, min(sigma(5,:))*scale),   max(limits, max(sigma(5,:))*scale)]);   
subplot(3,3,9); plot(epsilon(6,:),sigma(6,:)); legend('\sigma_{23}'); grid on;
    legend('\sigma_{23}'); grid on; hold on;
    plot(epsilon(6,1),sigma(6,1), 'r.','MarkerSize',10)
    xlim([min(-limits, min(epsilon(6,:))*scale), max(limits, max(epsilon(6,:))*scale)]);
    ylim([min( limits, min(sigma(6,:))*scale),   max(limits, max(sigma(6,:))*scale)]);   

tauDev   = (sigma(1,:)-sigma(3,:))/2;
if norm(tauDev) > norm(tauXY)
    tau  = tauDev;
else
    tau  = tauXY;
end
gamma = max(epsilon1 - epsilon3, gamXY);

subplot(3,3,1), plot(gamma, tau);
    grid on; hold on;
    plot(gamma(1),tau(1), 'r.','MarkerSize',10)
    xlabel('\gamma'); ylabel('\tau_{max, cartesian}'); grid on;
    xlim([min(-limits, min(gamma)*scale), max(limits, max(gamma)*scale)]);
    ylim([min( limits, min(tau)*scale),   max(limits, max(tau)*scale)]);    

subplot(3,3,4), plot(tauDev,tauXY);
    grid on; hold on;
    plot(tauDev(1),tauXY(1), 'r.','MarkerSize',10)
    xlabel('0.5(\sigma_{11} - \sigma_{33})'); ylabel('\tau_{12}');
    xlim([min(-limits, min(tauDev)*scale), max(limits, max(tauDev)*scale)]);
    ylim([min( limits, min(tauXY)*scale),  max(limits, max(tauXY)*scale)]);   
    
subplot(3,3,7), plot(p,q);
    grid on; hold on;
    plot(p(1),q(1), 'r.','MarkerSize',10)
    xlabel('p'); ylabel('q'); grid on; 
    xlim([min(-limits, min(p)*scale), max(limits, max(p)*scale)]);
    ylim([min( limits, min(q)*scale), max(limits, max(q)*scale)]);  
    
%  v = [stress; 1];
%  c = c;
%  dEps = dEps;
%  jacobians(v,v,c,dEps)
 