function [newStress, newProps] = umatex(oldStress, dEps, dt, c, props, tol)
% Made 22.01.16 Jon A. Ronningen

% Initialize properties
if (isempty(props))
    v       = [oldStress; 0];
    v0      = v;
    props   = initStateVars(v, v0, c', dEps);
end

% Retrieve stresses and properties
stateVars = props;
len       = length(stateVars);
%pause

% Trial elastic update
SigTr       = sigTrials(stateVars, stateVars, c', dEps);
stateVarsTr = [SigTr; stateVars(7:len)]; % First 6 entries are stress
fTr         = failureCheck(stateVarsTr, stateVarsTr, c', dEps);
    
    if (fTr >= 0)        
         % Perform Newton-Raphson iteration
         newStateVars = newtonraphson(stateVars, c', dEps, dt, tol);
         
         % Update stresses and state variables
         SigTr = newStateVars(1:6);
         props = newStateVars;
    else
         props(1:6) = SigTr;          
    end
    
    newStress = double(SigTr);
    newProps  = props;
    
end