!-------------------------------------------------------------------------------
!---------------------------   MATLAB GENERATED CODE   -------------------------
!-------------------------------------------------------------------------------
module CamClayVars ! This module contains material specific variables.
 use precision
 use standardVars 

 real(kind=realDBL), dimension(1) :: matrStVars, matrStVars0
 real(kind=realDBL), dimension(5) :: c
 real(kind=realDBL) :: phi 
 real(kind=realDBL) :: l 
 real(kind=realDBL) :: k 
 real(kind=realDBL) :: nu 
 real(kind=realDBL) :: OCR 
 real(kind=realDBL) :: pm 
 real(kind=realDBL) :: pm0 
 integer            :: state
 integer, parameter :: lenStateVars = size(minStVars)+size(matrStVars)
 real(kind=realDBL), dimension(lenStateVars) :: stateVars, stateVars0
end module CamClayVars
!-------------------------------------------------------------------------------
module CamClayClass
 use precision
 use standardVars
 use mapping
 use CamClayVars
 use matlabCalculations
 use PLAXISroutines
 implicit none
 private

 type, public, extends(matlabSymbolics) :: CamClay
 contains
  procedure :: materialMapping  =>  materialMapping
  procedure :: update           =>  update
  procedure :: updateStateVars  =>  updateStateVars
  procedure :: initialize       =>  initialize
  procedure :: stressUpdate     =>  stressUpdate
  procedure :: calcond          =>  calcond
  procedure :: NR               =>  NR
  procedure :: v00              =>  v00
  procedure :: DMatrix          =>  DMatrix
  procedure :: SigTr            =>  SigTr
  procedure :: resid            =>  resid
  procedure :: jacobian         =>  jacobian
  procedure :: IDTasker         =>  IDTasker
 end type CamClay
 contains
!-------------------------------------------------------------------------------
 subroutine materialMapping(this)
  class(CamClay) :: this
  ! Material parameters defined in Matlab
     phi = c(1) 
     l = c(2) 
     k = c(3) 
     nu = c(4) 
     OCR = c(5) 
     pm = matrStVars(1) 
     pm0 = matrStVars0(1) 
 end subroutine materialMapping
!-------------------------------------------------------------------------------
 subroutine update(this)
  class(CamClay) :: this
  call this%updateStateVars
  call this%standardMapping
  call this%materialMapping
  call this%numhelp
 end subroutine update
!-------------------------------------------------------------------------------
 subroutine updateStateVars(this)
  class(CamClay) :: this
  minStVars   = stateVars(1:7)
  matrStVars  = stateVars(8:(size(matrStVars)+8))

  minStVars0   = stateVars0(1:7)
  matrStVars0  = stateVars0(8:(size(matrStVars0)+8))
 end subroutine updateStateVars
!-------------------------------------------------------------------------------
 subroutine initialize(this)
  class(CamClay) :: this
  call this%update()
  call this%v00(stateVars)     ! Initialize state variables
 end subroutine initialize
!-------------------------------------------------------------------------------
 subroutine stressUpdate(this)
  class(CamClay) :: this
  logical :: ii

  call this%calcond(ii) ! Check if iterations are needed
  if (ii) then
   call this%NR()
  end if
 end subroutine stressUpdate
!-------------------------------------------------------------------------------
 subroutine calcond(this, aa)
  class(CamClay) :: this
   real(kind=realDBL), dimension(6)   :: Sig0
   real(kind=realDBL), dimension(6,6) :: D
   real(kind=realDBL), dimension(lenStateVars) :: fr
   logical :: aa
   call this%update()

     !Trial predictor - check yield condition
     call this%DMatrix(D)
     call this%SigTr(stateVars)
     call this%resid(fr)
     aa = fr(8) > 0
 end subroutine calcond
!-------------------------------------------------------------------------------
 recursive subroutine NR(this)
  use toolBox, only: gauss
  class(CamClay) :: this
   real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: J0
   real(kind=realDBL), dimension(lenStateVars)               :: r , x
   real(kind=realDBL) :: tol, res
   integer :: maxIter, Iter

   Iter    = 0
   tol     = 1.0D-8
   maxIter = 75
   res     = 1.0D0/tol
   x       = 0.0D0
   r       = 0.0D0
     do while (res > tol .and. Iter < maxIter)
         ! Calculate new position
         call this%resid(r)
         call this%jacobian(J0)
         !call WriMat ( 1, 'Jacobian' , J0 , 6, 6, 6 )
         !call flush(1)
         call gauss(J0, -r, x, lenStateVars)
         stateVars = stateVars + x
         ! Check residual
         call this%resid(r)
         Iter = Iter + 1
         res = dot_product(r,r)
     end do
     ! In case of to many iterations
     if (iter > maxIter) then
       state = 2
       !write(100, *) res
     end if
 end subroutine NR
!-------------------------------------------------------------------------------
 subroutine v00(this, A0) ! Initialization of state variables
  !The routine returns statevariables according to the CAS calculation in Matlab.
  Implicit Double Precision (A-Z)
  class(CamClay) :: this
  real(kind=realDBL), dimension(lenStateVars, 1) :: A0
  call this%update()
  A0    = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = phi*3.141592653589793D0*(1.0D0/1.8D2)
      t3 = sin(t2)
      t4 = t3-3.0D0
      t5 = s22*(1.0D0/3.0D0)
      t6 = s33*(1.0D0/3.0D0)
      t7 = s33*(1.0D0/2.0D0)
      t8 = s11*(1.0D0/3.0D0)
      t9 = s11*(1.0D0/2.0D0)
      t10 = s22*(1.0D0/2.0D0)
      t11 = t5+t6+t8+1.0D0/1.0D2
      A0(1,1) = s11
      A0(2,1) = s22
      A0(3,1) = s33
      A0(4,1) = s12
      A0(5,1) = s23
      A0(6,1) = s13
      A0(8,1) = (OCR*1.0D0/t3**2*t4**2*((s11*(-2.0D0/3.0D0)+t5+t6)*(-s11+t7+t10)+(-s22+t7+t9)*(s22*(-2.0D0/3.0D0)+t6+t8)+(s33*(-2.0D0/3.0D0)+t5+t8)*(-s33+t9+t10)+s12**2*3.0D0+s13**2*3.0D0+s23**2*3.0D0+t3**2*1.0D0/t4**2*t11**2*3.6D1)*(1.0D0/3.6D1))/t11
 end subroutine v00
!-------------------------------------------------------------------------------
 subroutine DMatrix(this, A0) ! Elastic stiffness matrix
  !The routine returns the elastic stiffness matrix
  Implicit Double Precision (A-Z)
  class(CamClay) :: this
  real(kind=realDBL), dimension(6,6) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = 1.0D0/k
      t3 = s11*(1.0D0/3.0D0)
      t4 = s22*(1.0D0/3.0D0)
      t5 = s33*(1.0D0/3.0D0)
      t6 = t3+t4+t5+1.0D0/1.0D2
      t7 = t2*t6
      t8 = nu*6.0D0
      t9 = t8-3.0D0
      t10 = nu+1.0D0
      t11 = 1.0D0/t10
      t12 = t2*t6*t9*t11*(1.0D0/3.0D0)
      t13 = t7+t12
      t15 = t2*t6*t9*t11*(2.0D0/3.0D0)
      t14 = t7-t15
      A0(1,1) = t14
      A0(1,2) = t13
      A0(1,3) = t13
      A0(2,1) = t13
      A0(2,2) = t14
      A0(2,3) = t13
      A0(3,1) = t13
      A0(3,2) = t13
      A0(3,3) = t14
      A0(4,4) = t2*t6*t9*t11*(-1.0D0/2.0D0)
      A0(5,5) = t2*t6*t9*t11*(-1.0D0/2.0D0)
      A0(6,6) = t2*t6*t9*t11*(-1.0D0/2.0D0)
 end subroutine Dmatrix
!-------------------------------------------------------------------------------
 subroutine SigTr(this, A0) ! Trial stress
  !The routine calculates the elastic predictor stress
  Implicit Double Precision (A-Z)
  class(CamClay) :: this
  real(kind=realDBL), dimension(6, 1) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = 1.0D0/k
      t3 = s11*(1.0D0/3.0D0)
      t4 = s22*(1.0D0/3.0D0)
      t5 = s33*(1.0D0/3.0D0)
      t6 = t3+t4+t5+1.0D0/1.0D2
      t7 = t2*t6
      t8 = nu*6.0D0
      t9 = t8-3.0D0
      t10 = nu+1.0D0
      t11 = 1.0D0/t10
      t12 = t2*t6*t9*t11*(1.0D0/3.0D0)
      t13 = t7+t12
      t18 = t2*t6*t9*t11*(2.0D0/3.0D0)
      t14 = t7-t18
      t15 = de33*t13
      t16 = de11*t13
      t17 = de22*t13
      A0(1,1) = s110+t15+t17+de11*t14
      A0(2,1) = s220+t15+t16+de22*t14
      A0(3,1) = s330+t16+t17+de33*t14
      A0(4,1) = s120-de12*t2*t6*t9*t11*(1.0D0/2.0D0)
      A0(5,1) = s230-de23*t2*t6*t9*t11*(1.0D0/2.0D0)
      A0(6,1) = s130-de13*t2*t6*t9*t11*(1.0D0/2.0D0)
 end subroutine SigTr
!-------------------------------------------------------------------------------
 subroutine resid(this, A0) ! Residual equation
  !The routine calculates the residuals
  Implicit Double Precision (A-Z)
  class(CamClay) :: this
  real(kind=realDBL), dimension(lenStateVars, 1) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t3 = phi*3.141592653589793D0*(1.0D0/1.8D2)
      t2 = sin(t3)
      t4 = t2**2
      t5 = t2-3.0D0
      t6 = 1.0D0/t5**2
      t7 = s11*(1.0D0/3.0D0)
      t8 = s22*(1.0D0/3.0D0)
      t9 = s33*(1.0D0/3.0D0)
      t10 = t7+t8+t9+1.0D0/1.0D2
      t11 = 1.0D0/k
      t12 = lam-lam0
      t13 = -pm+t7+t8+t9+1.0D0/1.0D2
      t14 = t4*t6*t13*1.2D1
      t15 = t4*t6*t10*1.2D1
      t16 = t10*t11
      t17 = nu*6.0D0
      t18 = t17-3.0D0
      t19 = nu+1.0D0
      t20 = 1.0D0/t19
      t21 = t10*t11*t18*t20*(1.0D0/3.0D0)
      t22 = t16+t21
      t23 = s11*2.0D0
      t24 = -s22-s33+t14+t15+t23
      t33 = t12*t24
      t25 = de11-t33
      t26 = s22*2.0D0
      t27 = -s11-s33+t14+t15+t26
      t34 = t12*t27
      t28 = de22-t34
      t36 = t10*t11*t18*t20*(2.0D0/3.0D0)
      t29 = t16-t36
      t30 = s33*2.0D0
      t31 = -s11-s22+t14+t15+t30
      t35 = t12*t31
      t32 = de33-t35
      t37 = s33*(1.0D0/2.0D0)
      t38 = s11*(1.0D0/2.0D0)
      t39 = s22*(1.0D0/2.0D0)
      A0(1,1) = s11-s110-t22*t28-t22*t32-t25*t29
      A0(2,1) = s22-s220-t22*t25-t22*t32-t28*t29
      A0(3,1) = s33-s330-t22*t25-t22*t28-t29*t32
      A0(4,1) = s12-s120+t10*t11*t18*t20*(de12-s12*t12*6.0D0)*(1.0D0/2.0D0)
      A0(5,1) = s23-s230+t10*t11*t18*t20*(de23-s23*t12*6.0D0)*(1.0D0/2.0D0)
      A0(6,1) = s13-s130+t10*t11*t18*t20*(de13-s13*t12*6.0D0)*(1.0D0/2.0D0)
      A0(7,1) = pm-pm0+(pm*t12*(t4*t6*t10*3.6D1+t4*t6*t13*3.6D1))/(k-l)
      A0(8,1) = (s11*(-2.0D0/3.0D0)+t8+t9)*(-s11+t37+t39)+(s22*(-2.0D0/3.0D0)+t7+t9)*(-s22+t37+t38)+(s33*(-2.0D0/3.0D0)+t7+t8)*(-s33+t38+t39)+s12**2*3.0D0+s13**2*3.0D0+s23**2*3.0D0+t4*t6*t10*t13*3.6D1
 end subroutine resid
!-------------------------------------------------------------------------------
 subroutine jacobian(this, A0) ! Jacobian matrix
  Implicit Double Precision (A-Z)
  class(CamClay) :: this
  real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = 1.0D0/k
      t4 = phi*3.141592653589793D0*(1.0D0/1.8D2)
      t3 = sin(t4)
      t5 = t3**2
      t6 = t3-3.0D0
      t7 = 1.0D0/t6**2
      t8 = s11*(1.0D0/3.0D0)
      t9 = s22*(1.0D0/3.0D0)
      t10 = s33*(1.0D0/3.0D0)
      t11 = t2*(1.0D0/3.0D0)
      t12 = nu*6.0D0
      t13 = t12-3.0D0
      t14 = nu+1.0D0
      t15 = 1.0D0/t14
      t16 = lam-lam0
      t17 = -pm+t8+t9+t10+1.0D0/1.0D2
      t18 = t5*t7*t17*1.2D1
      t19 = t8+t9+t10+1.0D0/1.0D2
      t20 = t5*t7*t19*1.2D1
      t21 = t2*t13*t15*(1.0D0/9.0D0)
      t22 = t11+t21
      t23 = t5*t7*8.0D0
      t24 = t2*t19
      t40 = t2*t13*t15*(2.0D0/9.0D0)
      t25 = t11-t40
      t26 = s11*2.0D0
      t27 = -s22-s33+t18+t20+t26
      t41 = t16*t27
      t28 = de11-t41
      t29 = s22*2.0D0
      t30 = -s11-s33+t18+t20+t29
      t42 = t16*t30
      t31 = de22-t42
      t32 = s33*2.0D0
      t33 = -s11-s22+t18+t20+t32
      t43 = t16*t33
      t34 = de33-t43
      t35 = t23-1.0D0
      t36 = t2*t13*t15*t19*(1.0D0/3.0D0)
      t37 = t24+t36
      t45 = t2*t13*t15*t19*(2.0D0/3.0D0)
      t38 = t24-t45
      t39 = t23+2.0D0
      t44 = t16*t35*t37
      t46 = t16*t35*t38
      t47 = t16*t37*t39
      t49 = t22*t34
      t56 = t22*t31
      t48 = t44+t46+t47-t49-t56-t25*t28
      t50 = t16*t35*t37*2.0D0
      t51 = t16*t38*t39
      t55 = t22*t28
      t52 = t44+t46+t47-t49-t55-t25*t31
      t53 = t33*t37
      t61 = t5*t7*t16*t37*2.4D1
      t62 = t5*t7*t16*t38*1.2D1
      t54 = -t61-t62
      t58 = t25*t34
      t57 = t44+t46+t47-t55-t56-t58
      t59 = t30*t37
      t60 = t27*t37
      t65 = s12*t16*6.0D0
      t63 = de12-t65
      t64 = t2*t13*t15*t63*(1.0D0/6.0D0)
      t68 = s23*t16*6.0D0
      t66 = de23-t68
      t67 = t2*t13*t15*t66*(1.0D0/6.0D0)
      t69 = lam*6.0D0
      t70 = lam0*6.0D0
      t71 = t69-t70
      t76 = t2*t13*t15*t19*t71*(1.0D0/2.0D0)
      t72 = -t76+1.0D0
      t75 = s13*t16*6.0D0
      t73 = de13-t75
      t74 = t2*t13*t15*t73*(1.0D0/6.0D0)
      t77 = k-l
      t78 = 1.0D0/t77
      t79 = pm*t5*t7*t16*t78*2.4D1
      t80 = t5*t7*t17*3.6D1
      t81 = t5*t7*t19*3.6D1
      t82 = t80+t81
      A0(1,1) = t50+t51-t22*t31-t25*t28-t22*t34+1.0D0
      A0(1,2) = t48
      A0(1,3) = t48
      A0(1,7) = t53+t59+t27*t38
      A0(1,8) = t54
      A0(2,1) = t52
      A0(2,2) = -t49+t50+t51-t22*t28-t25*t31+1.0D0
      A0(2,3) = t52
      A0(2,7) = t53+t60+t30*t38
      A0(2,8) = t54
      A0(3,1) = t57
      A0(3,2) = t57
      A0(3,3) = t50+t51-t55-t56-t58+1.0D0
      A0(3,7) = t59+t60+t33*t38
      A0(3,8) = t54
      A0(4,1) = t64
      A0(4,2) = t64
      A0(4,3) = t64
      A0(4,4) = t72
      A0(4,7) = s12*t2*t13*t15*t19*(-3.0D0)
      A0(5,1) = t67
      A0(5,2) = t67
      A0(5,3) = t67
      A0(5,5) = t72
      A0(5,7) = s23*t2*t13*t15*t19*(-3.0D0)
      A0(6,1) = t74
      A0(6,2) = t74
      A0(6,3) = t74
      A0(6,6) = t72
      A0(6,7) = s13*t2*t13*t15*t19*(-3.0D0)
      A0(7,1) = t79
      A0(7,2) = t79
      A0(7,3) = t79
      A0(7,7) = pm*t78*t82
      A0(7,8) = t16*t78*t82-pm*t5*t7*t16*t78*3.6D1+1.0D0
      A0(8,1) = t27
      A0(8,2) = t30
      A0(8,3) = t33
      A0(8,4) = s12*6.0D0
      A0(8,5) = s23*6.0D0
      A0(8,6) = s13*6.0D0
      A0(8,8) = -t81
 end subroutine jacobian
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
 subroutine IDTasker(this, IDTask, IsUndr, &
					iStep, iTer, iEl, Int, &
					X, Y, Z, &
					Time0, dTime, &
					Props, Sig0, Swp0, StVar0,&
					dEps, D, BulkW, & 
					Sig, Swp, StVar, ipl, &
					nStat,NonSym, iStrsDep, &
					iTimeDep, iTang, iAbort, &
 				 iPrjDir, iPrjLen, IO)
!-------------------------------------------------------------------------------
! Import local scripts and local modules
  ! use toolBox
  use PLAXISroutines
  use precision
  use standardVars
  use CamClayVars
!-------------------------------------------------------------------------------
! PLAXIS - Dictionary
!-------------------------------------------------------------------------------
!  Depending on IDTask, 1 : Initialize state variables
!                       2 : calculate stresses
!                       3 : calculate material stiffness matrix
!                       4 : return number of state variables
!                       5 : inquire matrix properties
!                           return switch for non-symmetric D-matrix
!                           stress/time dependent matrix
!                       6 : calculate elastic material stiffness matrix
!-------------------------------------------------------------------------------
 Implicit Double Precision (A-H, O-Z)
 class(CamClay) :: this
! Declare variable types
!-------------------------------------------------------------------------------
 Character(len=255) :: PrjDir
 Character(len=*)   :: iPrjDir
 INTEGER :: IDTask, IsUndr, iStep, iTer, iEl, Int, iPrjLen IO
 INTEGER :: ipl,  nStat, NonSym, iStrsDep, iTimeDep, iTang, iAbort, IO
 real(kind=realDBL), DIMENSION(6,6) ::  D
 real(kind=realDBL), DIMENSION(6)   ::  Sig0, Sig, dEps
 real(kind=realDBL), DIMENSION(lenStateVars)  ::  StVar0, StVar, StVarPrevMatr
 real(kind=realDBL), DIMENSION(lenStateVars)  ::  StVarPrev, StVarInitPrev
 real(kind=realDBL), DIMENSION(size(c))       ::  Props
 ! Local variables
 INTEGER ::  ne, nt, n, i, msf, iOpt
 real(kind=realDBL) :: dNorm, dEpsMax, dTimeMax
 real(kind=realDBL), DIMENSION(6)   ::  dAbsEps
 Logical :: existence = .FALSE.
!-------------------------------------------------------------------------------
!******************************* MODEL DESCRIPTION *****************************
!-------------------------------------------------------------------------------
! PLAXIS - IDTask
!-------------------------------------------------------------------------------
! Initialize state variables
 Task_1: if (IDTask == 1) then
     stateVars       =  0.0D0
     stateVars0      =  0.0D0
     stateVars(1:6)  = -1.0D0*Sig0(1:6)  ! Define compression as positive
     c               =  Props

     if (norm2(stateVars0) == 0) then
       call this%initialize()
       StVar0 = stateVars
     end if
  end if Task_1
!-------------------------------------------------------------------------------
 ! Calculate stresses
 Task_2: if (IDTask == 2) then
     ipl            = 0
     stateVars      = StVar0
     stateVars(1:6) = -Sig0(1:6)
     stateVars0     = stateVars
     dt             = dTime
     dEpsilon       = 0.D0
     dEpsilon(1:6)  = -1.0D0*dEps(1:6)
     state          = StVar0(lenStateVars + 1)
     c              = Props

     If (IsUndr == 1) then
       Swp = Swp0 + BulkW*(dEps(1) + dEps(2) + dEps(3))
     else
       Swp = Swp0
     end if

     dNorm = norm2(dEpsilon)

     if (dnorm /= 0) then
       dAbsEps  = abs(dEpsilon)
       dEpsMax  = max(dAbsEps(1), dAbsEps(2), dAbsEps(3), dAbsEps(4), dAbsEps(5), dAbsEps(6))
       dTimeMax = dt
       ne       = nint(demax/1.0D-3)
       nt       = nint(dtmax/1.0D3)

       if (n == 0) then
         n = 1
       end if

       if (nt == 0) then
         nt = 1
       end if

       n   = max(ne, nt)
       dEpsilon  = dEpsilon/n*1.0D0
       dt  = dt/n*1.0D0

       do i = 1, n
         call this%stressupdate()     ! Note: Updates stateVars
         stateVars0  = stateVars
         ipl         = state
        end do
     end if

     ! Store updated values
     Sig(1:6)              = -1.0D0*stateVars(1:6)
     StVar                 = stateVars
     StVar(lenStateVars+1) = state
  end if Task_2
!-------------------------------------------------------------------------------
 ! Calculate material stiffness matrix
 Task_3: if (IDTask == 3) then
     stateVars = StVar0     ! Used if matrix is stress dependent
     call this%DMatrix(D)
     !D = 2.0d0*D     ! TODO: Why 2x ?
     msf = Sig0(15)  ! TODO: What does this do??
  end if Task_3
!-------------------------------------------------------------------------------
 ! Number of state variables
 Task_4: if (IDTask == 4) then
     nStat = lenStateVars + 1
  end if Task_4
!-------------------------------------------------------------------------------
 ! Inquire matrix properties
 Task_5: if (IDTask == 5) then
     NonSym      =   0   ! 1 for non-symmetric D-matrix
     iStrsDep    =   0   ! 1 for stress dependent D-matrix
     iTang       =   0   ! 1 for tanget D-matrix
     iTimeDep    =   0   ! 1 for time dependent D-matrix
  end if Task_5
!-------------------------------------------------------------------------------
 ! Calculate elastic material stiffness matrix
 Task_6: if (IDTask == 6) then
     stateVars = StVar0     ! Used if matrix is stress dependent
     call this%DMatrix(D)
     !D = 2.0d0*D     ! TODO: Why 2x ?
  end if Task_6
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
end subroutine IDTasker
end module CamClayClass
