!-------------------------------------------------------------------------------
!---------------------------   MATLAB GENERATED CODE   -------------------------
!-------------------------------------------------------------------------------
module VonMisesVars ! This module contains material specific variables.
 use precision
 use standardVars

 real(kind=realDBL), dimension(0) :: matrStVars, matrStVars0
 real(kind=realDBL), dimension(3) :: c
 real(kind=realDBL) :: Su
 real(kind=realDBL) :: E
 real(kind=realDBL) :: nu
 integer            :: state
 integer, parameter :: lenStateVars = size(minStVars)+size(matrStVars)
 real(kind=realDBL), dimension(lenStateVars) :: stateVars, stateVars0
end module VonMisesVars
!-------------------------------------------------------------------------------
module VonMisesClass
 use precision
 use standardVars
 use mapping
 use VonMisesVars
 use matlabCalculations
 use PLAXISroutines
 implicit none
 private

 type, public, extends(matlabSymbolics) :: VonMises
 contains
  procedure :: materialMapping  =>  materialMapping
  procedure :: update           =>  update
  procedure :: updateStateVars  =>  updateStateVars
  procedure :: initialize       =>  initialize
  procedure :: stressUpdate     =>  stressUpdate
  procedure :: calcond          =>  calcond
  procedure :: NR               =>  NR
  procedure :: v00              =>  v00
  procedure :: DMatrix          =>  DMatrix
  procedure :: SigTr            =>  SigTr
  procedure :: resid            =>  resid
  procedure :: jacobian         =>  jacobian
  procedure :: IDTasker         =>  IDTasker
 end type VonMises
 contains
!-------------------------------------------------------------------------------
 subroutine materialMapping(this)
  class(VonMises) :: this
  ! Material parameters defined in Matlab
     Su = c(1)
     E = c(2)
     nu = c(3)
 end subroutine materialMapping
!-------------------------------------------------------------------------------
 subroutine update(this)
  class(VonMises) :: this
  call this%updateStateVars
  call this%standardMapping
  call this%materialMapping
  call this%numhelp
 end subroutine update
!-------------------------------------------------------------------------------
 subroutine updateStateVars(this)
  class(VonMises) :: this
  minStVars   = stateVars(1:7)
  matrStVars  = stateVars(8:(size(matrStVars)+8))

  minStVars0   = stateVars0(1:7)
  matrStVars0  = stateVars0(8:(size(matrStVars0)+8))
 end subroutine updateStateVars
!-------------------------------------------------------------------------------
 subroutine initialize(this)
  class(VonMises) :: this
  call this%update()
  call this%v00(stateVars)     ! Initialize state variables
 end subroutine initialize
!-------------------------------------------------------------------------------
 subroutine stressUpdate(this)
  class(VonMises) :: this
  logical :: ii

  call this%calcond(ii) ! Check if iterations are needed
  if (ii) then
   call this%NR()
  end if
 end subroutine stressUpdate
!-------------------------------------------------------------------------------
 subroutine calcond(this, aa)
  class(VonMises) :: this
   real(kind=realDBL), dimension(6)   :: Sig0
   real(kind=realDBL), dimension(6,6) :: D
   real(kind=realDBL), dimension(lenStateVars) :: fr
   logical :: aa
   call this%update()

     !Trial predictor - check yield condition
     call this%DMatrix(D)
     call this%SigTr(stateVars)
     call this%resid(fr)
     aa = fr(7) > 0
 end subroutine calcond
!-------------------------------------------------------------------------------
 recursive subroutine NR(this)
  use toolBox, only: gauss
  class(VonMises) :: this
   real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: J0
   real(kind=realDBL), dimension(lenStateVars)               :: r , x
   real(kind=realDBL) :: tol, res
   integer :: maxIter, Iter

   Iter    = 0
   tol     = 1.0D-8
   maxIter = 75
   res     = 1.0D0/tol
   x       = 0.0D0
   r       = 0.0D0
     do while (res > tol .and. Iter < maxIter)
         ! Calculate new position
         call this%resid(r)
         call this%jacobian(J0)
         !call WriMat ( 1, 'Jacobian' , J0 , 6, 6, 6 )
         !call flush(1)
         call gauss(J0, -r, x, lenStateVars)
         stateVars = stateVars + x
         ! Check residual
         call this%resid(r)
         Iter = Iter + 1
         res = dot_product(r,r)
     end do
     ! In case of to many iterations
     if (iter > maxIter) then
       state = 2
       !write(100, *) res
     end if
 end subroutine NR
!-------------------------------------------------------------------------------
 subroutine v00(this, A0) ! Initialization of state variables
  !The routine returns statevariables according to the CAS calculation in Matlab.
  Implicit Double Precision (A-Z)
  class(VonMises) :: this
  real(kind=realDBL), dimension(lenStateVars, 1) :: A0
  call this%update()
  A0    = 0.D0   ! Hard reset variables
  !Matlab code
      A0(1,1) = s11
      A0(2,1) = s22
      A0(3,1) = s33
      A0(4,1) = s12
      A0(5,1) = s23
      A0(6,1) = s13
 end subroutine v00
!-------------------------------------------------------------------------------
 subroutine DMatrix(this, A0) ! Elastic stiffness matrix
  !The routine returns the elastic stiffness matrix
  Implicit Double Precision (A-Z)
  class(VonMises) :: this
  real(kind=realDBL), dimension(6,6) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = nu*2.0D0
      t3 = t2+2.0D0
      t4 = 1.0D0/t3
      t5 = nu*6.0D0
      t6 = t5-3.0D0
      t7 = 1.0D0/t6
      t9 = E*t4*(2.0D0/3.0D0)
      t10 = E*t7
      t8 = -t9-t10
      t11 = E*t4*(4.0D0/3.0D0)
      t12 = -t10+t11
      t13 = E*t4
      A0(1,1) = t11-E*t7
      A0(1,2) = t8
      A0(1,3) = t8
      A0(2,1) = t8
      A0(2,2) = t12
      A0(2,3) = t8
      A0(3,1) = t8
      A0(3,2) = t8
      A0(3,3) = t12
      A0(4,4) = t13
      A0(5,5) = t13
      A0(6,6) = t13
 end subroutine Dmatrix
!-------------------------------------------------------------------------------
 subroutine SigTr(this, A0) ! Trial stress
  !The routine calculates the elastic predictor stress
  Implicit Double Precision (A-Z)
  class(VonMises) :: this
  real(kind=realDBL), dimension(6, 1) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = nu*2.0D0
      t3 = t2+2.0D0
      t4 = 1.0D0/t3
      t5 = nu*6.0D0
      t6 = t5-3.0D0
      t7 = 1.0D0/t6
      t8 = E*t4*(2.0D0/3.0D0)
      t9 = E*t7
      t10 = t8+t9
      t11 = E*t4*(4.0D0/3.0D0)
      t12 = t9-t11
      A0(1,1) = s110-de22*t10-de33*t10+de11*(t11-E*t7)
      A0(2,1) = s220-de11*t10-de22*t12-de33*t10
      A0(3,1) = s330-de11*t10-de22*t10-de33*t12
      A0(4,1) = s120+E*de12*t4
      A0(5,1) = s230+E*de23*t4
      A0(6,1) = s130+E*de13*t4
 end subroutine SigTr
!-------------------------------------------------------------------------------
 subroutine resid(this, A0) ! Residual equation
  !The routine calculates the residuals
  Implicit Double Precision (A-Z)
  class(VonMises) :: this
  real(kind=realDBL), dimension(lenStateVars, 1) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  !Matlab code
      t2 = s33*(1.0D0/2.0D0)
      t3 = s33*(1.0D0/3.0D0)
      t4 = s11*(1.0D0/2.0D0)
      t5 = s22*(1.0D0/2.0D0)
      t6 = s11*(1.0D0/3.0D0)
      t7 = s22*(1.0D0/3.0D0)
      t8 = -s11+t2+t5
      t34 = s11*(2.0D0/3.0D0)
      t9 = t3+t7-t34
      t10 = t8*t9
      t11 = -s22+t2+t4
      t35 = s22*(2.0D0/3.0D0)
      t12 = t3+t6-t35
      t13 = t11*t12
      t14 = -s33+t4+t5
      t36 = s33*(2.0D0/3.0D0)
      t15 = t6+t7-t36
      t16 = t14*t15
      t17 = s12**2
      t18 = t17*3.0D0
      t19 = s13**2
      t20 = t19*3.0D0
      t21 = s23**2
      t22 = t21*3.0D0
      t23 = t10+t13+t16+t18+t20+t22
      t24 = sqrt(t23)
      t25 = conjg(t24)
      t26 = 1.0D0/t25
      t27 = lam-lam0
      t28 = nu*2.0D0
      t29 = t28+2.0D0
      t30 = 1.0D0/t29
      t31 = nu*6.0D0
      t32 = t31-3.0D0
      t33 = 1.0D0/t32
      t37 = E*t30*(2.0D0/3.0D0)
      t38 = E*t33
      t39 = t37+t38
      t51 = s11*2.0D0
      t40 = s22+s33-t51
      t41 = t26*t27*t40*(1.0D0/2.0D0)
      t42 = de11+t41
      t53 = s22*2.0D0
      t43 = s11+s33-t53
      t44 = t26*t27*t43*(1.0D0/2.0D0)
      t45 = de22+t44
      t46 = E*t30*(4.0D0/3.0D0)
      t55 = s33*2.0D0
      t47 = s11+s22-t55
      t48 = t26*t27*t47*(1.0D0/2.0D0)
      t49 = de33+t48
      t50 = t39*t49
      t52 = t39*t42
      t54 = t39*t45
      A0(1,1) = s11-s110+t50+t54-t42*(t46-E*t33)
      A0(2,1) = s22-s220+t50+t52+t45*(t38-t46)
      A0(3,1) = s33-s330+t52+t54+t49*(t38-t46)
      A0(4,1) = s12-s120-E*t30*(de12-s12*t26*t27*3.0D0)
      A0(5,1) = s23-s230-E*t30*(de23-s23*t26*t27*3.0D0)
      A0(6,1) = s13-s130-E*t30*(de13-s13*t26*t27*3.0D0)
      A0(7,1) = Su*(-2.0D0)+t24
 end subroutine resid
!-------------------------------------------------------------------------------
 subroutine jacobian(this, A0) ! Jacobian matrix
  Implicit Double Precision (A-Z)
  class(VonMises) :: this
  real(kind=realDBL), dimension(lenStateVars, lenStateVars) :: A0
  call this%update()
  A0 = 0.D0   ! Hard reset variables
  call this%numhelp()
  !Matlab code
      t2 = s33*(1.0D0/2.0D0)
      t3 = s33*(1.0D0/3.0D0)
      t4 = s11*(1.0D0/2.0D0)
      t5 = s22*(1.0D0/2.0D0)
      t6 = s11*(1.0D0/3.0D0)
      t7 = s22*(1.0D0/3.0D0)
      t8 = -s11+t2+t5
      t36 = s11*(2.0D0/3.0D0)
      t9 = t3+t7-t36
      t10 = t8*t9
      t11 = -s22+t2+t4
      t37 = s22*(2.0D0/3.0D0)
      t12 = t3+t6-t37
      t13 = t11*t12
      t14 = -s33+t4+t5
      t38 = s33*(2.0D0/3.0D0)
      t15 = t6+t7-t38
      t16 = t14*t15
      t17 = s12**2
      t18 = t17*3.0D0
      t19 = s13**2
      t20 = t19*3.0D0
      t21 = s23**2
      t22 = t21*3.0D0
      t23 = t10+t13+t16+t18+t20+t22
      t24 = sqrt(t23)
      t25 = conjg(t24)
      t26 = lam-lam0
      t27 = nu*2.0D0
      t28 = t27+2.0D0
      t29 = 1.0D0/t28
      t30 = E*t29*(2.0D0/3.0D0)
      t31 = nu*6.0D0
      t32 = t31-3.0D0
      t33 = 1.0D0/t32
      t34 = E*t33
      t35 = t30+t34
      t39 = 1.0D0/t25
      t40 = t26*t39*(1.0D0/2.0D0)
      t41 = 1.0D0/t25**3
      t43 = s11*2.0D0
      t42 = s22+s33-t43
      t50 = s33*2.0D0
      t44 = s11+s22-t50
      t46 = s22*2.0D0
      t45 = s11+s33-t46
      t47 = t26*t41*t42*t45*(1.0D0/4.0D0)
      t48 = t40+t47
      t49 = t26*t39
      t51 = t26*t41*t44*t45*(1.0D0/4.0D0)
      t52 = t40+t51
      t53 = t35*t52
      t54 = t26*t41*t42*t44*(1.0D0/4.0D0)
      t55 = t40+t54
      t57 = E*t29*(4.0D0/3.0D0)
      t56 = t34-t57
      t58 = t35*t55
      t59 = t42**2
      t67 = t26*t41*t59*(1.0D0/4.0D0)
      t60 = t49-t67
      t61 = t35*t48
      t62 = t45**2
      t69 = t26*t41*t62*(1.0D0/4.0D0)
      t63 = t49-t69
      t64 = t44**2
      t70 = t26*t41*t64*(1.0D0/4.0D0)
      t65 = t49-t70
      t66 = t35*t39*t44*(1.0D0/2.0D0)
      t68 = t52*(t34-t57)
      t71 = t35*t39*t45*(1.0D0/2.0D0)
      t72 = t35*t39*t42*(1.0D0/2.0D0)
      t73 = t26*t39*3.0D0
      t74 = 1.0D0/sqrt(t23)
      A0(1,1) = t58+t61-t56*t60+1.0D0
      A0(1,2) = t53-t35*t63+t48*t56
      A0(1,3) = t53-t35*t65+t55*t56
      A0(1,4) = s12*t26*t35*t41*t44*(-3.0D0/2.0D0)-s12*t26*t35*t41*t45*(3.0D0/2.0D0)-s12*t26*t41*t42*t56*(3.0D0/2.0D0)
      A0(1,5) = s23*t26*t35*t41*t44*(-3.0D0/2.0D0)-s23*t26*t35*t41*t45*(3.0D0/2.0D0)-s23*t26*t41*t42*t56*(3.0D0/2.0D0)
      A0(1,6) = s13*t26*t35*t41*t44*(-3.0D0/2.0D0)-s13*t26*t35*t41*t45*(3.0D0/2.0D0)-s13*t26*t41*t42*t56*(3.0D0/2.0D0)
      A0(1,7) = t66+t71+t39*t42*(t34-t57)*(1.0D0/2.0D0)
      A0(2,1) = t58-t35*t60+t48*(t34-t57)
      A0(2,2) = t53+t61-t56*t63+1.0D0
      A0(2,3) = t58+t68-t35*t65
      A0(2,4) = s12*t26*t35*t41*t42*(-3.0D0/2.0D0)-s12*t26*t35*t41*t44*(3.0D0/2.0D0)-s12*t26*t41*t45*t56*(3.0D0/2.0D0)
      A0(2,5) = s23*t26*t35*t41*t42*(-3.0D0/2.0D0)-s23*t26*t35*t41*t44*(3.0D0/2.0D0)-s23*t26*t41*t45*t56*(3.0D0/2.0D0)
      A0(2,6) = s13*t26*t35*t41*t42*(-3.0D0/2.0D0)-s13*t26*t35*t41*t44*(3.0D0/2.0D0)-s13*t26*t41*t45*t56*(3.0D0/2.0D0)
      A0(2,7) = t66+t72+t39*t45*(t34-t57)*(1.0D0/2.0D0)
      A0(3,1) = t61-t35*t60+t55*(t34-t57)
      A0(3,2) = t61+t68-t35*t63
      A0(3,3) = t53+t58-t56*t65+1.0D0
      A0(3,4) = s12*t26*t35*t41*t42*(-3.0D0/2.0D0)-s12*t26*t35*t41*t45*(3.0D0/2.0D0)-s12*t26*t41*t44*t56*(3.0D0/2.0D0)
      A0(3,5) = s23*t26*t35*t41*t42*(-3.0D0/2.0D0)-s23*t26*t35*t41*t45*(3.0D0/2.0D0)-s23*t26*t41*t44*t56*(3.0D0/2.0D0)
      A0(3,6) = s13*t26*t35*t41*t42*(-3.0D0/2.0D0)-s13*t26*t35*t41*t45*(3.0D0/2.0D0)-s13*t26*t41*t44*t56*(3.0D0/2.0D0)
      A0(3,7) = t71+t72+t39*t44*(t34-t57)*(1.0D0/2.0D0)
      A0(4,1) = E*s12*t26*t29*t41*t42*(3.0D0/2.0D0)
      A0(4,2) = E*s12*t26*t29*t41*t45*(3.0D0/2.0D0)
      A0(4,3) = E*s12*t26*t29*t41*t44*(3.0D0/2.0D0)
      A0(4,4) = E*t29*(t73-t17*t26*t41*9.0D0)+1.0D0
      A0(4,5) = E*s12*s23*t26*t29*t41*(-9.0D0)
      A0(4,6) = E*s12*s13*t26*t29*t41*(-9.0D0)
      A0(4,7) = E*s12*t29*t39*3.0D0
      A0(5,1) = E*s23*t26*t29*t41*t42*(3.0D0/2.0D0)
      A0(5,2) = E*s23*t26*t29*t41*t45*(3.0D0/2.0D0)
      A0(5,3) = E*s23*t26*t29*t41*t44*(3.0D0/2.0D0)
      A0(5,4) = E*s12*s23*t26*t29*t41*(-9.0D0)
      A0(5,5) = E*t29*(t73-t21*t26*t41*9.0D0)+1.0D0
      A0(5,6) = E*s13*s23*t26*t29*t41*(-9.0D0)
      A0(5,7) = E*s23*t29*t39*3.0D0
      A0(6,1) = E*s13*t26*t29*t41*t42*(3.0D0/2.0D0)
      A0(6,2) = E*s13*t26*t29*t41*t45*(3.0D0/2.0D0)
      A0(6,3) = E*s13*t26*t29*t41*t44*(3.0D0/2.0D0)
      A0(6,4) = E*s12*s13*t26*t29*t41*(-9.0D0)
      A0(6,5) = E*s13*s23*t26*t29*t41*(-9.0D0)
      A0(6,6) = E*t29*(t73-t19*t26*t41*9.0D0)+1.0D0
      A0(6,7) = E*s13*t29*t39*3.0D0
      A0(7,1) = t42*t74*(-1.0D0/2.0D0)
      A0(7,2) = t45*t74*(-1.0D0/2.0D0)
      A0(7,3) = t44*t74*(-1.0D0/2.0D0)
      A0(7,4) = s12*t74*3.0D0
      A0(7,5) = s23*t74*3.0D0
      A0(7,6) = s13*t74*3.0D0
 end subroutine jacobian
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
 subroutine IDTasker(this, IDTask, IsUndr, &
					iStep, iTer, iEl, Int, &
					X, Y, Z, &
					Time0, dTime, &
					Props, Sig0, Swp0, StVar0,&
					dEps, D, BulkW, &
					Sig, Swp, StVar, ipl, &
					nStat,NonSym, iStrsDep, &
					iTimeDep, iTang, iAbort, IO)
!-------------------------------------------------------------------------------
! Import local scripts and local modules
  ! use toolBox
  use PLAXISroutines
  use precision
  use standardVars
  use VonMisesVars
!-------------------------------------------------------------------------------
! PLAXIS - Dictionary
!-------------------------------------------------------------------------------
!  Depending on IDTask, 1 : Initialize state variables
!                       2 : calculate stresses
!                       3 : calculate material stiffness matrix
!                       4 : return number of state variables
!                       5 : inquire matrix properties
!                           return switch for non-symmetric D-matrix
!                           stress/time dependent matrix
!                       6 : calculate elastic material stiffness matrix
!-------------------------------------------------------------------------------
 Implicit Double Precision (A-H, O-Z)
 class(VonMises) :: this
! Declare variable types
!-------------------------------------------------------------------------------
 INTEGER :: IDTask, IsUndr, iStep, iTer, iEl, Int
 INTEGER :: ipl,  nStat, NonSym, iStrsDep, iTimeDep, iTang, iAbort, IO
 real(kind=realDBL), DIMENSION(6,6) ::  D
 real(kind=realDBL), DIMENSION(6)   ::  Sig0, Sig, dEps
 real(kind=realDBL), DIMENSION(lenStateVars)  ::  StVar0, StVar
 real(kind=realDBL), DIMENSION(size(c))       ::  Props
 ! Local variables
 INTEGER ::  ne, nt, n, i, msf, iOpt
 real(kind=realDBL) :: dNorm, dEpsMax, dTimeMax
 real(kind=realDBL), DIMENSION(6)   ::  dAbsEps
!-------------------------------------------------------------------------------
!******************************* MODEL DESCRIPTION *****************************
!-------------------------------------------------------------------------------
! PLAXIS - IDTask
!-------------------------------------------------------------------------------
! Initialize state variables
 Task_1: if (IDTask == 1) then
     stateVars       =  0.0D0
     stateVars0      =  0.0D0
     stateVars(1:6)  = -1.0D0*Sig0(1:6)  ! Define compression as positive
     c               =  Props

     if (norm2(stateVars0) == 0) then
       call this%initialize()
       StVar0 = stateVars
     end if
  end if Task_1
!-------------------------------------------------------------------------------
 ! Calculate stresses
 Task_2: if (IDTask == 2) then
     ipl            = 0
     stateVars      = StVar0
     stateVars(1:6) = -Sig0(1:6)
     stateVars0     = stateVars
     dt             = dTime
     dEpsilon       = 0.D0
     dEpsilon(1:6)  = -1.0D0*dEps(1:6)
     state          = StVar0(lenStateVars + 1)
     c              = Props

     If (IsUndr == 1) then
       Swp = Swp0 + BulkW*(dEps(1) + dEps(2) + dEps(3))
     else
       Swp = Swp0
     end if

     dNorm = norm2(dEpsilon)

     if (dnorm /= 0) then
       dAbsEps  = abs(dEpsilon)
       dEpsMax  = max(dAbsEps(1), dAbsEps(2), dAbsEps(3), dAbsEps(4), dAbsEps(5), dAbsEps(6))
       dTimeMax = dt
       ne       = nint(demax/1.0D-3)
       nt       = nint(dtmax/1.0D3)

       if (n == 0) then
         n = 1
       end if

       if (nt == 0) then
         nt = 1
       end if

       n   = max(ne, nt)
       dEpsilon  = dEpsilon/n*1.0D0
       dt  = dt/n*1.0D0

       do i = 1, n
         call this%stressupdate()     ! Note: Updates stateVars
         stateVars0  = stateVars
         ipl         = state
        end do
     end if

     ! Store updated values
     Sig(1:6)              = -1.0D0*stateVars(1:6)
     StVar                 = stateVars
     StVar(lenStateVars+1) = state
  end if Task_2
!-------------------------------------------------------------------------------
 ! Calculate material stiffness matrix
 Task_3: if (IDTask == 3) then
     call this%DMatrix(D)
     !D = 2.0d0*D     ! TODO: Why 2x ?
     msf = Sig0(15)  ! TODO: What does this do??
  end if Task_3
!-------------------------------------------------------------------------------
 ! Number of state variables
 Task_4: if (IDTask == 4) then
     nStat = lenStateVars + 1
  end if Task_4
!-------------------------------------------------------------------------------
 ! Inquire matrix properties
 Task_5: if (IDTask == 5) then
     NonSym      =   0   ! 1 for non-symmetric D-matrix
     iStrsDep    =   0   ! 1 for stress dependent D-matrix
     iTang       =   0   ! 1 for tanget D-matrix
     iTimeDep    =   0   ! 1 for time dependent D-matrix
  end if Task_5
!-------------------------------------------------------------------------------
 ! Calculate elastic material stiffness matrix
 Task_6: if (IDTask == 6) then
     call this%DMatrix(D)
     !D = 2.0d0*D     ! TODO: Why 2x ?
  end if Task_6
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
end subroutine IDTasker
end module VonMisesClass
