module modelInfoVars
  !-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  Integer, parameter :: numberMods = 2
  Integer, parameter :: maxCharLen = 50

  Character (Len=maxCharLen), dimension(numberMods) :: names =    &
      ['Mod. CamClay', 'Von Mises']
  Integer, dimension(numberMods) :: numberParam =                 &
      [size(matrParam1(:,1)), size(matrParam2(:,1))]
  Integer, dimension(numberMods) :: numberOfStateVars =           &
      [size(matrStVars1(:,1)), size(matrStVars2(:,1))]

  ! CamClay
  Character (Len=maxCharLen), dimension(5,2) :: matrParam1 =              [&
      ['@j' , '@l', '@k', '@u', 'OCR'],          &
      ['deg',  '-', '-',  '-',  '-' ]           ]
  Character (Len=maxCharLen), dimension(8,2) :: matrStVars1 =             					[&
      ['S11'   ,    'S22',    'S33',    'S12',    'S23',    'S13', '@l',     'p_m' ], &
      ['F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#',  '-', 'F/L^2#'  ]  ]

	! NonLin. VonMises
  Character (Len=maxCharLen), dimension(3,2) :: matrParam2 =              [&
      ['S_u', 			'E', 	   '@u'],                          		   &
      ['F/L^2#',   'F/L^2#',  '-']      			                   ]
  Character (Len=maxCharLen), dimension(7,2) :: matrStVars2 =             [&
      ['S11',       'S22',    'S33',    'S12',    'S23',    'S13', '@l' ],  &
      ['F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#', 'F/L^2#',  '-' ]   ]

	!-----------------------------------------------------------------------------
  !-----------------------------------------------------------------------------
  contains
    function returnMatrParam(iMod, iParam, choice) result(output)
      integer :: iMod, iParam, choice
      Character (Len=255) :: output
      if (iMod==1 .and. (iParam<=size(matrParam1(:,1)))) then
        output = matrParam1(iParam, choice)
	  else if (iMod==2 .and. (iParam<=size(matrParam2(:,1)))) then
		output = matrParam2(iParam, choice)
	  else
        output = '???'
      end if
    end function returnMatrParam
  !-----------------------------------------------------------------------------
    function returnStVarParam(iMod, iVar, choice) result(output)
      integer :: iMod, iVar, choice
      Character (Len=255) :: output
      if (iMod==1 .and. (iVar<=size(matrStVars1(:,1))) ) then
        output = matrStVars1(iVar, choice)
	  else if (iMod==2 .and. (iParam<=size(matrStVars2(:,1)))) then
		    output = matrStVars2(iParam, choice)
      else
        output = '???'
      end if
    end function returnStVarParam
  !-----------------------------------------------------------------------------
  subroutine selectModel(  IDTask, iMod, IsUndr, &
                           iStep, iTer, iEl, Int, &
                           X, Y, Z, &
                           Time0, dTime, &
                           Props, Sig0, Swp0, StVar0, &
                           dEps, D, BulkW, &
                           Sig, Swp, StVar, ipl, &
                           nStat, NonSym, iStrsDep, &
                           iTimeDep, iTang, iAbort, IO  )
    ! Import soil models
    use precision
    use VonMisesClass
    use CamClayClass

    Implicit Double Precision (A-H,O-Z)

    INTEGER :: IDTask, iMod, IsUndr, iStep, iTer, iEl, Int
    INTEGER :: ipl,  nStat, NonSym, iStrsDep, iTimeDep, iTang, iAbort, IO
    real(kind=realDBL), DIMENSION(6,6) ::  D
    real(kind=realDBL), DIMENSION(6)   ::  Sig0, Sig, dEps
    real(kind=realDBL), DIMENSION(*)   ::  StVar0, StVar
    real(kind=realDBL), DIMENSION(*)   ::  Props

    type(CamClay)   :: material_1
	  type(VonMises)  :: material_2

    material_1 = CamClay()
	  material_2 = VonMises()

    Select Case (iMod)
      Case(1)
          call material_1%IDTasker(  IDTask, IsUndr, &
                                   iStep, iTer, iEl, Int, &
                                   X, Y, Z, &
                                   Time0, dTime, &
                                   Props, Sig0, Swp0, StVar0, &
                                   dEps, D, BulkW, &
                                   Sig, Swp, StVar, ipl, &
                                   nStat, NonSym, iStrsDep, &
                                   iTimeDep, iTang, iAbort, IO)
  	  Case(2)
  		    call material_2%IDTasker(  IDTask, IsUndr, &
                                     iStep, iTer, iEl, Int, &
                                     X, Y, Z, &
                                     Time0, dTime, &
                                     Props, Sig0, Swp0, StVar0, &
                                     dEps, D, BulkW, &
                                     Sig, Swp, StVar, ipl, &
                                     nStat, NonSym, iStrsDep, &
                                     iTimeDep, iTang, iAbort, IO)
      Case Default
        !Write(IO,*) 'invalid model number in UsrMod', iMod
        !Write(IO,*) 'IDTask: ',IDTask
        Stop 'invalid model number in UsrMod'
        iAbort=1
        Return
      end select
    end subroutine selectModel
  !-----------------------------------------------------------------------------
end module modelInfoVars
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
