clear all; close all; clc;

%% Montonic 1-cycle failure
S_u  = 50;
E    = 200;
alpha= 0.3;

q_a     = 2*S_u;
q       = 0:1:2*S_u;
eps     = 1/E*(q./(1 - q./q_a));

%% Contour variables
eps_rep = [0.05,0.1,0.25,0.5,1,5,15];
q_rep   = (E.*eps_rep.*q_a)./(q_a+E.*eps_rep);
tau_rep = q_rep/2;

a = 2;
b = 1;
c = 10;

N     =  1:1:10000;

%% Calculate contour
for i=1:length(tau_rep)
    Tau(i,:) = tau_rep(i)*(c+b).^(a/c)*(c*N+b).^(-a/c);
end

%% CYCLIC LOADING 
% Storm loading
%--------------------------------------------
N_parcel_1 = 1:1:100;
Tau_cy0_1  = 0.1*S_u;
Tau_cy0_1  = ones(1,length(N_parcel_1))*Tau_cy0_1;

Tau_cy0_2  = 0.2*S_u;
Neq1        = ((Tau_cy0_1(end)/Tau_cy0_2)^(c/a)*(c*N_parcel_1(end)+b)-b)/c;
N_parcel_2 = Neq1:1:(Neq1+50);
Tau_cy0_2  = ones(1,length(N_parcel_2))*Tau_cy0_2;
% 
Tau_cy0_3  = 0.3*S_u;
Neq2        = ((Tau_cy0_2(end)/Tau_cy0_3)^(c/a)*(c*N_parcel_2(end)+b)-b)/c;
N_parcel_3 = Neq2:1:(Neq2+30);
Tau_cy0_3  = ones(1,length(N_parcel_3))*Tau_cy0_3;

Tau_cy0_4  = 0.4*S_u;
Neq3        = ((Tau_cy0_3(end)/Tau_cy0_4)^(c/a)*(c*N_parcel_3(end)+b)-b)/c;
N_parcel_4 = Neq3:1:(Neq3+20);
Tau_cy0_4  = ones(1,length(N_parcel_4))*Tau_cy0_4;

Tau_cy0_5  = 0.5*S_u;
Neq4        = ((Tau_cy0_4(end)/Tau_cy0_5)^(c/a)*(c*N_parcel_4(end)+b)-b)/c;
N_parcel_5 = Neq4:1:(Neq4+10);
Tau_cy0_5  = ones(1,length(N_parcel_5))*Tau_cy0_5;

Tau_cy0  = [Tau_cy0_1,   Tau_cy0_2, Tau_cy0_3,  Tau_cy0_4,  Tau_cy0_5];
N_parcel = [N_parcel_1, N_parcel_2,N_parcel_3, N_parcel_4, N_parcel_5];

dKappa = Tau_cy0;
for i = 1:length(N_parcel)
    tau_0 = Tau_cy0(i)*(c*N_parcel(i)+b)^(a/c)/(c+b)^(-a/c);
    dKappa(i) = alpha*-a*(c+b)^(a/c)*tau_0*(c*N_parcel(i)+b)^(-a/c-1);
end


% High cyclic stress
% --------------------------------------------
N_parcel_1 = 1:1:3;
Tau_cy0_1  = 0.8*S_u;
Tau_cy0_1  = ones(1,length(N_parcel_1))*Tau_cy0_1;

Tau_cy01  = [Tau_cy0_1];
N_parcel1 = [N_parcel_1];

dKappa1 = Tau_cy01;
for i = 1:length(N_parcel1)
    tau_0 = Tau_cy01(i)*(c*N_parcel1(i)+b)^(a/c)/(c+b)^(-a/c);
    dKappa1(i) = alpha*-a*(c+b)^(a/c)*tau_0*(c*N_parcel1(i)+b)^(-a/c-1);
end

% Low cyclic stress
% --------------------------------------------
N_parcel_1 = 1:1:10000;
Tau_cy0_1  = 0.05*S_u;
Tau_cy0_1  = ones(1,length(N_parcel_1))*Tau_cy0_1;

Tau_cy02  = [Tau_cy0_1];
N_parcel2 = [N_parcel_1];

dKappa2 = Tau_cy02;
for i = 1:length(N_parcel2)
    tau_0 = Tau_cy02(i)*(c*N_parcel2(i)+b)^(a/c)/(c+b)^(-a/c);
    dKappa2(i) = alpha*-a*(c+b)^(a/c)*tau_0*(c*N_parcel2(i)+b)^(-a/c-1);
end

% Strain controlled
% --------------------------------------------
N_parcel_1 = 1:1:1000;
Tau_cy0_1  = tau_rep(end)*(c+b).^(a/c)*(c*N_parcel_1+b).^(-a/c);

Tau_cy03  = [Tau_cy0_1];
N_parcel3 = [N_parcel_1];

dKappa3 = Tau_cy03;
for i = 1:length(N_parcel3)
    tau_0 = Tau_cy03(i)*(c*N_parcel3(i)+b)^(a/c)/(c+b)^(-a/c);
    dKappa3(i) = alpha*-a*(c+b)^(a/c)*tau_0*(c*N_parcel3(i)+b)^(-a/c-1);
end


%% Plots
fig = figure('Position',[300,100,1200,800]);
subplot(2,2,3)
hold on; grid on;
plot(eps,q/2); 
plot(eps_rep, q_rep/2,'*');
axis([0,20,0,max(q/2)*1.2]);
xlabel('\gamma_{cy}'); ylabel('\tau_{cy}')
title('Shear-Strain - N=1')

subplot(2,2,1)
semilogx(N_parcel,Tau_cy0,'LineWidth',2,'color', 'black')
hold on
semilogx(N_parcel1,Tau_cy01, '--','LineWidth',2,'color', 'black')
semilogx(N_parcel2,Tau_cy02, '-.','LineWidth',2,'color', 'black')
semilogx(N_parcel3,Tau_cy03, '*', 'markersize', 2,'LineWidth',2,'color', 'black')

for i=1:length(q_rep)
    semilogx(N,Tau(i,:))
end

% tau_lol = 0.25*(c+b).^(a/c)*(c*N_parcel+b).^(-a/c);
% semilogx(N_parcel,tau_lol, '--','LineWidth',2 )
% semilogx(N_parcel4,Tau_cy04, 'color', 'black','LineWidth',2)
grid on;
legend('Storm load', '\tau_{cy}=0.8S_u', '\tau_{cy}=0.05S_u', '\gamma_{cy}=15%')
xlabel('N'); ylabel('\tau_{cy}')
title('Cyclic contour')

subplot(2,2,2)
loglog(N_parcel, dKappa)
hold on; grid on;
loglog(N_parcel1, dKappa1)
loglog(N_parcel2, dKappa2)
loglog(N_parcel3, dKappa3)
% loglog(N_parcel4, dKappa4)
legend('Storm load', '\tau_{cy}=0.8S_u', '\tau_{cy}=0.05S_u', '\gamma_{cy}=15%')
xlabel('N'); ylabel('\Delta\chi')
title('Degradation rate')

subplot(2,2,4)
semilogx(N_parcel,cumsum(dKappa))
hold on; grid on;
semilogx(N_parcel1,cumsum(dKappa1))
semilogx(N_parcel2,cumsum(dKappa2))
semilogx(N_parcel3,cumsum(dKappa3))
% semilogx(N_parcel4,cumsum(dKappa4))
set(gca,'Ydir','reverse')
legend('Storm load', '\tau_{cy}=0.8S_u', '\tau_{cy}=0.05S_u', '\gamma_{cy}=15%')
xlabel('Accumulated N'); ylabel('Accumulated \chi')
title('Degradation accumulation')
