clear all; close all; clc;

%% Montonic 1-cycle failure
S_u  = 50;
E    = 200;
alpha= 0.3;

q_a     = 2*S_u;
q       = 0:1:2*S_u;
eps     = 1/E*(q./(1 - q./q_a));

%% Contour variables
eps_rep = [0.05,0.1,0.25,0.5,1,5,15];
q_rep   = (E.*eps_rep.*q_a)./(q_a+E.*eps_rep);
tau_rep = q_rep/2;

a = 2;
b = 1;
c = 10;

N     =  1:1:10000;

%% Calculate contour
for i=1:length(tau_rep)
    Tau(i,:) = tau_rep(i)*(c+b).^(a/c)*(c*N+b).^(-a/c);
end

%% Control of degradation equiv.
% First load level
N_parcel_1 = 1:1:1000;
Tau_cy0_1  = 0.1*S_u;
Tau_cy0_1  = ones(1,length(N_parcel_1))*Tau_cy0_1;

Tau_cy0  = [Tau_cy0_1];
N_parcel = [N_parcel_1];

dKappa = Tau_cy0;
for i = 1:length(N_parcel)
    tau_0 = Tau_cy0(i)*(c*N_parcel(i)+b)^(a/c)/(c+b)^(-a/c);
    dKappa(i) = alpha*-a*(c+b)^(a/c)*tau_0*(c*N_parcel(i)+b)^(-a/c-1);
end

% Next load level
Tau_cy0_1  = 0.25*S_u;

Neq        = ((Tau_cy0(end)/Tau_cy0_1)^(c/a)*(c*N_parcel(end)+b)-b)/c;
N_parcel_1 = 1:1:Neq;

Tau_cy0_1  = ones(1,length(N_parcel_1))*Tau_cy0_1;
Tau_cy04  = [Tau_cy0_1];
N_parcel4 = [N_parcel_1];


dKappa4 = Tau_cy04;
for i = 1:length(N_parcel4)
    tau_0 = Tau_cy04(i)*(c*N_parcel4(i)+b)^(a/c)/(c+b)^(-a/c);
    dKappa4(i) = alpha*-a*(c+b)^(a/c)*tau_0*(c*N_parcel4(i)+b)^(-a/c-1);
end


%% Plots
fig = figure('Position',[300,100,1200,800]);
subplot(1,2,1)
semilogx(N_parcel,Tau_cy0,'LineWidth',2,'color', 'black')
hold on
semilogx(N_parcel4,Tau_cy04, 'color', 'blue','LineWidth',2)
for i=1:length(q_rep)
    semilogx(N,Tau(i,:))
end
grid on;
legend('\tau_{cy}=0.1S_u', '\tau_{cy}=0.25S_u')
xlabel('N'); ylabel('\tau_{cy}')
title('Cyclic contour')

subplot(1,2,2)
semilogx(N_parcel,cumsum(dKappa))
hold on; grid on;
semilogx(N_parcel4,cumsum(dKappa4))
set(gca,'Ydir','reverse')
legend('\tau_{cy}=0.1S_u', '\tau_{cy}=0.25S_u')
xlabel('Accumulated N'); ylabel('Accumulated \chi')
title('Degradation accumulation')
