localhostport_output = 10001
plaxis_path = r'C:\Program Files (x86)\Plaxis\PLAXIS 2D' #no trailing backslash!

import imp
found_module = imp.find_module('plxscripting', [plaxis_path])
plxscripting = imp.load_module('plxscripting', *found_module)
from plxscripting.easy import *

s_o, g_o = new_server('localhost', localhostport_output)

#geometric limits for the left retaining wall:                    
x_left = 40.0

#for all phases, starting from the second phase (Phase_1)
for phase in g_o.Phases[1:]:
    #initialize defaults:
    maxM = 0.0
    xAtMaxM = 0.0
    yAtMaxM = 0.0
    minM = 0.0
    xAtMinM = 0.0
    yAtMinM = 0.0

    #obtain result tables
    plateX = g_o.getresults(phase, g_o.ResultTypes.Plate.X, 'node')
    plateY = g_o.getresults(phase, g_o.ResultTypes.Plate.Y, 'node')
    plateM = g_o.getresults(phase, g_o.ResultTypes.Plate.M2D, 'node')

    #determine minimum and maximum bending moment:
    for x, y, M in zip(plateX, plateY, plateM):
        #is it on the left wall (with small numerical tolerance)?
        if abs(x - x_left) < 1E-5:
            if M > maxM:
                maxM = M
                xAtMaxM = x
                yAtMaxM = y
            if M < minM:
                minM = M
                xAtMinM = x
                yAtMinM = y

    print( "{}: ".format( phase.Name ) +
           "Mmax = {:.2f} kNm/m at Y={:.2f} m; ".format(maxM, yAtMaxM) +
           "Mmin = {:.2f} kNm/m at Y={:.2f} m".format(minM, yAtMinM) )
           
          