# TODO: PLAXIS path determined from user

from PyQt4.uic import loadUiType
from PyQt4 import QtGui, QtCore

import os, subprocess, imp, ctypes, time
import numpy as np

from matplotlib.figure import Figure
from matplotlib.backends.backend_qt4agg import (
    FigureCanvasQTAgg as FigureCanvas,
    NavigationToolbar2QT as NavigationToolbar)


Ui_MainWindow,  QMainWindow = loadUiType("degrader.ui")
Ui_PopupWindow,QPopupWindow = loadUiType("popupWindow.ui")
Ui_LoadWindow,  QLoadWindow = loadUiType("loadWindow.ui")
Ui_MatrWindow,  QMatrWindow = loadUiType("matrWindow.ui")
Ui_NumWindow,    QNumWindow = loadUiType("numWindow.ui")

class Degrader(QMainWindow, Ui_MainWindow):
    
    def __init__(self):
        super(Degrader, self).__init__()
        self.setupUi(self)
        
        self.create_Canvas()
        try: # Remove files in case of improper shutdown of script
            self.removefiles_phaseInfo()
        except:
            pass
        
        self.btnStartPlaxis.clicked.connect(self.start_Plaxis)
        self.btnPrjDir.clicked.connect(self.get_Directory)
        self.btnModel_1.clicked.connect(lambda: self.create_Model(1))
        self.btnModel_2.clicked.connect(lambda: self.create_Model(2))
        self.btnModel_3.clicked.connect(lambda: self.create_Model(3))
        self.btnMaterial_1.clicked.connect(lambda: self.create_Material(1))
        self.btnMaterial_2.clicked.connect(lambda: self.create_Material(2))
        self.btnLoad.clicked.connect(self.define_Load)
        self.btnPoint.clicked.connect(self.select_curvePoint)
        self.btnNumeric.clicked.connect(self.define_numerical)
        self.btnCalc.clicked.connect(self.calculate_phases)
        self.btnConnect.clicked.connect(self.connect_Plaxis)
        
        self.btnPlot_figures.setEnabled(False)
        self.btnPlot_curves.setEnabled(False)

        # TODO: Remove this when done
#         self.load = np.ones((1,3))
#         self._folder = r'E:\OneDrive - NTNU\4-Master\4. PLAXIS'
#         self._avg_param = [('MaterialName', 'Average Material'), ('Comments', ''), ('SoilModel', 100), ('DrainageType', 'Undrained A'), ('gammaUnsat', 20.0), ('gammaSat', 20.0), ('UserDLLName', 'Average.dll'), ('UserModel', 'Mod. CamClay with destr.'), ('perm_primary_horizontal_axis', 1.0), ('perm_vertical_axis', 1.0), ('InterfaceStrength', 1), ('Gref', 1), ('cref', 1), ('K0PrimaryIsK0Secondary', False), ('K0Determination', 0), ('K0Primary', 1.0), ('K0Secondary', 1.0), ['User1', 27.0], ['User2', 0.05], ['User3', 0.025], ['User4', 0.2], ['User5', 1.4], ['User6', 0.5], ['User7', 0.5], ['User8', 20.0]]
#         
# self._cyc_param = [('MaterialName', 'Cyclic Material'), ('Comments', ''), ('SoilModel', 100), ('DrainageType', 'Undrained A'), ('gammaUnsat', 0), ('gammaSat', 0), ('UserDLLName', 'Cyclic.dll'), ('UserModel', 'Nonlin Von Mises'), ('perm_primary_horizontal_axis', 1.0), ('perm_vertical_axis', 1.0), ('InterfaceStrength', 1), ('Gref', 1), ('cref', 1), ('K0PrimaryIsK0Secondary', False), ('K0Determination', 0), ('K0Primary', 1.0), ('K0Secondary', 1.0), ['User1', 50000.0], ['User2', 3000000.0], ['User3', 0.3]]
#         k0x = 1.0
#         self._load_param = [('MaterialName', 'Stress generation material'),('SoilModel', 1),('DrainageType', 0),('gammaUnsat', 20),('gammaSat', 20),('Gref', 43000),('nu', k0x/(1+k0x))]
         
    def create_Canvas(self): 
        self.fig_phase = Figure()
        self.fig_load  = Figure()
        self.fig_phase.set_figheight(4)
        self.fig_phase.set_figwidth(10)
                
        self.canvas_phase = FigureCanvas(self.fig_phase)
        self.canvas_load  = FigureCanvas(self.fig_load)
        
        widget = self.phaseGrid.addWidget(self.canvas_phase)
        self.loadGrid.addWidget(self.canvas_load)
        
        self.scroll = QtGui.QScrollArea(widget)
        self.scroll.setWidget(self.canvas_phase)
        #self.scroll.verticalScrollBar().setEnabled(True)
        self.scroll.setVerticalScrollBarPolicy(1)
        self.phaseGrid.layout().addWidget(self.scroll)
        
        #self.plot_Phases()
    
    def start_Plaxis(self):
        try: 
            plaxisPath = r"C:\Program Files (x86)\Plaxis\PLAXIS 2D"
            serverPort = 10000
            
            found_module = imp.find_module('plxscripting', [plaxisPath])
            plxscripting = imp.load_module('plxscripting', *found_module)
            
            from plxscripting.easy import new_server
            self._new_server = new_server
            
            # Starts PLAXIS.exe
            args = [os.path.join(plaxisPath, "Plaxis2DXInput.exe"), "--appServerPort={}".format(serverPort)]
            self._input_process = subprocess.Popen(args)
            
            #subprocess.Popen(['C:\\Program Files (x86)\\Plaxis\\PLAXIS 2D\\Plaxis2dXInput.exe', "--AppServerPort="+str(serverPort)])
            
            # Creates instances of the PLAXIS GUI and Model
            self.GUI_input, self.model = self._new_server('localhost', serverPort, timeout=10.0)
            self.GUI_input.new()
        except:
            text = 'Error: PLAXIS could not be opened!'
            self.infoWindow = PopupWindow(text)
            self.infoWindow.show()   
            
    def connect_Plaxis(self):
        try: 
            plaxisPath = r"C:\Program Files (x86)\Plaxis\PLAXIS 2D"
            serverPort = 10000
            
            found_module = imp.find_module('plxscripting', [plaxisPath])
            plxscripting = imp.load_module('plxscripting', *found_module)
            
            from plxscripting.easy import new_server
            self._new_server = new_server
            self.GUI_input, self.model = self._new_server('localhost', serverPort, timeout=10.0)
        except:
            text = 'Error: PLAXIS could not be opened!'
            self.infoWindow = PopupWindow(text)
            self.infoWindow.show()
            
    def get_Directory(self):
        self._folder = QtGui.QFileDialog.getExistingDirectory(self, "Select Directory")

    def create_Model(self, choice):
        try:
            self.GUI_input.new()
            CustomFEMmodel(self.model, choice)
        except:
            text = 'Error: Could not connect to PLAXIS.'
            self.infoWindow = PopupWindow(text)
            self.infoWindow.show()   

    def create_Material(self, choice):
        self.matrWindow = MatrWindow(self, choice)
        self.matrWindow.show()

    def define_Load(self):
        self.loadWindow = LoadWindow(self)
        self.loadWindow.show()
        
    def plot_Load(self):
        fig    = self.fig_load
        canvas = self.canvas_load
        ax     = fig.add_subplot(111)
        
        ax.cla()
        
        load    = self.load
        lenLoad = len(load[:,0])
        
        dataY1 = np.zeros(lenLoad*2)
        dataY2 = np.zeros(lenLoad*2)
        dataX  = np.zeros(lenLoad*2)
        
        j = 0
        for i in range(lenLoad):
            dataY1[j]   = load[i,0]
            dataY1[j+1] = load[i,0]
            dataY2[j]   = load[i,1]
            dataY2[j+1] = load[i,1]
            dataX[j]    = dataX[j-1]
            dataX[j+1]  = dataX[j]+load[i,2]
            j = j+2
                            
        ax.plot(dataX, dataY1, label='Average')
        ax.plot(dataX, dataY2, label='Cyclic')
        ax.set_xlabel("N [-]")
        ax.xaxis.set_label_coords(1.05, 0)
        ax.set_ylabel("F [kN]")
        ax.grid(True)
        
        ax.legend(loc=4)
        
        xLimits = ax.get_xlim()
        yLimits = ax.get_ylim()
        ax.set_xlim([0,xLimits[1]*1.2])
        ax.set_ylim([0,yLimits[1]*1.2])
        
        canvas.draw()
    
    def plot_Phases(self):
        from numpy.random import rand
        fig    = self.fig_phase
        canvas = self.canvas_phase
        ax     = fig.add_subplot(111)
        #fig.subplots_adjust(left=0,right=1,bottom=0,top=1)
        fig.tight_layout()
        
        # ax.spines['left'].set_position('center')
        # ax.spines['right'].set_color('black')
        # #ax.spines['bottom'].set_position('bottom')
        # ax.spines['top'].set_color('black')
        # ax.spines['left'].set_smart_bounds(True)
        # ax.spines['bottom'].set_smart_bounds(True)
        
        
        
        x = [0, 0,  0,1000]
        y = [0,10,-10,0]
        #x, y, c, s = rand(4, 100)
        def onpick3(event):
            ind = event.ind
            textstr = 'onpick3 scatter:', ind, np.take(x, ind), np.take(y, ind)
            print(textstr)
            props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
            ax.text(0.05, 0.95, textstr, transform=ax.transAxes, fontsize=14, verticalalignment='top', bbox=props)
                
        col = ax.scatter(x, y, picker=True)#, 100*s, c, picker=True)
        #fig.savefig('pscoll.eps')
        fig.canvas.mpl_connect('pick_event', onpick3)
        canvas.draw()
        
    def define_numerical(self):
        self.numWindow = NumericalWindow(self)
        self.numWindow.show()

    def select_curvePoint(self):
        try:
            self.model.gotomesh() 
            output_port = self.model.selectmeshpoints()
            s_output, g_output = self._new_server('localhost', output_port)
            self.model.gotostages()
        except:
            text = 'Error: Could not connect to PLAXIS.'
            self.infoWindow = PopupWindow(text)
            self.infoWindow.show() 
        
    def calculate_phases(self):
        try:
            # Stress generation and apply materials.
            self._avgMatr = self.model.soilmat(*self._avg_param)
            self._cycMatr = self.model.soilmat(*self._cyc_param)
            self._loadMatr = self.model.soilmat(*self._load_param)
            for phase in self.model.phases[:]:
                for soilPoly in self.model.SoilPolygons:
                    if phase == self.model.phases[0] or phase == self.model.phases[2]:
                        self.model.setmaterial(soilPoly, phase, self._cycMatr)
                    elif phase == self.model.phases[1]:
                        self.model.setmaterial(soilPoly, phase, self._loadMatr)
                    else:
                        self.model.setmaterial(soilPoly, phase, self._avgMatr)
            self.model.calculate()
            
            
            # Cylcic degradation iterations
            for i in range(len(self.load[:,0])):
                #--------------------- Average --------------------
                average_phase = self.model.phase(self.model.phases[-1])
                average_phase.Identification = "Avg " + str(i+1) + "_0"
                average_phase.Deform.ResetDisplacementsToZero = True
                self.model.activate(self.model.pointloads[:], average_phase)
                self.model.PointLoads[0].Fx[average_phase] = self.load[i,0]-1    # Avoid load procedure error
                self.model.calculate()
                if not(self.checkCalcResult()):
                    break
                time.sleep(1)
                self.writeToFile_phaseInfo(1)
                
                
                cyclicIter = self._numericParam['intermediateIter']
                if cyclicIter>1:
                    for j in range(cyclicIter):
                        #---------------------- Cyclic ---------------------
                        cyclic_phase = self.model.phase(self.model.phases[2])
                        cyclic_phase.Identification = "Cyc " + str(i+1) + "_" + str(j+1)
                        self.model.activate(self.model.PointLoads[:],cyclic_phase)
                        cyclic_phase.TimeInterval = self.load[i,2]
                        cyclic_phase.Deform.ResetDisplacementsToZero = True
                        self.model.PointLoads[0].Fx[cyclic_phase]  = self.load[i,1]
                        self.model.calculate()
                        if not(self.checkCalcResult()):
                            break
                        time.sleep(1)
                        self.writeToFile_phaseInfo(0)
                        
                        #--------------------- Average --------------------
                        average_phase = self.model.phase(self.model.phases[4+(2*cyclicIter+1)*i])
                        average_phase.Identification = "Avg " + str(i+1) + "_" + str(j+1)
                        self.model.activate(self.model.pointloads[:], average_phase)
                        self.model.PointLoads[0].Fx[average_phase] = self.load[i,0]
                        self.model.calculate()
                        if not(self.checkCalcResult()):
                            break
                        time.sleep(1)
                        self.writeToFile_phaseInfo(0)
                else:
                    #---------------------- Cyclic ---------------------
                    cyclic_phase = self.model.phase(self.model.phases[2])
                    cyclic_phase.Identification = "Cyc " + str(i+1) + "_1"
                    self.model.activate(self.model.PointLoads[:],cyclic_phase)
                    cyclic_phase.TimeInterval = self.load[i,2]
                    cyclic_phase.Deform.ResetDisplacementsToZero = True
                    self.model.PointLoads[0].Fx[cyclic_phase]  = self.load[i,1]
                    self.model.calculate()
                    if not(self.checkCalcResult()):
                        break
                    time.sleep(1)
                    self.writeToFile_phaseInfo(0)                        
                    
                    #--------------------- Average --------------------
                    average_phase = self.model.phase(self.model.phases[3+3*i])
                    average_phase.Identification = "Avg " + str(i+1) + "_1"
                    self.model.activate(self.model.pointloads[:], average_phase)
                    self.model.PointLoads[0].Fx[average_phase] = self.load[i,0]
                    self.model.calculate()
                    if not(self.checkCalcResult()):
                        break
                    time.sleep(1)
                    self.writeToFile_phaseInfo(0)

                time.sleep(1)
                self.removefiles_phaseInfo()
            #self.fig_phase.set_figwidth(10+i*5?)
            self.model.save(self._folder+"\\DegradationCalc")
            
        except:
            text = 'Error: Could not connect to PLAXIS. \n Make sure steps 1-4 are done'
            self.infoWindow = PopupWindow(text)
            self.infoWindow.show() 
    
    def checkCalcResult(self):
        calcResult = self.model.phases[-1].CalculationResult.echo()
        calcResult = calcResult[-6:-4]
        if ('OK' in calcResult):
            OK = True
        else:
            OK = False
        return OK
            
    
    def writeToFile_phaseInfo(self, switch):
        LocalAppdata = os.getenv('LocalAPPDATA')
        plaxisTempFolder = LocalAppdata+"\\Temp\PLAXIS~1\\"
        
        filesInFolder = os.listdir(plaxisTempFolder)
        stVarFiles    = []
        for file in filesInFolder:
            if 'data.0' in file:
                stVarFiles.append(file)
        
        # Contains the name of the last data.00x
        text_file = open(plaxisTempFolder+'lastPhase.txt', "w")
        text_file.write(stVarFiles[-1])
        text_file.close()
        
        if switch == 1:
            # Store data.00x of same material statevariables
            text_file = open(plaxisTempFolder+'lastInitAvgPhase.txt', "w")
            text_file.write(stVarFiles[-1])
            text_file.close()
        
        # Reset Gauss counter 
        text_file = open(plaxisTempFolder+'gp.txt', "w")
        text_file.write('1')
        text_file.close()

        # Store initial values from average model
        text_file = open(plaxisTempFolder+'avgInput.txt', "w")
        text_file.write(str(self._avg_param[-1][-1])) # Here goes data of interest, ex. initial structure
        text_file.close()
        
        # Store binary statevariables for next calculation
        try:
            os.rename(plaxisTempFolder+'stVars.bin',plaxisTempFolder+'stVarsPrev.bin')
        except:
            pass
                
    def removefiles_phaseInfo(self):
        try:
            LocalAppdata = os.getenv('LocalAPPDATA')
            plaxisTempFolder = LocalAppdata+"\\Temp\PLAXIS~1\\"
            os.remove(plaxisTempFolder+'lastPhase.txt')
            os.remove(plaxisTempFolder+'lastSamePhase.txt')
            os.remove(plaxisTempFolder+'gp.txt')
            os.remove(plaxisTempFolder+'RECL.txt')
            os.remove(plaxisTempFolder+'stVars.bin')
            os.remove(plaxisTempFolder+'stVarsPrev.bin')
            os.remove(plaxisTempFolder+'avgInput.txt')
        except:
            pass
        
                
        
class CustomFEMmodel(object):
    # TODO: Define custom (choice == 3)
    def __init__(self, plaxisModel, choice):
        param = self.set_Param(choice)
        self.add_soil_layers     (plaxisModel, param)
        self.add_structures      (plaxisModel, param)
        self.apply_plate_material(plaxisModel)
        self.mesh                (plaxisModel)
        self.initialPhase        (plaxisModel)
        
    def add_soil_layers(self, plaxisModel, param):
        plaxisModel.SoilContour.initializerectangular(-param['B']/2, -param['H'], param['B']/2, 0)
        
        borehole = plaxisModel.borehole(-param['B']/2)
        plaxisModel.soillayer(param['H'])
        borehole.Head = -param['H']-1
        
    def mesh(self, plaxisModel):
        plaxisModel.gotomesh() 
        plaxisModel.mesh(0.06)
        #plaxisModel.mesh(0.1)
        #plaxisModel.mesh(0.5)
    
    def add_structures(self, plaxisModel, param):
        plaxisModel.line(-param['b']/2, -param['d']/2,  param['b']/2, -param['d']/2) # Bottom
        plaxisModel.line(-param['b']/2,  param['h']/2,             0,  param['h']/2) # Top 1
        plaxisModel.line(            0,  param['h']/2,  param['b']/2,  param['h']/2) # Top 2
        plaxisModel.line(-param['b']/2, -param['d']/2, -param['b']/2,  param['h']/2) # Left
        plaxisModel.line( param['b']/2, -param['d']/2,  param['b']/2,  param['h']/2) # Right
        
        plaxisModel.lineload (plaxisModel.Lines[0], "qy_start", -param['w'])
        plaxisModel.plate    (plaxisModel.Lines[:])
        plaxisModel.pointload(plaxisModel.points[4], "Fx",1, "Fy",0)
        
        plaxisModel.SoilLayers # Creates a handle for soil layer
        
        # Add interface??
    
    def apply_plate_material(self, plaxisModel):
        steel = [
        ('MaterialName', 'Plate'),
        ('Colour', 16711680),
        ('Elasticity', 0),
        ('IsIsotropic', True),
        ('IsEndBearing', True),
        ('EA', 1.0*10**12),
        ('EI', 1.0*10**9),
        ('nu', 0.15),
        ('d', 0.34641),
        ('w', 0),
        ('Mp', 1000000000000000.0),
        ('Np', 10000000000.0),
        ('Np2', 10000000000.0),
        ('RayleighAlpha', 0),
        ('RayleighBeta', 0),
        ('Gref', 15061311)
        ]
        
        steel_material = plaxisModel.platemat(*steel)
        
        for plate in plaxisModel.Plates:
            plaxisModel.setmaterial(plate, steel_material)
    
    def initialPhase(self, plaxisModel):
        plaxisModel.gotostages()
        plaxisModel.set(plaxisModel.GroundwaterFlow.BoundaryXMin,plaxisModel.phases[-1] ,"Closed")
        plaxisModel.set(plaxisModel.GroundwaterFlow.BoundaryXMax,plaxisModel.phases[-1] ,"Closed")
        loadPhase = plaxisModel.phase(plaxisModel.phases[-1])
        loadPhase.Identification = "Generate stress"
        
        buildPhaseCyclic = plaxisModel.phase(plaxisModel.phases[0])
        buildPhaseCyclic.Identification = "Install of construction - Cyclic"
        plaxisModel.activate(plaxisModel.lines[:], buildPhaseCyclic)
        plaxisModel.deactivate(plaxisModel.SoilPolygons[0], buildPhaseCyclic)
        plaxisModel.deactivate(plaxisModel.lineLoads[:], buildPhaseCyclic)
        
        buildPhaseAverage = plaxisModel.phase(plaxisModel.phases[-2])
        plaxisModel.activate(plaxisModel.lines[:], buildPhaseAverage)
        plaxisModel.deactivate(plaxisModel.SoilPolygons[0],buildPhaseAverage)
        buildPhaseAverage.Identification = "Install of construction - Average"
        buildPhaseAverage.Deform.ResetDisplacementsToZero = True
        buildPhaseAverage.Deform.IgnoreUndrainedBehaviour = True

        
    def set_Param(self, choice):
        if choice == 1:
            H = 5*2
            B = 25*2
            h = 10
            d = 2
            b = 8
            w = 10
        elif choice == 2:
            H = 10*2
            B = 20*2
            h = 10
            d = 10
            b = 1
            w = 10
            
        param = {'H':H, 'B':B, 'b':b, 'h':h, 'd':d, 'w':w}
        return param
    
class PopupWindow(QPopupWindow, Ui_PopupWindow):
    def __init__(self, string):
        super(PopupWindow, self).__init__()
        
        self.setupUi(self)
        self.setWindowTitle("Error")
        
        self.textBox.setAlignment(QtCore.Qt.AlignCenter)
        self.textBox.setText(string)
        self.textBox.setWordWrap(True)
        
        self.btnClose.clicked.connect(self.btn_Close)
        
    def btn_Close(self):
        self.close()
        
class LoadWindow(QLoadWindow, Ui_LoadWindow):
    def __init__(self, mainWindow):
        super(LoadWindow, self).__init__()
        
        self.setupUi(self)
         
        # Set text in middle
        self.setup_Table()
        
        self.btnAddLoad.clicked.connect(self.addLoad_Parcel)
        self.btnDelLoad.clicked.connect(self.delLoad_Parcel)
        self.buttonBox.accepted.connect(lambda: self.save_Load(mainWindow))
        self.buttonBox.rejected.connect(self.exit)
        
        self.tableWidget.cellClicked.connect(self.highLight_Row)
    
        
    def setup_Table(self):
        # -*- coding: utf8 -*-
        # TODO: Format so it looks better
        self.tableWidget.setHorizontalHeaderLabels(["Favg [kN]", "Fcyc [kN]", "Neq [-]"]) 
        self.addLoad_Parcel()
            
            
    def save_Load(self, mainWindow):
        row = self.tableWidget.rowCount()
        col = self.tableWidget.columnCount()
        
        data = np.zeros((row,col))
        
        for i in range(row):
            for j in range(col):
                value = self.tableWidget.item(i,j).text()
                data[i,j] = float(value)           
        i = 0
        while i < len(data[:,1]):
            if (data[i,2] == 0.0):
                data = np.delete(data, i, 0)
                i = i -1 
            i = i + 1

        mainWindow.load = data
        mainWindow.plot_Load()
        self.close()
        
    def exit(self):
        self.close()

    def addLoad_Parcel(self):
        self.tableWidget.insertRow(self.tableWidget.rowCount())
        row = self.tableWidget.rowCount()-1     # Must subtract 1 to be right
        col = self.tableWidget.columnCount()
        for i in range(col):
            item = QtGui.QTableWidgetItem('0.0')
            item.setTextAlignment(QtCore.Qt.AlignCenter);
            self.tableWidget.setItem(row,i,item)
        
    def delLoad_Parcel(self):
        row = self.tableWidget.currentRow()
        self.tableWidget.removeRow(row)

    def highLight_Row(self, item):
        self.tableWidget.selectRow(item)
        self.tableWidget.setFocus()


class MatrWindow(QMatrWindow, Ui_MatrWindow):
    def __init__(self, mainWindow, choice):
        super(MatrWindow, self).__init__()
        
        self.setupUi(self)
        self.setup_DLL()
        
        self.btnInterface.setEnabled(False) #TODO: Add functionality
        if choice == 1:
            self.comboDrain.setCurrentIndex(1)
        elif choice == 2:
            self.lineWeight.setEnabled(False)
            self.lineKx.setEnabled(False)
            self.comboDrain.setCurrentIndex(1)
            self.comboDrain.setEnabled(False)
            
            
        self.btnBox.accepted.connect(lambda: self.save_material(mainWindow, choice))
        self.btnBox.rejected.connect(self.exit)
        self.btnInterface.clicked.connect(self.define_Interface)
        
        self.checkIsoK.toggled.connect(self.update_Flow_GUI)
        self.checkIsoK0.toggled.connect(self.update_K0_GUI)

        self.comboFile.activated.connect(self.selection_DLL)
        self.comboModel.activated.connect(self.selection_MODEL)
     
    def setup_DLL(self):
        plaxisPath = r"C:\Program Files (x86)\Plaxis\PLAXIS 2D\udsm"
        dllFiles = []
        for i in os.listdir(plaxisPath): # Note: i = name[1,2,3,...]
            if '64' in i and '.dll' in i or '.DLL' in i :
            #if '64' not in i and '.dll' in i or '.DLL' in i :
                dllFiles.append(i)
        self.comboFile.addItems(dllFiles)
    
    def selection_DLL(self, i):
        self.comboModel.clear()
        for k in reversed(range(self.gridUserParam.count())):  
            self.gridUserParam.itemAt(k).widget().setParent(None)
            
        self._file = r"C:\Program Files (x86)\Plaxis\PLAXIS 2D\udsm"+"\\"+ self.comboFile.itemText(i)

        DLL        = ctypes.WinDLL(self._file)
        nMod       = ctypes.c_int()
        ModelName  = ModelName = ctypes.create_string_buffer(255)
        modelNames = ['']
        
        DLL.getmodelcount(ctypes.byref(nMod))
        
        for i in range(nMod.value):
            iMod    = ctypes.c_int(i+1)
            DLL.getmodelname(ctypes.byref(iMod), ctypes.byref(ModelName), ctypes.sizeof(ModelName))
            modelNameUnicode = ModelName.value.decode('utf-8', 'ignore')  
            modelName = " ".join(modelNameUnicode[1:].split())
            modelNames.append(modelName)

        self.comboModel.addItems(modelNames)
    
    def selection_MODEL(self, i):
        for k in reversed(range(self.gridUserParam.count())): 
            self.gridUserParam.itemAt(k).widget().setParent(None)
        
        DLL       = ctypes.WinDLL(self._file)
        iMod      = ctypes.c_int(i)
        nParam    = ctypes.c_int()
        userLines = []
        userUnits = []

        DLL.getparamcount(ctypes.byref(iMod), ctypes.byref(nParam))
                
        for j in range(nParam.value+1):
            iParam    = ctypes.c_int(j)
            ParamName = ctypes.create_string_buffer(255)
            Units     = ctypes.create_string_buffer(255)
        
            DLL.getparamname(ctypes.byref(iMod), ctypes.byref(iParam), ctypes.byref(ParamName), ctypes.sizeof(ParamName))
            DLL.getparamunit(ctypes.byref(iMod), ctypes.byref(iParam), ctypes.byref(Units), ctypes.sizeof(Units))
            
            paramNameUniCode = ParamName.value.decode('utf-8', 'ignore')  
            paramName = " ".join(paramNameUniCode[1:].split())
            
            unitsUniCode = Units.value.decode('utf-8', 'ignore')  
            units = " ".join(unitsUniCode[1:].split())
            userUnits.append(units)
            
            string = paramName + '\t[ '+units+' ]'
            
            if j > 0:
                label = QtGui.QLabel(string)
                label.setFixedWidth(150)
                self.gridUserParam.addWidget(label, j, 0)
                userLine = QtGui.QLineEdit()
                self.gridUserParam.addWidget(userLine, j, 1)
                userLine.setAlignment(QtCore.Qt.AlignCenter)
                userLines.append(userLine)

        self._userLines  = userLines 
        self._userUnits  = userUnits
         
    def update_Flow_GUI(self, item):
        if not(item):
            self.lineKy.setEnabled(True)
        else:
            self.lineKy.setEnabled(False)
            self.lineKy.setText('')
        
    def update_K0_GUI(self, item):
        if not(item):
            self.lineK0y.setEnabled(True)
        else:
            self.lineK0y.setEnabled(False)
            self.lineK0y.setText('')
            
    def define_Interface(self):
        print('Set interface')
            
    def save_material(self, mainWindow, choice):
        IdName  = self.lineID.text()
        if IdName == '' and choice == 1:
            IdName = 'Average Material'
        elif IdName == '' and choice ==2:
            IdName = 'Cyclic Material'
        Drain   = self.comboDrain.currentText() # .currentIndex()
        Comment = self.lineComment.text()
        
        Weight = self.lineWeight.text().replace(",", ".")
        if Weight == '' and choice == 1:
            text = 'Error: The weight can not be zero!'
            self.infoWindow_1 = PopupWindow(text)
            self.infoWindow_1.show()
        elif Weight == '' and choice == 2:
            Weight = 0
        else:
            Weight = float(Weight)
                
        File    = self.comboFile.currentText()
        File    = File.replace("64", "")
        Model   = self.comboModel.currentText()
        
        userParams = []
        for i in range(len(self._userLines)):
            # Error in PLAXIS with units: Corrected here Pa --> kPa
            if 'F/L^2' in self._userUnits[i+1]:
                userParams.append(['User'+str(i+1), float(self._userLines[i].text().replace(",", "."))*1000])
            else:
                userParams.append(['User'+str(i+1), float(self._userLines[i].text().replace(",", "."))])
        
        
        kx = self.lineKx.text().replace(",", ".")
        ky = self.lineKy.text().replace(",", ".")
        if ky == '' and kx == '':
            ky = 0.0
            kx = 0.0
            if choice == 1:
                text = 'Note: kx & ky was set to zero.'
                self.infoWindow_2 = PopupWindow(text)
                self.infoWindow_2.show()
        elif ky == '':
            kx = float(kx)
            ky = kx
        elif not(ky==''):
            kx = float(kx)
            ky = float(ky)
        else:
            text = 'Error: Could not define kx & ky.'
            self.infoWindow_3 = PopupWindow(text)
            self.infoWindow_3.show()
        
        if choice == 1:
            try:
                k0x = float(self.lineK0x.text().replace(",", "."))
            except:
                text = 'Error: k0x must be defined.'
                self.infoWindow = PopupWindow(text)
                self.infoWindow.show()
                
            k0x = float(self.lineK0x.text().replace(",", "."))
            k0y = self.lineK0y.text().replace(",", ".")
            if k0y == '':
                k0y = k0x
            else:
                k0y = float(k0y)
        else:
            k0x = 1.0
            k0y = 1.0
        
        
               
        if choice == 1:
            mainWindow._avg_param = [
            #General
            ('MaterialName', IdName),
            ('Comments', Comment),
            ('SoilModel', 100), # Userdefined
            ('DrainageType', Drain),
            ('gammaUnsat', Weight),
            ('gammaSat', Weight),
            #DLL
            ('UserDLLName', File),
            ('UserModel', Model),                        
            #Flow
            ('perm_primary_horizontal_axis', kx),
            ('perm_vertical_axis', ky),
            #Interface
            ("InterfaceStrength", 1),
            ("Gref", 1),
            ("cref", 1),            
            #K0- Determination
            ("K0PrimaryIsK0Secondary", False),
            ("K0Determination", 0),
            ("K0Primary", k0x),
            ("K0Secondary", k0y),
            ]
            
            mainWindow._load_param = [
            ('MaterialName', 'Stress generation material'),
            ('SoilModel', 1), # Linear elastic
            ('DrainageType', 0),
            ('gammaUnsat', Weight),
            ('gammaSat', Weight),
            ('Gref', 43000),
            ('nu', k0x/(1+k0x)),
            #Flow
            ('perm_primary_horizontal_axis', 1.0E12),
            ('perm_vertical_axis', 1.0E12)]
            #Add user defined parameters
            mainWindow._avg_param = mainWindow._avg_param + userParams
            mainWindow._load_param = mainWindow._load_param
        else:
            mainWindow._cyc_param = [
            ('MaterialName', IdName),
            ('Comments', Comment),
            ('SoilModel', 100), # Userdefined
            ('DrainageType', Drain),
            ('gammaUnsat', Weight),
            ('gammaSat', Weight),
            #DLL
            ('UserDLLName', File),
            ('UserModel', Model),                        
            #Flow
            ('perm_primary_horizontal_axis', kx),
            ('perm_vertical_axis', ky),
            #Interface
            ("InterfaceStrength", 1),
            ("Gref", 1),
            ("cref", 1),    
            #K0- Determination
            ("K0PrimaryIsK0Secondary", False),
            ("K0Determination", 0),
            ("K0Primary", k0x),
            ("K0Secondary", k0y),
            ]
            mainWindow._cyc_param = mainWindow._cyc_param + userParams
        self.exit()
        
    def exit(self):
        self.close()
        
class NumericalWindow(QNumWindow, Ui_NumWindow):
    def __init__(self, mainWindow):
        super(NumericalWindow, self).__init__()
        
        self.setupUi(self)
        
        self.btnBox.accepted.connect(lambda: self.saveInput(mainWindow))
        self.btnBox.rejected.connect(self.exit)
        
        # TODO: Not implemented yet
        # self.spinBox.setEnabled(False)
        self.lineMesh.setEnabled(False)
        self.checkDefaultParam.setEnabled(False)
        self.lineCores.setEnabled(False)
        self.lineStepsStored.setEnabled(False)
        self.lineMaxSteps.setEnabled(False)
        self.lineError.setEnabled(False)
        self.lineUnloading.setEnabled(False)
        self.lineLoadFraction.setEnabled(False)
        self.lineRelaxation.setEnabled(False)
        self.lineMaxIter.setEnabled(False)
        self.lineDesMinIter.setEnabled(False)
        self.lineDesMaxIter.setEnabled(False)
        self.comboBoxArcLength.setEnabled(False)
        self.checkLineSearch.setEnabled(False)
        self.checkGradualError.setEnabled(False)
        
        
    def exit(self):
        self.close()
        
    def saveInput(self, mainWindow):
        intermediateIter = self.spinBox.value()
        numericParam = {'intermediateIter':intermediateIter}
        mainWindow._numericParam = numericParam
        self.exit()
        
            
if __name__ == "__main__":
    import sys
    from PyQt4 import QtGui
    import numpy as np
    
    app = QtGui.QApplication(sys.argv)
    main = Degrader()
    main.show()
    sys.exit(app.exec_())