from simple_project import SimpleProject, run

try:
    from matplotlib import pyplot
except ImportError:
    raise Exception("matplotlib must be installed to use this script.")
    
SAMPLE_COUNT = 16

class CrossSectionProject(SimpleProject):
    def add_soil_layers(self):
        self._g_input.borehole(0)
        self._g_input.soillayer(4)
        
    def add_structures(self):
        self._g_input.gotostructures()
        self._g_input.lineload(5, 0, 7, 0)
        
    def configure_phases(self):
        self._g_input.gotostages()
        self._g_input.activate(self._g_input.Plates, self._g_input.InitialPhase)
        
        next_phase = self._g_input.phase(self._g_input.InitialPhase)
        self._g_input.activate(self._g_input.LineLoads, next_phase)
        
    def gather_results(self):
        start = (0.0, -4.0)
        end = (6.0, 0.0)
        
        output_port = self._g_input.view(self._g_input.Phases[-1])
        s_output, g_output = self._new_server('localhost', output_port)
        
        # Assumes that start and end contain floats, will not work correctly with integers in Python 2
        step = [(e - s)/(SAMPLE_COUNT - 1) for e,s in zip(end, start)]
        
        self._results = []
        for i in range(SAMPLE_COUNT):
            position = (start[0] + i*step[0], start[1] + i*step[1])
            result_string = g_output.getsingleresult(g_output.Phases[-1], g_output.ResultTypes.Soil.Utot, position)
            
            # Check if position lies outside the mesh, it might make more sense to use 0 as the result for other result types.
            if result_string == "not found":
                raise Exception("Used getsingleresult for point outside mesh.")
                
            self._results.append(float(result_string))
            
        g_output.update()
        
    def output_results(self):
        pyplot.plot(range(len(self._results)), self._results, 'r')
        pyplot.grid(True)
        pyplot.show()
        
if __name__ == '__main__':
    run(CrossSectionProject)