from simple_project import SimpleProject, run

try:
    from PIL import Image, ImageFont, ImageDraw
except ImportError:
    raise Exception("Pillow must be installed to use this script.")
    
class ExportPlotProject(SimpleProject):
    def add_soil_layers(self):
        self._g_input.borehole(0)
        self._g_input.soillayer(4)
        
    def add_structures(self):
        self._g_input.gotostructures()
        self._g_input.plate(6, 0, 6, -2)
        
        self._g_input.pointload(self._g_input.Points[0])
        self._g_input.PointLoads[0].Fy = -10
        
    def configure_phases(self):
        self._g_input.gotostages()
        self._g_input.activate(self._g_input.Plates, self._g_input.InitialPhase)
        
        next_phase = self._g_input.phase(self._g_input.InitialPhase)
        self._g_input.activate(self._g_input.PointLoads, next_phase)
        
    def gather_results(self):
        width, height = 800, 600
        
        output_port = self._g_input.view(self._g_input.InitialPhase)
        s_output, g_output = self._new_server('localhost', output_port)
        
        # Hide everything except the mesh itself.
        g_output.Plots[0].DrawFrame = False
        g_output.Plots[0].DrawTitle = False
        g_output.Plots[0].DrawLegend = False
        g_output.Plots[0].DrawAxes = False
        
        g_output.Plots[0].Phase = g_output.InitialPhase
        self._initial_phase_image = g_output.Plots[0].export(width, height)
        
        g_output.Plots[0].Phase = g_output.Phase_1
        self._phase_1_image = g_output.Plots[0].export(width, height)
        
        g_output.close()
        
    def output_results(self):
        # Get Pillow.Image objects.
        inner_initial_image = self._initial_phase_image.image
        inner_phase_1_image = self._phase_1_image.image
        
        assert inner_initial_image.size == inner_phase_1_image.size
        
        cropping_left = (0, 0, inner_initial_image.width // 2, inner_initial_image.height)
        left_image = inner_initial_image.crop(cropping_left)
        
        cropping_right = (inner_phase_1_image.width // 2, 0, inner_phase_1_image.width, inner_phase_1_image.height)
        right_image = inner_phase_1_image.crop(cropping_right)
        
        result_image = Image.new(inner_initial_image.mode, inner_initial_image.size)
        result_image.paste(left_image, cropping_left)
        result_image.paste(right_image, cropping_right)
        
        default_font = ImageFont.load_default()
        drawing_context = ImageDraw.Draw(result_image)
        
        initial_phase_ID = self._g_input.Phases[0].Identification.value
        drawing_context.text((200,160), initial_phase_ID, font=default_font, fill=(0,0,0,255))
        
        phase_1_ID = self._g_input.Phases[1].Identification.value
        after_text_size = default_font.getsize(phase_1_ID)
        drawing_context.text((600 - after_text_size[0], 160), phase_1_ID, font=default_font, fill=(0, 0, 0, 255))
        
        result_image.save("plot_initial_vs_phase_1.png")
        
        
if __name__ == '__main__':
    run(ExportPlotProject)