# # # -*- coding: utf-8 -*-
# # 
# # ### Boilerplate code ###
# # localhostport_input = 10000
# # #localhostport_output= 10001
# # 
# # # Libraries
# # from PyQt4 import QtCore, QtGui
# # import numpy as np
# # import os, imp, subprocess
# # 
# # # Location of plaxis.exe folder
# # plaxis_path = r'C:\Program Files (x86)\Plaxis\PLAXIS 2D' #no trailing backslash!
# # # 
# # found_module = imp.find_module('plxscripting', [plaxis_path])
# # plxscripting = imp.load_module('plxscripting', *found_module)
# # from plxscripting.easy import *
# # 
# # # Input
# # # s - Store remote scripting server (PLAXIS Project Interface) as an object
# # # g - Global object (actual model)
# # s_i, g_i = new_server('localhost', localhostport_input)
# # #s_o, g_o = new_server('localhost', localhostport_output)
# # 
# # # Start Program
# # subprocess.Popen(['C:\\Program Files (x86)\\Plaxis\\PLAXIS 2D\\Plaxis2dXInput.exe', "--AppServerPort="+str(localhostport_input)])

import os
import imp
import subprocess

INPUT_SERVER_PORT = 10000

class SimpleProject(object):
    """
    Class that provides a way to quickly setup a project for example purposes.
    """
    
    def __init__(self, g_input):
        # Import module from plaxis_path. This enables us to store this script anywhere we want to
        #plaxis_path = r"C:\Program Files (x86)\Plaxis\PLAXIS 2D"
        found_module = imp.find_module('plxscripting', [g_input])
        plxscripting = imp.load_module('plxscripting', *found_module)
        
        from plxscripting.easy import new_server
        self._new_server = new_server
        
        args = [os.path.join(g_input, "Plaxis2DXInput.exe"), "--appServerPort={}".format(INPUT_SERVER_PORT)]
        self._input_process = subprocess.Popen(args)
        
        self._s_input, self._g_input = self._new_server('localhost', 10000, timeout=10.0)
        
        self._s_input.new()
    
    def gather_results(self):
        raise NotImplementedError("Override gather_results in subclass.")
        
    def output_results(self):
        raise NotImplementedError("Override output_results in subclass.")
        
    def close_input(self):
        self._input_process.kill()
        
    @property
    def g_input(self):
        return self._g_input
        
    def add_soil_layers(self):
        raise NotImplementedError("Override add_soil_layers in subclass.")
        
    def apply_soil_material(self):
        SAND_PARAMETERS = [
        ('MaterialName', 'Sand'),
        ('Colour', 10676870),
        ('SoilModel', 3), # HS
        ('DrainageType', 'Drained'),
        ('gammaUnsat', 17),
        ('gammaSat', 20),
        ('E50ref', 43000),
        ('EoedRef', 28000),
        ('EurRef', 129000),
        ('powerm', 0.5),
        ('cref', 1),
        ('phi', 34.0),
        ('psi', 4.0),
        ('nu', 0.2),
        ('Rinter', 0.7),
        ('K0NC', 0.5),
        ('OCR', 1.0),
        ('POP', 0.0)
        ]

        sand = self._g_input.soilmat(*SAND_PARAMETERS)
    
        for soil_layer in self._g_input.SoilLayers:
            self._g_input.setmaterial(soil_layer, sand)
        
    def add_structures(self):
        pass # Not adding any plates is fine too.
        
    def apply_plate_material(self):
        DIAPHRAGM_WALL_PARAMETERS = [
        ('MaterialName', 'Wall'),
        ('Colour', 16711680),
        ('Elasticity', 0),
        ('IsIsotropic', True),
        ('IsEndBearing', True),
        ('EA', 12000000),
        ('EI', 120000),
        ('nu', 0.15),
        ('d', 0.34641),
        ('w', 8.3),
        ('Mp', 1000000000000000.0),
        ('Np', 10000000000.0),
        ('Np2', 10000000000.0),
        ('RayleighAlpha', 0),
        ('RayleighBeta', 0),
        ('Gref', 15061311)
        ]
        diaphragm_wall_material = self._g_input.platemat(*DIAPHRAGM_WALL_PARAMETERS)
        
        for plate in self._g_input.Plates:
            self._g_input.setmaterial(plate, diaphragm_wall_material)
            
    def mesh(self):
        self._g_input.gotomesh()
        self._g_input.mesh(0.06)
        
    def select_curve_points(self):
        pass # Not selecting any curve-points is fine too.
        
    def configure_phases(selef):
        raise NotImplementedError("Override configure_phases in subclass.")
        
    def make_project(self):
        self.add_soil_layers()
        self.apply_soil_material()
        self.add_structures()
        self.apply_plate_material()
        self.mesh()
        self.select_curve_points()
        self.configure_phases()
        self._g_input.calculate()
        
def run(project_class):
    # Replace with the path to your PLAXIS installation.
    project = project_class(r"C:\Program Files (x86)\Plaxis\PLAXIS 2D")
    project.make_project()
    project.gather_results()
    project.output_results()
    project.close_input()
    