import sys, os, time

from blankSlate import BlankSlate, run 

try:
    from PIL import Image, ImageFont, ImageDraw
except ImportError:
    raise Exception("Pillow must be installed to use this script.")

class BearingCap(BlankSlate):
    def add_soil_layers(self):
        
        self.model.SoilContour.initializerectangular(-6, -8, 6, 0)
        
        borehole = self.model.borehole(0)
        self.model.soillayer(8)
        borehole.Head = -9
        
        
    def add_structures(self):
        #self.model.gotostructures()
        self.model.plate(-1, 0, 1, 0)
        self.model.line(-1,0,-1,-0.5)
        self.model.line(+1,-0.5,+1,0)
        
        # for line in self.model.Lines:
        #     self.model.neginterface(line)
        #     self.model.posinterface(line)
        
        self.model.lineload(self.model.Lines[0])
        
        
    def select_curve_points(self):
        output_port = self.model.selectmeshpoints()
        s_output, g_output = self._new_server('localhost', output_port)
        g_output.addcurvepoint('node', 0, -0.25)
        g_output.update()
        
        
    def configure_phases(self):
        self.model.gotostages()
        
        loadPhase = self.model.phase(self.model.Phases[0])
        self.model.Phases[1].Identification = "Load"
        self.model.Phases[1].TimeInterval = 1
        # self.model.Phases[1].Deform.IgnoreUndrainedBehaviour = True
        
        self.model.activate(self.model.Lines, loadPhase)
        line_load = self.model.LineLoads[0]
        line_load.qy_start[loadPhase] = -1000.0
        
        # Numerical control parameters
        self.model.Phases[1].Deform.UseDefaultIterationParams = False
        self.model.Phases[1].Deform.MaxUnloadingSteps = 10
 
        
    def gather_results(self):
        time.sleep(1)
        width, height = 800, 600
        
        output_port = self.model.view(self.model.Phase_1)
        s_output, g_output = self._new_server('localhost', output_port)
        
        # Hide everything except the mesh itself.
        switch = True
        
        g_output.Plots[0].DrawFrame = False
        g_output.Plots[0].DrawTitle = switch
        g_output.Plots[0].DrawLegend = switch
        g_output.Plots[0].DrawAxes = switch
  
        g_output.Plots[0].Phase = g_output.Phase_1
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.Uy)
        uy = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.Utot)
        #g_output.set(g_output.Plots[0].PlotType,'nodearrows')
        Utot = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.IncrementalDeviatoricStrain)
        incrDevStrain = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.PExcess)
        pexcess = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.MeanEffStress)
        MeanEffStress = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.DeviatoricStress)
        DeviatoricStress = g_output.Plots[0].export(width, height)
        
        self._image = [uy, Utot, incrDevStrain, pexcess, MeanEffStress, DeviatoricStress]
        
        g_output.update()
        
        
    def output_results(self):
        textTitles = ['uy', 'Utot', 'incrDevStrain', 'pexcess', 'MeanEffStress', 'DeviatoricStress']
        for number, img in enumerate(self._image):
            figure = img.image
        
            result_image = Image.new(figure.mode, figure.size)
            result_image.paste(figure)
        
            default_font = ImageFont.load_default()
            drawing_context = ImageDraw.Draw(result_image)
            # text = textTitles[number]
            # drawing_context.text((figure.size[0]/2,figure.size[1]-50), text, font=default_font, fill=(0,0,0,255))
            
            if not os.path.exists('./figures'):
                os.makedirs('./figures')
            
            result_image.save("./figures\\"+ textTitles[number]+".png")
        
                
if __name__ == '__main__':
    run(BearingCap)