import sys, os

from blankSlate import BlankSlate, run 

try:
    from PIL import Image, ImageFont, ImageDraw
except ImportError:
    raise Exception("Pillow must be installed to use this script.")

class PassiveFailure(BlankSlate):
    def add_soil_layers(self):
        
        self.model.SoilContour.initializerectangular(0, 0, 12, 6)
        
        borehole = self.model.borehole(12)
        self.model.soillayer(6)
        borehole.Head = -6
        
        
    def add_structures(self):
        #self.model.gotostructures()
        self.model.line(2, 0, 2, -3)
        self.model.line(0, -3, 2, -3)
        
        self.model.linedispl(self.model.Lines[0])
        
        
    
    def configure_phases(self):
        self.model.gotostages()
        
        initial_phase = self.model.Phases[0]
        
        self.model.activate(self.model.Lines[0], initial_phase)
        self.model.deactivate(self.model.SoilPolygons[0], initial_phase)
        
        displacement = self.model.LineDisplacements[0]
        displacement.Displacement_x[initial_phase] = "fixed"
        displacement.Displacement_y[initial_phase] = "fixed"
        
        loadPhase = self.model.phase(initial_phase)
        self.model.Phases[1].Identification = "Displacement"
        
        displacement.Displacement_x[self.model.Phases[0]] = "prescribed"
        displacement.ux_start[self.model.phases[1]] = 10
        
        # # Numerical control parameters
        # self.model.Phases[1].Deform.UseDefaultIterationParams = False
        # self.model.Phases[1].Deform.MaxUnloadingSteps = 10
 
        
    def gather_results(self):
        width, height = 800, 600
        
        output_port = self.model.view(self.model.Phase_1)
        s_output, g_output = self._new_server('localhost', output_port)
        
        # Hide everything except the mesh itself.
        switch = False
        
        g_output.Plots[0].DrawFrame = switch
        g_output.Plots[0].DrawTitle = switch
        g_output.Plots[0].DrawLegend = switch
        g_output.Plots[0].DrawAxes = switch
  
        g_output.Plots[0].Phase = g_output.Phase_1
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.Ux)
        ux = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.Utot)
        #g_output.set(g_output.Plots[0].PlotType,'nodearrows')
        Utot = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.IncrementalDeviatoricStrain)
        incrDevStrain = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.PExcess)
        pexcess = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.MeanEffStress)
        MeanEffStress = g_output.Plots[0].export(width, height)
        
        g_output.set(g_output.Plots[0].ResultType, g_output.soil.DeviatoricStress)
        DeviatoricStress = g_output.Plots[0].export(width, height)
        
        self._image = [ux, Utot, incrDevStrain, pexcess, MeanEffStress, DeviatoricStress]
        
        g_output.update()
        
        
    def output_results(self):
        textTitles = ['Horizontal deformation', 'Utot', 'Incremental Deviatoric Strain', 'Pexcess', 'Mean Eff. Stress', 'Deviatoric Stress']
        for number, img in enumerate(self._image):
            figure = img.image
        
            result_image = Image.new(figure.mode, figure.size)
            result_image.paste(figure)
        
            default_font = ImageFont.load_default()
            drawing_context = ImageDraw.Draw(result_image)
            text = textTitles[number]
            drawing_context.text((figure.size[0]/2,figure.size[1]-50), text, font=default_font, fill=(0,0,0,255))
            
            if not os.path.exists('./figures'):
                os.makedirs('./figures')
            
            result_image.save("./figures\\"+ textTitles[number]+".png")
        
                
if __name__ == '__main__':
    run(PassiveFailure)